@@ -51,7 +51,7 @@
 #include <emmintrin.h>
 #endif
 
-CPL_CVSID("$Id: gdalwarper.cpp 8c3e4ef55212f20eec95aa7e12ba5d48dacfdc47 2020-10-01 21:20:51 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                         GDALReprojectImage()                         */
@@ -1112,12 +1112,50 @@ GDALWarpDstAlphaMasker( void *pMaskFuncArg, int nBandCount,
  * situations. Starting with GDAL 2.4, gdalwarp will automatically enable this
  * option when it is assumed to be safe to do so.</li>
  *
- * <li>UNIFIED_SRC_NODATA=YES/NO: By default nodata masking values considered
- * independently for each band.  However, sometimes it is desired to treat all
- * bands as nodata if and only if, all bands match the corresponding nodata
- * values. To get this behavior set this option to YES.
- * Note: UNIFIED_SRC_NODATA=YES is set by default, when called from gdalwarp /
- * GDALWarp()</li>
+ * <li>UNIFIED_SRC_NODATA=YES/NO/PARTIAL: This setting determines
+ * how to take into account nodata values when there are several input bands.
+ * <ul>
+ * <li>When YES, all bands are considered as nodata if and only if, all bands
+ *     match the corresponding nodata values.
+ *     Note: UNIFIED_SRC_NODATA=YES is set by default, when called from gdalwarp /
+ *     GDALWarp() with an explicit -srcnodata setting.
+ *
+ *     Example with nodata values at (1, 2, 3) and target alpha band requested.
+ *     <ul>
+ *     <li>input pixel = (1, 2, 3) ==> output pixel = (0, 0, 0, 0)</li>
+ *     <li>input pixel = (1, 2, 127) ==> output pixel = (1, 2, 127, 255)</li>
+ *     </ul>
+ * </li>
+ * <li>When NO, nodata masking values is considered independently for each band.
+ *     A potential target alpha band will always be valid if there are multiple
+ *     bands.
+ *
+ *     Example with nodata values at (1, 2, 3) and target alpha band requested.
+ *     <ul>
+ *     <li>input pixel = (1, 2, 3) ==> output pixel = (0, 0, 0, 255)</li>
+ *     <li>input pixel = (1, 2, 127) ==> output pixel = (0, 0, 127, 255)</li>
+ *     </ul>
+ *
+ *     Note: NO was the default behavior before GDAL 3.3.2
+ * </li>
+ * <li>When PARTIAL, or not specified at all (default behavior),
+ *     nodata masking values is considered independently for each band.
+ *     But, and this is the difference with NO, if for a given pixel, it
+ *     evaluates to the nodata value of each band, the target pixel is
+ *     considered as globally invalid, which impacts the value of a potential
+ *     target alpha band.
+ *
+ *     Note: PARTIAL is new to GDAL 3.3.2 and should not be used with
+ *     earlier versions. The default behavior of GDAL < 3.3.2 was NO.
+ *
+ *     Example with nodata values at (1, 2, 3) and target alpha band requested.
+ *     <ul>
+ *     <li>input pixel = (1, 2, 3) ==> output pixel = (0, 0, 0, 0)</li>
+ *     <li>input pixel = (1, 2, 127) ==> output pixel = (0, 0, 127, 255)</li>
+ *     </ul>
+ * </li>
+ * </ul>
+ * </li>
  *
  * <li>CUTLINE: This may contain the WKT geometry for a cutline.  It will
  * be converted into a geometry by GDALWarpOperation::Initialize() and assigned
@@ -1208,6 +1246,12 @@ GDALWarpDstAlphaMasker( void *pMaskFuncArg, int nBandCount,
  * window for a given request, and by default it is 1 to take care of rounding
  * error.  Setting this larger will increase the amount of data that needs to
  * be read, but can avoid missing source data.</li>
+ * <li>APPLY_VERTICAL_SHIFT=YES/NO: Force the use of vertical shift.
+ * This option is generally not necessary, except when using an explicit
+ * coordinate transformation (COORDINATE_OPERATION), and not specifying
+ * an explicit source and target SRS.</li>
+ * <li>MULT_FACTOR_VERTICAL_SHIFT: Multiplication factor for the vertical
+ * shift. Default 1.0</li>
  * </ul>
  */
 
@@ -1315,7 +1359,7 @@ namespace
     {
         if( nBandCount <= 0 ) { return; }
         if( *ppdNoDataReal != nullptr ) { return; }
-        
+
         *ppdNoDataReal = static_cast<double *>(
             CPLMalloc(sizeof(double) * nBandCount));
 
@@ -1336,9 +1380,9 @@ namespace
  *
  * @param psOptionsIn options to initialize.
  * @param dNoDataReal value to initialize to.
- * 
+ *
  */
-void CPL_STDCALL 
+void CPL_STDCALL
 GDALWarpInitDstNoDataReal( GDALWarpOptions * psOptionsIn, double dNoDataReal )
 {
     VALIDATE_POINTER0(psOptionsIn, "GDALWarpInitDstNoDataReal");
@@ -1356,9 +1400,9 @@ GDALWarpInitDstNoDataReal( GDALWarpOptions * psOptionsIn, double dNoDataReal )
  *
  * @param psOptionsIn options to initialize.
  * @param dNoDataReal value to initialize to.
- * 
+ *
  */
-void CPL_STDCALL 
+void CPL_STDCALL
 GDALWarpInitSrcNoDataReal( GDALWarpOptions * psOptionsIn, double dNoDataReal )
 {
     VALIDATE_POINTER0(psOptionsIn, "GDALWarpInitSrcNoDataReal");
@@ -1376,9 +1420,9 @@ GDALWarpInitSrcNoDataReal( GDALWarpOptions * psOptionsIn, double dNoDataReal )
  *
  * @param psOptionsIn options to initialize.
  * @param dNoDataReal value to initialize to.
- * 
+ *
  */
-void CPL_STDCALL 
+void CPL_STDCALL
 GDALWarpInitNoDataReal(GDALWarpOptions * psOptionsIn, double  dNoDataReal)
 {
     GDALWarpInitDstNoDataReal(psOptionsIn, dNoDataReal);
@@ -1394,9 +1438,9 @@ GDALWarpInitNoDataReal(GDALWarpOptions * psOptionsIn, double  dNoDataReal)
  *
  * @param psOptionsIn options to initialize.
  * @param dNoDataImag value to initialize to.
- * 
+ *
  */
-void CPL_STDCALL 
+void CPL_STDCALL
 GDALWarpInitDstNoDataImag( GDALWarpOptions * psOptionsIn, double dNoDataImag )
 {
     VALIDATE_POINTER0(psOptionsIn, "GDALWarpInitDstNoDataImag");
@@ -1413,9 +1457,9 @@ GDALWarpInitDstNoDataImag( GDALWarpOptions * psOptionsIn, double dNoDataImag )
  *
  * @param psOptionsIn options to initialize.
  * @param dNoDataImag value to initialize to.
- * 
+ *
  */
-void CPL_STDCALL 
+void CPL_STDCALL
 GDALWarpInitSrcNoDataImag( GDALWarpOptions * psOptionsIn, double dNoDataImag )
 {
     VALIDATE_POINTER0(psOptionsIn, "GDALWarpInitSrcNoDataImag");
@@ -1430,10 +1474,10 @@ GDALWarpInitSrcNoDataImag( GDALWarpOptions * psOptionsIn, double dNoDataImag )
 /**
  * \brief If the working data type is unknown, this method will determine
  *  a valid working data type to support the data in the src and dest
- *  data sets and any noData values. 
+ *  data sets and any noData values.
  *
  * @param psOptions options to initialize.
- * 
+ *
  */
 void CPL_STDCALL
 GDALWarpResolveWorkingDataType( GDALWarpOptions *psOptions )
@@ -1447,7 +1491,7 @@ GDALWarpResolveWorkingDataType( GDALWarpOptions *psOptions )
 /* -------------------------------------------------------------------- */
     if( psOptions->eWorkingDataType != GDT_Unknown ) { return; }
 
-    
+
     psOptions->eWorkingDataType = GDT_Byte;
 
     for( int iBand = 0; iBand < psOptions->nBandCount; iBand++ )
@@ -1456,7 +1500,7 @@ GDALWarpResolveWorkingDataType( GDALWarpOptions *psOptions )
         {
             GDALRasterBandH hDstBand = GDALGetRasterBand(
                 psOptions->hDstDS, psOptions->panDstBands[iBand] );
-            
+
             if( hDstBand != nullptr )
             {
                 psOptions->eWorkingDataType =
@@ -1479,38 +1523,52 @@ GDALWarpResolveWorkingDataType( GDALWarpOptions *psOptions )
 
         if( psOptions->padfSrcNoDataReal != nullptr )
         {
-            psOptions->eWorkingDataType = GDALDataTypeUnionWithValue( 
-                psOptions->eWorkingDataType, 
-                psOptions->padfSrcNoDataReal[iBand], 
+            psOptions->eWorkingDataType = GDALDataTypeUnionWithValue(
+                psOptions->eWorkingDataType,
+                psOptions->padfSrcNoDataReal[iBand],
                 false );
         }
 
         if( psOptions->padfSrcNoDataImag != nullptr &&
             psOptions->padfSrcNoDataImag[iBand] != 0.0 )
         {
-           psOptions->eWorkingDataType = GDALDataTypeUnionWithValue( 
-                psOptions->eWorkingDataType, 
-                psOptions->padfSrcNoDataImag[iBand], 
+           psOptions->eWorkingDataType = GDALDataTypeUnionWithValue(
+                psOptions->eWorkingDataType,
+                psOptions->padfSrcNoDataImag[iBand],
                 true );
         }
 
         if( psOptions->padfDstNoDataReal != nullptr )
         {
-            psOptions->eWorkingDataType = GDALDataTypeUnionWithValue( 
-                psOptions->eWorkingDataType, 
-                psOptions->padfDstNoDataReal[iBand], 
+            psOptions->eWorkingDataType = GDALDataTypeUnionWithValue(
+                psOptions->eWorkingDataType,
+                psOptions->padfDstNoDataReal[iBand],
                 false );
         }
 
         if( psOptions->padfDstNoDataImag != nullptr &&
             psOptions->padfDstNoDataImag[iBand] != 0.0 )
         {
-            psOptions->eWorkingDataType = GDALDataTypeUnionWithValue( 
-                psOptions->eWorkingDataType, 
-                psOptions->padfDstNoDataImag[iBand], 
+            psOptions->eWorkingDataType = GDALDataTypeUnionWithValue(
+                psOptions->eWorkingDataType,
+                psOptions->padfDstNoDataImag[iBand],
                 true );
         }
     }
+
+    const bool bApplyVerticalShift = CPLFetchBool(
+        psOptions->papszWarpOptions, "APPLY_VERTICAL_SHIFT", false);
+    if( bApplyVerticalShift && GDALDataTypeIsInteger(psOptions->eWorkingDataType) )
+    {
+        const double dfMultFactorVerticalShift = CPLAtof(
+            CSLFetchNameValueDef(psOptions->papszWarpOptions,
+                                 "MULT_FACTOR_VERTICAL_SHIFT", "1.0") );
+        if( dfMultFactorVerticalShift != 1 )
+        {
+            psOptions->eWorkingDataType =
+                GDALDataTypeUnion( psOptions->eWorkingDataType, GDT_Float32 );
+        }
+    }
 }
 
 /************************************************************************/
@@ -1524,13 +1582,13 @@ GDALWarpResolveWorkingDataType( GDALWarpOptions *psOptions )
  *
  * @param psOptionsIn options to initialize.
  * @param nBandCount bands to initialize for.
- * 
+ *
  */
 void CPL_STDCALL
 GDALWarpInitDefaultBandMapping( GDALWarpOptions * psOptionsIn, int nBandCount )
 {
     if( psOptionsIn->nBandCount != 0 ) { return; }
-    
+
     psOptionsIn->nBandCount = nBandCount;
 
     psOptionsIn->panSrcBands = static_cast<int *>(
@@ -1583,6 +1641,8 @@ GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
         pszAlgName = "Lanczos";
     else if( psWO->eResampleAlg == GRA_Average )
         pszAlgName = "Average";
+    else if( psWO->eResampleAlg == GRA_RMS )
+        pszAlgName = "RootMeanSquare";
     else if( psWO->eResampleAlg == GRA_Mode )
         pszAlgName = "Mode";
     else if( psWO->eResampleAlg == GRA_Max )
@@ -1623,7 +1683,8 @@ GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
         // EXTRA_ELTS is an internal detail that we will recover
         // no need to serialize it.
         // And CUTLINE is also serialized in a special way
-        if( !EQUAL(pszName, "EXTRA_ELTS") && !EQUAL(pszName, "CUTLINE") )
+        if( pszName != nullptr &&
+            !EQUAL(pszName, "EXTRA_ELTS") && !EQUAL(pszName, "CUTLINE") )
         {
             CPLXMLNode *psOption =
                 CPLCreateXMLElementAndValue(
@@ -1834,6 +1895,8 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
         psWO->eResampleAlg = GRA_Lanczos;
     else if( EQUAL(pszValue, "Average") )
         psWO->eResampleAlg = GRA_Average;
+    else if( EQUAL(pszValue, "RootMeanSquare") )
+        psWO->eResampleAlg = GRA_RMS;
     else if( EQUAL(pszValue, "Mode") )
         psWO->eResampleAlg = GRA_Mode;
     else if( EQUAL(pszValue, "Maximum") )
@@ -1853,7 +1916,7 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
     else
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "Unrecognise ResampleAlg value '%s'.",
+                  "Unrecognised ResampleAlg value '%s'.",
                   pszValue );
     }
 
@@ -1893,11 +1956,17 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
 
     if( pszValue != nullptr )
     {
+        CPLXMLNode* psGeoLocNode = CPLSearchXMLNode(psTree, "GeoLocTransformer");
+        if( psGeoLocNode )
+        {
+            CPLCreateXMLElementAndValue(psGeoLocNode, "SourceDataset", pszValue);
+        }
+
         CPLConfigOptionSetter oSetter("CPL_ALLOW_VSISTDIN", "NO", true);
 
         char** papszOpenOptions = GDALDeserializeOpenOptionsFromXML(psTree);
         psWO->hSrcDS = GDALOpenEx(
-            pszValue, GDAL_OF_SHARED | GDAL_OF_RASTER | GDAL_OF_VERBOSE_ERROR,
+            pszValue, GDAL_OF_RASTER | GDAL_OF_VERBOSE_ERROR,
             nullptr,
             papszOpenOptions, nullptr );
         CSLDestroy(papszOpenOptions);
@@ -1935,7 +2004,7 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
 /*      Now actually process each bandmapping.                          */
 /* ==================================================================== */
     int iBand = 0;
-    
+
     psBand = psBandTree ? psBandTree->psChild : nullptr;
 
     for( ; psBand != nullptr; psBand = psBand->psNext )
