@@ -36,7 +36,7 @@
 #include <map>
 #include <set>
 
-CPL_CVSID("$Id: gdalmdimtranslate_lib.cpp 35484f86aa4de17ff9caf11c3e9442ca30e4963f 2021-03-15 14:52:04 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                     GDALMultiDimTranslateOptions                     */
@@ -844,9 +844,10 @@ static bool TranslateArray(DimensionRemapper& oDimRemapper,
         viewExpr += ']';
         if( bHasModifiedDim )
         {
-            tmpArray = tmpArray->GetView(viewExpr, false, viewSpecs);
-            if( !tmpArray )
+            auto tmpArrayNew = tmpArray->GetView(viewExpr, false, viewSpecs);
+            if( !tmpArrayNew )
                 return false;
+            tmpArray = tmpArrayNew;
             size_t j = 0;
             const auto& tmpArrayDims(tmpArray->GetDimensions());
             for( size_t i = 0; i < srcArrayDims.size(); ++i )
@@ -1209,13 +1210,14 @@ static std::shared_ptr<GDALGroup> GetGroup(const std::shared_ptr<GDALGroup>& poR
         fullName.c_str(), "/", 0));
     for( int i = 0; i < aosTokens.size(); i++ )
     {
-        poCurGroup = poCurGroup->OpenGroup(aosTokens[i], nullptr);
-        if( !poCurGroup )
+        auto poCurGroupNew = poCurGroup->OpenGroup(aosTokens[i], nullptr);
+        if( !poCurGroupNew )
         {
             CPLError(CE_Failure, CPLE_AppDefined,
                     "Cannot find group %s", aosTokens[i]);
             return nullptr;
         }
+        poCurGroup = poCurGroupNew;
     }
     return poCurGroup;
 }
@@ -1497,6 +1499,12 @@ static bool TranslateInternal(std::shared_ptr<GDALGroup>& poDstRootGroup,
     }
     else
     {
+        if( poSrcRootGroup == nullptr )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                 "No multidimensional source dataset");
+            return false;
+        }
         return CopyGroup(oDimRemapper,
                          poDstRootGroup,
                          poDstRootGroup,
@@ -1863,6 +1871,13 @@ GDALMultiDimTranslateOptions *GDALMultiDimTranslateOptionsNew(
             psOptions->aosCreateOptions.AddString( papszArgv[i] );
         }
 
+        else if( EQUAL(papszArgv[i], "-oo") && i+1 < argc )
+        {
+            if( psOptionsForBinary )
+                psOptionsForBinary->papszOpenOptions = CSLAddString(
+                                                psOptionsForBinary->papszOpenOptions,
+                                                papszArgv[++i] );
+        }
         else if( papszArgv[i][0] == '-' )
         {
             CPLError(CE_Failure, CPLE_NotSupported,
