@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 1999, Frank Warmerdam
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -53,7 +53,7 @@
 #include "ogrsf_frmts.h"
 
 
-CPL_CVSID("$Id: ogrinfo.cpp b628c5c75561341c377d2356a2c3d24af1d7756b 2019-03-28 23:35:51 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 bool bVerbose = true;
 bool bSuperQuiet = false;
@@ -72,7 +72,8 @@ static void Usage( const char* pszErrorMsg = nullptr )
            "               [-sql statement|@filename] [-dialect sql_dialect] [-al] [-rl] [-so] [-fields={YES/NO}]\n"
            "               [-geom={YES/NO/SUMMARY}] [[-oo NAME=VALUE] ...]\n"
            "               [-nomd] [-listmdd] [-mdd domain|`all`]*\n"
-           "               [-nocount] [-noextent] [-wkt_format WKT1|WKT2|...]\n"
+           "               [-nocount] [-noextent] [-nogeomtype] [-wkt_format WKT1|WKT2|...]\n"
+           "               [-fielddomain name]\n"
            "               datasource_name [layer [layer ...]]\n");
 
     if( pszErrorMsg != nullptr )
@@ -162,7 +163,7 @@ static void GDALInfoReportMetadata( GDALMajorObjectH hObject,
 
             while( papszIter != nullptr && *papszIter != nullptr )
             {
-                if( !EQUAL(*papszIter, "") )
+                if( !EQUAL(*papszIter, "") && !EQUAL(*papszIter, "SUBDATASETS") )
                 {
                     papszExtraMDDomainsExpanded = CSLAddString(papszExtraMDDomainsExpanded, *papszIter);
                 }
@@ -189,6 +190,7 @@ static void GDALInfoReportMetadata( GDALMajorObjectH hObject,
 
         CSLDestroy(papszExtraMDDomainsExpanded);
     }
+    GDALInfoPrintMetadata(hObject, "SUBDATASETS", "Subdatasets", pszIndent);
 }
 
 /************************************************************************/
@@ -203,6 +205,7 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
                            char** papszExtraMDDomains,
                            bool bFeatureCount,
                            bool bExtent,
+                           bool bGeomType,
                            const char* pszWKTFormat )
 {
     OGRFeatureDefn      *poDefn = poLayer->GetLayerDefn();
@@ -253,7 +256,7 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
     if( bVerbose )
     {
         const int nGeomFieldCount =
-            poLayer->GetLayerDefn()->GetGeomFieldCount();
+            bGeomType ? poLayer->GetLayerDefn()->GetGeomFieldCount() : 0;
         if( nGeomFieldCount > 1 )
         {
             for(int iGeom = 0;iGeom < nGeomFieldCount; iGeom ++ )
@@ -264,7 +267,7 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
                        OGRGeometryTypeToName(poGFldDefn->GetType()));
             }
         }
-        else
+        else if( bGeomType )
         {
             printf("Geometry: %s\n",
                    OGRGeometryTypeToName(poLayer->GetGeomType()));
@@ -295,7 +298,20 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
                       oExt.MinX, oExt.MinY, oExt.MaxX, oExt.MaxY);
         }
 
-        const auto displayDataAxisMapping = [](const OGRSpatialReference* poSRS) {
+        const auto displayExtraInfoSRS = [](const OGRSpatialReference* poSRS) {
+
+            const double dfCoordinateEpoch = poSRS->GetCoordinateEpoch();
+            if( dfCoordinateEpoch > 0 )
+            {
+                std::string osCoordinateEpoch = CPLSPrintf("%f", dfCoordinateEpoch);
+                if( osCoordinateEpoch.find('.') != std::string::npos )
+                {
+                    while( osCoordinateEpoch.back() == '0' )
+                        osCoordinateEpoch.resize(osCoordinateEpoch.size()-1);
+                }
+                printf("Coordinate epoch: %s\n", osCoordinateEpoch.c_str());
+            }
+
             const auto mapping = poSRS->GetDataAxisToSRSAxisMapping();
             printf("Data axis to CRS axis mapping: ");
             for( size_t i = 0; i < mapping.size(); i++ )
@@ -335,7 +351,7 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
                 CPLFree(pszWKT);
                 if( poSRS )
                 {
-                    displayDataAxisMapping(poSRS);
+                    displayExtraInfoSRS(poSRS);
                 }
             }
         }
@@ -349,14 +365,18 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
             }
             else
             {
-                poSRS->exportToPrettyWkt(&pszWKT);
+                CPLString osWKTFormat("FORMAT=");
+                osWKTFormat += pszWKTFormat;
+                const char* const apszWKTOptions[] =
+                    { osWKTFormat.c_str(), "MULTILINE=YES", nullptr };
+                poSRS->exportToWkt(&pszWKT, apszWKTOptions);
             }
 
             printf("Layer SRS WKT:\n%s\n", pszWKT);
             CPLFree(pszWKT);
             if( poSRS )
             {
-                displayDataAxisMapping(poSRS);
+                displayExtraInfoSRS(poSRS);
             }
         }
 
@@ -386,18 +406,26 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
             const char* pszType = (poField->GetSubType() != OFSTNone)
                 ? CPLSPrintf(
                       "%s(%s)",
-                      poField->GetFieldTypeName(poField->GetType()),
-                      poField->GetFieldSubTypeName(poField->GetSubType()))
-                : poField->GetFieldTypeName(poField->GetType());
+                      OGRFieldDefn::GetFieldTypeName(poField->GetType()),
+                      OGRFieldDefn::GetFieldSubTypeName(poField->GetSubType()))
+                : OGRFieldDefn::GetFieldTypeName(poField->GetType());
             printf("%s: %s (%d.%d)",
                    poField->GetNameRef(),
                    pszType,
                    poField->GetWidth(),
                    poField->GetPrecision());
+            if( poField->IsUnique() )
+                printf(" UNIQUE");
             if( !poField->IsNullable() )
                 printf(" NOT NULL");
             if( poField->GetDefault() != nullptr )
                 printf(" DEFAULT %s", poField->GetDefault());
+            const char* pszAlias = poField->GetAlternativeNameRef();
+            if( pszAlias != nullptr && pszAlias[0])
+                printf(", alternative name=\"%s\"", pszAlias);
+            const std::string& osDomain = poField->GetDomainName();
+            if( !osDomain.empty() )
+                printf(", domain name=%s", osDomain.c_str());
             printf("\n");
         }
     }
@@ -408,12 +436,12 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
 
     if( nFetchFID == OGRNullFID && !bSummaryOnly )
     {
-        OGRFeature *poFeature = nullptr;
-        while( (poFeature = poLayer->GetNextFeature()) != nullptr )
+        if( !bSuperQuiet )
         {
-            if( !bSuperQuiet )
-                poFeature->DumpReadable(nullptr, papszOptions);
-            OGRFeature::DestroyFeature(poFeature);
+            for( auto& poFeature: poLayer )
+            {
+                    poFeature->DumpReadable(nullptr, papszOptions);
+            }
         }
     }
     else if( nFetchFID != OGRNullFID )
@@ -433,6 +461,133 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
     }
 }
 
+/************************************************************************/
+/*                        ReportFieldDomain()                           */
+/************************************************************************/
+
+static void ReportFieldDomain(const OGRFieldDomain* poDomain)
+{
+    printf("Domain %s:\n", poDomain->GetName().c_str());
+    const std::string& osDesc = poDomain->GetDescription();
+    if( !osDesc.empty() )
+    {
+        printf("  Description: %s\n", osDesc.c_str());
+    }
+    switch( poDomain->GetDomainType() )
+    {
+        case OFDT_CODED: printf("  Type: coded\n"); break;
+        case OFDT_RANGE: printf("  Type: range\n"); break;
+        case OFDT_GLOB:  printf("  Type: glob\n"); break;
+    }
+    const char* pszFieldType = (poDomain->GetFieldSubType() != OFSTNone)
+        ? CPLSPrintf(
+              "%s(%s)",
+              OGRFieldDefn::GetFieldTypeName(poDomain->GetFieldType()),
+              OGRFieldDefn::GetFieldSubTypeName(poDomain->GetFieldSubType()))
+        : OGRFieldDefn::GetFieldTypeName(poDomain->GetFieldType());
+    printf("  Field type: %s\n", pszFieldType);
+    switch( poDomain->GetSplitPolicy() )
+    {
+        case OFDSP_DEFAULT_VALUE:  printf("  Split policy: default value\n"); break;
+        case OFDSP_DUPLICATE:      printf("  Split policy: duplicate\n"); break;
+        case OFDSP_GEOMETRY_RATIO: printf("  Split policy: geometry ratio\n"); break;
+    }
+    switch( poDomain->GetMergePolicy() )
+    {
+        case OFDMP_DEFAULT_VALUE:     printf("  Merge policy: default value\n"); break;
+        case OFDMP_SUM:               printf("  Merge policy: sum\n"); break;
+        case OFDMP_GEOMETRY_WEIGHTED: printf("  Merge policy: geometry weighted\n"); break;
+    }
+    switch( poDomain->GetDomainType() )
+    {
+        case OFDT_CODED:
+        {
+            const auto poCodedFieldDomain =
+                cpl::down_cast<const OGRCodedFieldDomain*>(poDomain);
+            const OGRCodedValue* enumeration = poCodedFieldDomain->GetEnumeration();
+            printf("  Coded values:\n");
+            for( int i = 0; enumeration[i].pszCode != nullptr; ++i )
+            {
+                if( enumeration[i].pszValue )
+                {
+                    printf("    %s: %s\n",
+                           enumeration[i].pszCode,
+                           enumeration[i].pszValue);
+                }
+                else
+                {
+                    printf("    %s\n", enumeration[i].pszCode);
+                }
+            }
+            break;
+        }
+
+        case OFDT_RANGE:
+        {
+            const auto poRangeFieldDomain =
+                cpl::down_cast<const OGRRangeFieldDomain*>(poDomain);
+            bool bMinIsIncluded = false;
+            const OGRField& sMin = poRangeFieldDomain->GetMin(bMinIsIncluded);
+            bool bMaxIsIncluded = false;
+            const OGRField& sMax = poRangeFieldDomain->GetMax(bMaxIsIncluded);
+            if( poDomain->GetFieldType() == OFTInteger )
+            {
+                if( !OGR_RawField_IsUnset(&sMin) )
+                {
+                    printf("  Minimum value: %d%s\n",
+                           sMin.Integer,
+                           bMinIsIncluded ? "" : " (excluded)");
+                }
+                if( !OGR_RawField_IsUnset(&sMax) )
+                {
+                    printf("  Maximum value: %d%s\n",
+                           sMax.Integer,
+                           bMaxIsIncluded ? "" : " (excluded)");
+                }
+            }
+            else if( poDomain->GetFieldType() == OFTInteger64 )
+            {
+                if( !OGR_RawField_IsUnset(&sMin) )
+                {
+                    printf("  Minimum value: " CPL_FRMT_GIB "%s\n",
+                           sMin.Integer64,
+                           bMinIsIncluded ? "" : " (excluded)");
+                }
+                if( !OGR_RawField_IsUnset(&sMax) )
+                {
+                    printf("  Maximum value: " CPL_FRMT_GIB "%s\n",
+                           sMax.Integer64,
+                           bMaxIsIncluded ? "" : " (excluded)");
+                }
+            }
+            else if( poDomain->GetFieldType() == OFTReal )
+            {
+                if( !OGR_RawField_IsUnset(&sMin) )
+                {
+                    printf("  Minimum value: %g%s\n",
+                           sMin.Real,
+                           bMinIsIncluded ? "" : " (excluded)");
+                }
+                if( !OGR_RawField_IsUnset(&sMax) )
+                {
+                    printf("  Maximum value: %g%s\n",
+                           sMax.Real,
+                           bMaxIsIncluded ? "" : " (excluded)");
+                }
+            }
+            break;
+        }
+
+        case OFDT_GLOB:
+        {
+            const auto poGlobFieldDomain =
+                cpl::down_cast<const OGRGlobFieldDomain*>(poDomain);
+            printf("  Glob: %s\n", poGlobFieldDomain->GetGlob().c_str());
+            break;
+        }
+    }
+}
+
 /************************************************************************/
 /*                             RemoveBOM()                              */
 /************************************************************************/
@@ -453,17 +608,124 @@ static void RemoveSQLComments(char*& pszSQL)
     CPLString osSQL;
     for( char** papszIter = papszLines; papszIter && *papszIter; ++papszIter )
     {
-        if( !STARTS_WITH(*papszIter, "--") )
+        const char* pszLine = *papszIter;
+        char chQuote = 0;
+        int i = 0;
+        for(; pszLine[i] != '\0'; ++i )
         {
-            osSQL += *papszIter;
-            osSQL += " ";
+            if( chQuote )
+            {
+                if( pszLine[i] == chQuote )
+                {
+                    if( pszLine[i+1] == chQuote )
+                    {
+                        i++;
+                    }
+                    else
+                    {
+                        chQuote = 0;
+                    }
+                }
+            }
+            else if( pszLine[i] == '\'' || pszLine[i] == '"' )
+            {
+                chQuote = pszLine[i];
+            }
+            else if( pszLine[i] == '-' && pszLine[i+1] == '-' )
+            {
+                break;
+            }
         }
+        if( i > 0 )
+        {
+            osSQL.append(pszLine, i);
+        }
+        osSQL += ' ';
     }
     CSLDestroy(papszLines);
     CPLFree(pszSQL);
     pszSQL = CPLStrdup(osSQL);
 }
 
+/************************************************************************/
+/*                           PrintLayerSummary()                        */
+/************************************************************************/
+
+static void PrintLayerSummary(OGRLayer* poLayer, bool bGeomType, bool bIsPrivate=false)
+{
+    printf("%s", poLayer->GetName());
+
+    const char* pszTitle = poLayer->GetMetadataItem("TITLE");
+    if( pszTitle )
+    {
+        printf(" (title: %s)", pszTitle);
+    }
+
+    const int nGeomFieldCount =
+        bGeomType ? poLayer->GetLayerDefn()->GetGeomFieldCount() : 0;
+    if( nGeomFieldCount > 1 )
+    {
+        printf(" (");
+        for( int iGeom = 0; iGeom < nGeomFieldCount; iGeom++ )
+        {
+            if( iGeom > 0 )
+                printf(", ");
+            OGRGeomFieldDefn* poGFldDefn =
+                poLayer->GetLayerDefn()->
+                    GetGeomFieldDefn(iGeom);
+            printf(
+                "%s",
+                OGRGeometryTypeToName(
+                    poGFldDefn->GetType()));
+        }
+        printf(")");
+    }
+    else if( bGeomType && poLayer->GetGeomType() != wkbUnknown )
+        printf(" (%s)",
+               OGRGeometryTypeToName(
+                   poLayer->GetGeomType()));
+
+    if( bIsPrivate )
+    {
+        printf(" [private]");
+    }
+
+    printf("\n");
+}
+
+/************************************************************************/
+/*                       ReportHiearchicalLayers()                      */
+/************************************************************************/
+
+static void ReportHiearchicalLayers(const GDALGroup* group,
+                                    const std::string& indent,
+                                    bool bGeomType)
+{
+    const auto aosVectorLayerNames = group->GetVectorLayerNames();
+    for( const auto& osVectorLayerName: aosVectorLayerNames )
+    {
+        OGRLayer* poLayer = group->OpenVectorLayer(osVectorLayerName);
+        if( poLayer )
+        {
+            printf("%sLayer: ", indent.c_str());
+            PrintLayerSummary(poLayer, bGeomType);
+        }
+    }
+
+    const std::string subIndent(indent + "  ");
+    auto aosSubGroupNames = group->GetGroupNames();
+    for( const auto& osSubGroupName: aosSubGroupNames )
+    {
+        auto poSubGroup = group->OpenGroup(osSubGroupName);
+        if( poSubGroup )
+        {
+            printf("Group %s", indent.c_str());
+            printf("%s:\n", osSubGroupName.c_str());
+            ReportHiearchicalLayers(poSubGroup.get(), subIndent, bGeomType);
+        }
+    }
+}
+
 /************************************************************************/
 /*                                main()                                */
 /************************************************************************/
@@ -507,10 +769,12 @@ MAIN_START(nArgc, papszArgv)
     bool bShowMetadata = true;
     bool bFeatureCount = true;
     bool bExtent = true;
+    bool bGeomType = true;
     bool bDatasetGetNextFeature = false;
     bool bReadOnly = false;
     bool bUpdate = false;
     const char* pszWKTFormat = "WKT2";
+    std::string osFieldDomain;
 
     for( int iArg = 1; iArg < nArgc; iArg++ )
     {
@@ -680,6 +944,10 @@ MAIN_START(nArgc, papszArgv)
         {
             bExtent = false;
         }
+        else if( EQUAL(papszArgv[iArg], "-nogeomtype") )
+        {
+            bGeomType = false;
+        }
         else if( EQUAL(papszArgv[iArg], "-rl"))
         {
             bDatasetGetNextFeature = true;
@@ -689,6 +957,11 @@ MAIN_START(nArgc, papszArgv)
             CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
             pszWKTFormat = papszArgv[++iArg];
         }
+        else if( EQUAL(papszArgv[iArg], "-fielddomain") )
+        {
+            CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
+            osFieldDomain = papszArgv[++iArg];
+        }
 
         else if( papszArgv[iArg][0] == '-' )
         {
@@ -761,6 +1034,7 @@ MAIN_START(nArgc, papszArgv)
     GDALDriver *poDriver = nullptr;
     if( poDS != nullptr )
         poDriver = poDS->GetDriver();
+    const int nLayerCount = poDS ? poDS->GetLayerCount() : 0;
 
 /* -------------------------------------------------------------------- */
 /*      Report failure                                                  */
@@ -806,6 +1080,19 @@ MAIN_START(nArgc, papszArgv)
                            bShowMetadata,
                            papszExtraMDDomains);
 
+    if( !osFieldDomain.empty() )
+    {
+        auto poDomain = poDS->GetFieldDomain(osFieldDomain);
+        if( poDomain == nullptr )
+        {
+            printf("Domain %s cannot be found.\n", osFieldDomain.c_str());
+            exit(1);
+        }
+        printf("\n");
+        ReportFieldDomain(poDomain);
+        printf("\n");
+    }
+
     if( bDatasetGetNextFeature )
     {
         nRepeatCount = 0;  // skip layer reporting.
@@ -880,6 +1167,7 @@ MAIN_START(nArgc, papszArgv)
                                   papszExtraMDDomains,
                                   bFeatureCount,
                                   bExtent,
+                                  bGeomType,
                                   pszWKTFormat);
                     bSummaryOnly = bSummaryOnlyBackup;
                 }
@@ -922,11 +1210,11 @@ MAIN_START(nArgc, papszArgv)
                 ReportOnLayer(poResultSet, nullptr,
                               pszGeomField, poSpatialFilter,
                               bListMDD, bShowMetadata, papszExtraMDDomains,
-                              bFeatureCount, bExtent, pszWKTFormat);
+                              bFeatureCount, bExtent, bGeomType, pszWKTFormat);
             else
                 ReportOnLayer(poResultSet, nullptr, nullptr, nullptr,
                               bListMDD, bShowMetadata, papszExtraMDDomains,
-                              bFeatureCount, bExtent, pszWKTFormat);
+                              bFeatureCount, bExtent, bGeomType, pszWKTFormat);
             poDS->ReleaseResultSet(poResultSet);
         }
     }
@@ -938,12 +1226,23 @@ MAIN_START(nArgc, papszArgv)
         {
             if( iRepeat == 0 )
                 CPLDebug("OGR", "GetLayerCount() = %d\n",
-                         poDS->GetLayerCount());
+                         nLayerCount);
+
+            bool bDone = false;
+            auto poRootGroup = poDS->GetRootGroup();
+            if( !bAllLayers && poRootGroup &&
+                (!poRootGroup->GetGroupNames().empty() ||
+                 !poRootGroup->GetVectorLayerNames().empty()) )
+            {
+                ReportHiearchicalLayers(poRootGroup.get(),
+                                        std::string(), bGeomType);
+                bDone = true;
+            }
 
 /* -------------------------------------------------------------------- */
 /*      Process each data source layer.                                 */
 /* -------------------------------------------------------------------- */
-            for( int iLayer = 0; iLayer < poDS->GetLayerCount(); iLayer++ )
+            for( int iLayer = 0; !bDone && iLayer < nLayerCount; iLayer++ )
             {
                 OGRLayer *poLayer = poDS->GetLayer(iLayer);
 
@@ -956,39 +1255,8 @@ MAIN_START(nArgc, papszArgv)
 
                 if( !bAllLayers )
                 {
-                    printf("%d: %s", iLayer + 1, poLayer->GetName());
-
-                    const char* pszTitle = poLayer->GetMetadataItem("TITLE");
-                    if( pszTitle )
-                    {
-                        printf(" (title: %s)", pszTitle);
-                    }
-
-                    const int nGeomFieldCount =
-                        poLayer->GetLayerDefn()->GetGeomFieldCount();
-                    if( nGeomFieldCount > 1 )
-                    {
-                        printf(" (");
-                        for( int iGeom = 0; iGeom < nGeomFieldCount; iGeom++ )
-                        {
-                            if( iGeom > 0 )
-                                printf(", ");
-                            OGRGeomFieldDefn* poGFldDefn =
-                                poLayer->GetLayerDefn()->
-                                    GetGeomFieldDefn(iGeom);
-                            printf(
-                                "%s",
-                                OGRGeometryTypeToName(
-                                    poGFldDefn->GetType()));
-                        }
-                        printf(")");
-                    }
-                    else if( poLayer->GetGeomType() != wkbUnknown )
-                        printf(" (%s)",
-                               OGRGeometryTypeToName(
-                                   poLayer->GetGeomType()));
-
-                    printf("\n");
+                    printf("%d: ", iLayer + 1);
+                    PrintLayerSummary(poLayer, bGeomType, poDS->IsLayerPrivate(iLayer));
                 }
                 else
                 {
@@ -998,7 +1266,7 @@ MAIN_START(nArgc, papszArgv)
                     ReportOnLayer(poLayer, pszWHERE,
                                   pszGeomField, poSpatialFilter,
                                   bListMDD, bShowMetadata, papszExtraMDDomains,
-                                  bFeatureCount, bExtent, pszWKTFormat);
+                                  bFeatureCount, bExtent, bGeomType, pszWKTFormat);
                 }
             }
         }
@@ -1026,7 +1294,7 @@ MAIN_START(nArgc, papszArgv)
 
                 ReportOnLayer(poLayer, pszWHERE, pszGeomField, poSpatialFilter,
                               bListMDD, bShowMetadata, papszExtraMDDomains,
-                              bFeatureCount, bExtent, pszWKTFormat);
+                              bFeatureCount, bExtent, bGeomType, pszWKTFormat);
             }
         }
     }
@@ -1052,7 +1320,7 @@ end:
     CPLFree(pszSQLStatement);
     CPLFree(pszWHERE);
 
-    OGRCleanupAll();
+    GDALDestroy();
 
     return nRet;
 }
