@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gridlib.c 5c54326d96d23b2fee099383897b85c15bef9a6a 2017-12-30 09:14:05Z Even Rouault $
+ * $Id$
  *
  * Project:  Arc/Info Binary Grid Translator
  * Purpose:  Grid file reading code.
@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 1999, Frank Warmerdam
- * Copyright (c) 2007-2010, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2007-2010, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,7 @@
 
 #include "aigrid.h"
 
-CPL_CVSID("$Id: gridlib.c 5c54326d96d23b2fee099383897b85c15bef9a6a 2017-12-30 09:14:05Z Even Rouault $")
+CPL_CVSID("$Id$")
 
 CPL_INLINE static void CPL_IGNORE_RET_VAL_INT(CPL_UNUSED int unused) {}
 
@@ -116,7 +116,10 @@ static GInt32 AIGRolloverSignedAdd(GInt32 a, GInt32 b)
     // Not really portable as assumes complement to 2 representation
     // but AIG assumes typical unsigned rollover on signed
     // integer operations.
-    return (GInt32)((GUInt32)(a) + (GUInt32)(b));
+    GInt32 res;
+    GUInt32 resUnsigned = (GUInt32)(a) + (GUInt32)(b);
+    memcpy(&res, &resUnsigned, sizeof(res));
+    return res;
 }
 
 /************************************************************************/
@@ -415,7 +418,7 @@ CPLErr AIGProcessBlock( GByte *pabyCur, int nDataSize, int nMin, int nMagic,
                 return CE_Failure;
             }
 
-            nValue = (pabyCur[0] * 256 + pabyCur[1]) + nMin;
+            nValue = AIGRolloverSignedAdd(pabyCur[0] * 256 + pabyCur[1], nMin);
             pabyCur += 2;
             nDataSize -= 2;
 
@@ -444,7 +447,7 @@ CPLErr AIGProcessBlock( GByte *pabyCur, int nDataSize, int nMin, int nMagic,
                 return CE_Failure;
             }
 
-            nValue = *(pabyCur++) + nMin;
+            nValue = AIGRolloverSignedAdd(*(pabyCur++), nMin);
             nDataSize--;
 
             for( i = 0; i < nMarker; i++ )
