@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2000, Atlantis Scientific Inc.
- * Copyright (c) 2009-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2009-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,7 +34,7 @@
 #include "rawdataset.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: sar_ceosdataset.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 static GInt16 CastToGInt16(float val)
 {
@@ -74,6 +74,9 @@ static const char * const CeosExtension[][6] = {
 /* Radarsat-1 per #2051 */
 { "vol", "sarl", "sard", "sart", "nvol", "ext" },
 
+/* Radarsat-1 ASF */
+{ "", "L", "D", "", "", "ext" },
+
 /* end marker */
 { nullptr, nullptr, nullptr, nullptr, nullptr, nullptr }
 };
@@ -107,6 +110,11 @@ static CeosTypeCode_t QuadToTC( int a, int b, int c, int d )
 /* see CEOS-SAR-CCT Iss/Rev: 2/0 February 10, 1989 */
 #define LEADER_MAP_PROJ_RECORD_JERS_TC         QuadToTC( 18, 20, 18, 20 )
 
+/* Leader from ASF has different identifiers */
+#define LEADER_MAP_PROJ_RECORD_ASF_TC      QuadToTC( 10, 20, 18, 20 )
+#define LEADER_DATASET_SUMMARY_ASF_TC      QuadToTC( 10, 10, 18, 20 )
+#define LEADER_FACILITY_ASF_TC             QuadToTC( 90, 210, 18, 61 )
+
 /* For ERS calibration and incidence angle information */
 #define ERS_GENERAL_FACILITY_DATA_TC  QuadToTC( 10, 200, 31, 50 )
 #define ERS_GENERAL_FACILITY_DATA_ALT_TC QuadToTC( 10, 216, 31, 50 )
@@ -123,7 +131,7 @@ class SAR_CEOSRasterBand;
 class CCPRasterBand;
 class PALSARRasterBand;
 
-class SAR_CEOSDataset : public GDALPamDataset
+class SAR_CEOSDataset final: public GDALPamDataset
 {
     friend class SAR_CEOSRasterBand;
     friend class CCPRasterBand;
@@ -141,6 +149,7 @@ class SAR_CEOSDataset : public GDALPamDataset
     void        ScanForGCPs();
     void        ScanForMetadata();
     int         ScanForMapProjection();
+    char        **papszExtraFiles;
 
   public:
     SAR_CEOSDataset();
@@ -157,6 +166,7 @@ class SAR_CEOSDataset : public GDALPamDataset
     char **GetMetadata( const char * pszDomain ) override;
 
     static GDALDataset *Open( GDALOpenInfo * );
+    virtual char **GetFileList(void) override;
 };
 
 /************************************************************************/
@@ -165,7 +175,7 @@ class SAR_CEOSDataset : public GDALPamDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class CCPRasterBand : public GDALPamRasterBand
+class CCPRasterBand final: public GDALPamRasterBand
 {
     friend class SAR_CEOSDataset;
 
@@ -181,7 +191,7 @@ class CCPRasterBand : public GDALPamRasterBand
 /* ==================================================================== */
 /************************************************************************/
 
-class PALSARRasterBand : public GDALPamRasterBand
+class PALSARRasterBand final: public GDALPamRasterBand
 {
     friend class SAR_CEOSDataset;
 
@@ -197,7 +207,7 @@ class PALSARRasterBand : public GDALPamRasterBand
 /* ==================================================================== */
 /************************************************************************/
 
-class SAR_CEOSRasterBand : public GDALPamRasterBand
+class SAR_CEOSRasterBand final: public GDALPamRasterBand
 {
     friend class SAR_CEOSDataset;
 
@@ -631,7 +641,8 @@ SAR_CEOSDataset::SAR_CEOSDataset() :
     fpImage(nullptr),
     papszTempMD(nullptr),
     nGCPCount(0),
-    pasGCPList(nullptr)
+    pasGCPList(nullptr),
+    papszExtraFiles(nullptr)
 {
     sVolume.Flavor = 0;
     sVolume.Sensor = 0;
@@ -675,7 +686,7 @@ SAR_CEOSDataset::SAR_CEOSDataset() :
 SAR_CEOSDataset::~SAR_CEOSDataset()
 
 {
-    FlushCache();
+    FlushCache(true);
 
     CSLDestroy( papszTempMD );
 
@@ -685,8 +696,8 @@ SAR_CEOSDataset::~SAR_CEOSDataset()
     if( nGCPCount > 0 )
     {
         GDALDeinitGCPs( nGCPCount, pasGCPList );
-        CPLFree( pasGCPList );
     }
+    CPLFree( pasGCPList );
 
     if( sVolume.RecordList )
     {
@@ -703,6 +714,7 @@ SAR_CEOSDataset::~SAR_CEOSDataset()
         DestroyList( sVolume.RecordList );
     }
     FreeRecipes();
+    CSLDestroy( papszExtraFiles );
 }
 
 /************************************************************************/
@@ -955,6 +967,10 @@ void SAR_CEOSDataset::ScanForMetadata()
     record = FindCeosRecord( sVolume.RecordList, LEADER_DATASET_SUMMARY_TC,
                              CEOS_LEADER_FILE, -1, -1 );
 
+    if( record == nullptr )
+        record = FindCeosRecord( sVolume.RecordList, LEADER_DATASET_SUMMARY_ASF_TC,
+                                 CEOS_LEADER_FILE, -1, -1 );
+
     if( record == nullptr )
         record = FindCeosRecord( sVolume.RecordList, LEADER_DATASET_SUMMARY_TC,
                                  CEOS_TRAILER_FILE, -1, -1 );
@@ -1110,6 +1126,14 @@ void SAR_CEOSDataset::ScanForMetadata()
             SetMetadataItem( "CEOS_INC_ANGLE", szField );
 
 /* -------------------------------------------------------------------- */
+/*      Facility                                                         */
+/* -------------------------------------------------------------------- */
+        GetCeosField( record, 1047, "A16", szField );
+        szField[16] = '\0';
+
+        if( !STARTS_WITH_CI(szField, "                ") )
+            SetMetadataItem( "CEOS_FACILITY", szField );
+/* -------------------------------------------------------------------- */
 /*      Pixel time direction indicator                                  */
 /* -------------------------------------------------------------------- */
         GetCeosField( record, 1527, "A8", szField );
@@ -1482,12 +1506,26 @@ int SAR_CEOSDataset::ScanForMapProjection()
                                            LEADER_MAP_PROJ_RECORD_TC,
                                            CEOS_LEADER_FILE, -1, -1 );
 
+    int gcp_ordering_mode = CEOS_STD_MAPREC_GCP_ORDER;
     /* JERS from Japan */
     if( record == nullptr )
         record = FindCeosRecord( sVolume.RecordList,
                              LEADER_MAP_PROJ_RECORD_JERS_TC,
                              CEOS_LEADER_FILE, -1, -1 );
 
+    if( record == nullptr ) {
+        record = FindCeosRecord( sVolume.RecordList,
+                             LEADER_MAP_PROJ_RECORD_ASF_TC,
+                             CEOS_LEADER_FILE, -1, -1 );
+        gcp_ordering_mode = CEOS_ASF_MAPREC_GCP_ORDER;
+    }
+    if( record == nullptr ) {
+        record = FindCeosRecord( sVolume.RecordList,
+                             LEADER_FACILITY_ASF_TC,
+                             CEOS_LEADER_FILE, -1, -1 );
+        gcp_ordering_mode = CEOS_ASF_FACREC_GCP_ORDER;
+    }
+
     if( record == nullptr )
         return FALSE;
 
@@ -1495,11 +1533,27 @@ int SAR_CEOSDataset::ScanForMapProjection()
     memset( szField, 0, 17 );
     GetCeosField( record, 29, "A16", szField );
 
-    if( !STARTS_WITH_CI(szField, "Slant Range") && !STARTS_WITH_CI(szField, "Ground Range")
-        && !STARTS_WITH_CI(szField, "GEOCODED") )
-        return FALSE;
+    int GCPFieldSize = 16;
+    int GCPOffset = 1073;
 
-    GetCeosField( record, 1073, "A16", szField );
+    if( !STARTS_WITH_CI(szField, "Slant Range") &&
+        !STARTS_WITH_CI(szField, "Ground Range")
+        && !STARTS_WITH_CI(szField, "GEOCODED") ) {
+        /* detect ASF map projection record */
+        GetCeosField( record, 1079, "A7", szField );
+        if( !STARTS_WITH_CI(szField, "Slant") &&
+            !STARTS_WITH_CI(szField, "Ground") ) {
+            return FALSE;
+        } else {
+            GCPFieldSize = 17;
+            GCPOffset = 157;
+        }
+    }
+
+    char FieldSize[4];
+    snprintf(FieldSize,sizeof(FieldSize),"A%d",GCPFieldSize);
+
+    GetCeosField( record, GCPOffset, FieldSize, szField );
     if( STARTS_WITH_CI(szField, "        ") )
         return FALSE;
 
@@ -1518,24 +1572,55 @@ int SAR_CEOSDataset::ScanForMapProjection()
         snprintf( szId, sizeof(szId), "%d", i+1 );
         pasGCPList[i].pszId = CPLStrdup( szId );
 
-        GetCeosField( record, 1073+32*i, "A16", szField );
+        GetCeosField( record, GCPOffset+(GCPFieldSize*2)*i, FieldSize, szField );
         pasGCPList[i].dfGCPY = CPLAtof(szField);
-        GetCeosField( record, 1089+32*i, "A16", szField );
+        GetCeosField( record, GCPOffset+GCPFieldSize+(GCPFieldSize*2)*i, FieldSize, szField );
         pasGCPList[i].dfGCPX = CPLAtof(szField);
         pasGCPList[i].dfGCPZ = 0.0;
     }
 
+    /* Map Projection Record has the order UL UR LR LL
+     ASF Facility Data Record has the order UL,LL,UR,LR
+     ASF Map Projection Record has the order LL, LR, UR, UL */
+
     pasGCPList[0].dfGCPLine = 0.5;
     pasGCPList[0].dfGCPPixel = 0.5;
 
-    pasGCPList[1].dfGCPLine = 0.5;
-    pasGCPList[1].dfGCPPixel = nRasterXSize-0.5;
+    switch(gcp_ordering_mode) {
+        case CEOS_ASF_FACREC_GCP_ORDER:
+            pasGCPList[1].dfGCPLine = nRasterYSize-0.5;
+            pasGCPList[1].dfGCPPixel = 0.5;
 
-    pasGCPList[2].dfGCPLine = nRasterYSize-0.5;
-    pasGCPList[2].dfGCPPixel = nRasterXSize-0.5;
+            pasGCPList[2].dfGCPLine = 0.5;
+            pasGCPList[2].dfGCPPixel = nRasterXSize-0.5;
 
-    pasGCPList[3].dfGCPLine = nRasterYSize-0.5;
-    pasGCPList[3].dfGCPPixel = 0.5;
+            pasGCPList[3].dfGCPLine = nRasterYSize-0.5;
+            pasGCPList[3].dfGCPPixel = nRasterXSize-0.5;
+            break;
+        case CEOS_STD_MAPREC_GCP_ORDER:
+            pasGCPList[1].dfGCPLine = 0.5;
+            pasGCPList[1].dfGCPPixel = nRasterXSize-0.5;
+
+            pasGCPList[2].dfGCPLine = nRasterYSize-0.5;
+            pasGCPList[2].dfGCPPixel = nRasterXSize-0.5;
+
+            pasGCPList[3].dfGCPLine = nRasterYSize-0.5;
+            pasGCPList[3].dfGCPPixel = 0.5;
+            break;
+        case CEOS_ASF_MAPREC_GCP_ORDER:
+            pasGCPList[0].dfGCPLine = nRasterYSize-0.5;
+            pasGCPList[0].dfGCPPixel = 0.5;
+
+            pasGCPList[1].dfGCPLine = nRasterYSize-0.5;
+            pasGCPList[1].dfGCPPixel = nRasterXSize-0.5;
+
+            pasGCPList[2].dfGCPLine = 0.5;
+            pasGCPList[2].dfGCPPixel = nRasterXSize-0.5;
+
+            pasGCPList[3].dfGCPLine = 0.5;
+            pasGCPList[3].dfGCPPixel = 0.5;
+            break;
+    }
 
     return TRUE;
 }
@@ -1558,6 +1643,14 @@ void SAR_CEOSDataset::ScanForGCPs()
         return;
     }
 
+    /* ASF L1 products do not have valid data
+       in the lat/long first/mid/last fields */
+    const char *pszValue = GetMetadataItem("CEOS_FACILITY");
+    if( (pszValue != nullptr) && (strncmp(pszValue,"ASF",3) == 0) ) {
+        ScanForMapProjection();
+        return;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Just sample fix scanlines through the image for GCPs, to        */
 /*      return 15 GCPs.  That is an adequate coverage for most          */
@@ -1775,6 +1868,9 @@ GDALDataset *SAR_CEOSDataset::Open( GDALOpenInfo * poOpenInfo )
             {
                 CPLDebug( "CEOS", "Opened %s.\n", pszFilename );
 
+                poDS->papszExtraFiles =
+                    CSLAddString( poDS->papszExtraFiles, pszFilename );
+
                 CPL_IGNORE_RET_VAL(VSIFSeekL( process_fp, 0, SEEK_END ));
                 if( ProcessData( process_fp, iFile, psVolume, -1,
                                  VSIFTellL( process_fp ) ) == 0 )
@@ -1885,7 +1981,7 @@ GDALDataset *SAR_CEOSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
 /* -------------------------------------------------------------------- */
-    poDS->nRasterXSize = psImageDesc->PixelsPerLine;
+    poDS->nRasterXSize = psImageDesc->PixelsPerLine + psImageDesc->LeftBorderPixels + psImageDesc->RightBorderPixels;
     poDS->nRasterYSize = psImageDesc->Lines;
 
     const int bNative =
@@ -2196,10 +2292,24 @@ void GDALRegister_SAR_CEOS()
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
                                "CEOS SAR Image" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
-                               "frmt_various.html#SAR_CEOS" );
+                               "drivers/raster/sar_ceos.html" );
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
     poDriver->pfnOpen = SAR_CEOSDataset::Open;
 
     GetGDALDriverManager()->RegisterDriver( poDriver );
 }
+
+/************************************************************************/
+/*                            GetFileList()                             */
+/************************************************************************/
+
+char **SAR_CEOSDataset::GetFileList()
+
+{
+    char **papszFileList = GDALPamDataset::GetFileList();
+
+    papszFileList = CSLInsertStrings( papszFileList, -1, papszExtraFiles );
+
+    return papszFileList;
+}
