@@ -38,7 +38,7 @@
 #include "gdal_frmts.h"
 #include "gdal_pam.h"
 
-CPL_CVSID("$Id: gs7bgdataset.cpp 86933038c3926cd4dc3ff37c431b317abb69e602 2021-03-27 23:20:49 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -77,7 +77,7 @@ class GS7BGDataset final: public GDALPamDataset
     static int          Identify( GDALOpenInfo * );
     static GDALDataset *Open( GDALOpenInfo * );
     static GDALDataset *Create( const char * pszFilename,
-                    int nXSize, int nYSize, int nBands,
+                    int nXSize, int nYSize, int nBandsIn,
                     GDALDataType eType,
                     char **papszParamList );
     static GDALDataset *CreateCopy( const char *pszFilename,
@@ -493,7 +493,7 @@ double GS7BGRasterBand::GetMaximum( int *pbSuccess )
 GS7BGDataset::~GS7BGDataset()
 
 {
-    FlushCache();
+    FlushCache(true);
     if( fp != nullptr )
         VSIFCloseL( fp );
 }
@@ -1069,7 +1069,7 @@ CPLErr GS7BGDataset::WriteHeader( VSILFILE *fp, GInt32 nXSize, GInt32 nYSize,
 GDALDataset *GS7BGDataset::Create( const char * pszFilename,
                                    int nXSize,
                                    int nYSize,
-                                   int nBands,
+                                   int nBandsIn,
                                    GDALDataType eType,
                                    CPL_UNUSED char **papszParamList )
 
@@ -1094,7 +1094,7 @@ GDALDataset *GS7BGDataset::Create( const char * pszFilename,
         return nullptr;
     }
 
-    if (nBands > 1)
+    if (nBandsIn > 1)
     {
         CPLError( CE_Failure, CPLE_NotSupported,
             "Unable to create copy, "
