@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2008, Xiuguang Zhou (ESRI)
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,7 @@
  ****************************************************************************/
 
 #include "cpl_port.h"
+
 #include "gt_citation.h"
 
 #include <cstddef>
@@ -45,7 +46,7 @@
 #include "gt_wkt_srs_priv.h"
 #include "ogr_core.h"
 
-CPL_CVSID("$Id: gt_citation.cpp 028edae15c93a64d26d691b7c3b45bf141baee2f 2018-05-23 17:44:17 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 static const char * const apszUnitMap[] = {
     "meters", "1.0",
@@ -361,7 +362,7 @@ void SetLinearUnitCitation( std::map<geokey_t, std::string>& oMapAsciiKeys,
 /************************************************************************/
 void SetGeogCSCitation( GTIF * psGTIF,
                         std::map<geokey_t, std::string>& oMapAsciiKeys,
-                        OGRSpatialReference *poSRS,
+                        const OGRSpatialReference *poSRS,
                         const char* angUnitName, int nDatum, short nSpheroid )
 {
     bool bRewriteGeogCitation = false;
@@ -493,8 +494,11 @@ OGRBoolean SetCitationToSRS( GTIF* hGTIF, char* szCTString, int nCTStringLen,
                 }
             }
             if( unitSize == 0.0 )
-                GDALGTIFKeyGetDOUBLE( hGTIF, ProjLinearUnitSizeGeoKey,
-                                      &unitSize, 0, 1 );
+            {
+                CPL_IGNORE_RET_VAL(
+                    GDALGTIFKeyGetDOUBLE( hGTIF, ProjLinearUnitSizeGeoKey,
+                                      &unitSize, 0, 1 ));
+            }
             poSRS->SetLinearUnits( ctNames[CitLUnitsName], unitSize);
             *linearUnitIsSet = TRUE;
         }
@@ -601,7 +605,7 @@ OGRBoolean CheckCitationKeyForStatePlaneUTM( GTIF* hGTIF, GTIFDefn* psDefn,
 
     bool hasUnits = false;
     if( GDALGTIFKeyGetASCII( hGTIF, GTCitationGeoKey, szCTString,
-                             0, sizeof(szCTString) ) )
+                             sizeof(szCTString) ) )
     {
         CPLString osLCCT = szCTString;
 
@@ -695,8 +699,8 @@ OGRBoolean CheckCitationKeyForStatePlaneUTM( GTIF* hGTIF, GTIFDefn* psDefn,
 
     // Check PCSCitationGeoKey if it exists.
     szCTString[0] = '\0';
-    if( hGTIF && GDALGTIFKeyGetASCII( hGTIF, PCSCitationGeoKey, szCTString,
-                                      0, sizeof(szCTString)) )
+    if( GDALGTIFKeyGetASCII( hGTIF, PCSCitationGeoKey, szCTString,
+                             sizeof(szCTString)) )
     {
         // For tif created by LEICA(ERDAS), ESRI state plane pe string was
         // used and the state plane zone is given in PCSCitation. Therefore
