@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gt_overview.h d80d9db4c465d5a3a22407e3a07d85ea4b583ced 2021-03-07 13:00:13 +0100 Even Rouault $
+ * $Id$
  *
  * Project:  GeoTIFF Driver
  * Purpose:  Code to build overviews of external databases as a TIFF file.
@@ -56,8 +56,7 @@ toff_t GTIFFWriteDirectory( TIFF *hTIFF, int nSubfileType,
                             const char* pszJPEGTablesMode,
                             const char* pszNoData,
                             const uint32_t* panLercAddCompressionAndVersion,
-                            bool DeferStrileArrayWriting,
-                            const char * pszWebpLevel );
+                            bool bDeferStrileArrayWriting );
 
 void GTIFFBuildOverviewMetadata( const char *pszResampling,
                                  GDALDataset *poBaseDS,
