@@ -42,7 +42,7 @@
 #include "hdf4compat.h"
 #include "hdf4dataset.h"
 
-CPL_CVSID("$Id: hdf4dataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 extern const char * const pszGDALSignature;
 
@@ -205,9 +205,9 @@ GDALDataType HDF4Dataset::GetDataType( int32 iNumType )
         case DFNT_UINT32:
             return GDT_UInt32;
         case DFNT_INT64:
-            return GDT_Unknown;
+            return GDT_Int64;
         case DFNT_UINT64:
-            return GDT_Unknown;
+            return GDT_UInt64;
         case DFNT_FLOAT32:
             return GDT_Float32;
         case DFNT_FLOAT64:
