@@ -58,7 +58,7 @@
 #include "cpl_vsi.h"
 #include "nitflib.h"
 
-CPL_CVSID("$Id: rpftocfile.cpp df398e80769422a4bbd5d4a295f4ede443c9fec6 2021-04-04 00:17:15 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                        RPFTOCTrim()                                    */
@@ -306,19 +306,14 @@ RPFToc* RPFTOCReadFromBuffer(const char* pszFilename, VSILFILE* fp, const char*
             return nullptr;
         }
 
-        if( toc->entries[i].vertInterval <= 1e-10 ||
+        // do some basic plausibility checks for all entries
+        if (toc->entries[i].vertInterval <= 1e-10 ||
             !CPLIsFinite(toc->entries[i].vertInterval) ||
             toc->entries[i].horizInterval <= 1e-10 ||
             !CPLIsFinite(toc->entries[i].horizInterval) ||
-            !(fabs(toc->entries[i].seLong) <= 360.0) ||
-            !(fabs(toc->entries[i].nwLong) <= 360.0) ||
-            !(fabs(toc->entries[i].nwLat) <= 90.0) ||
-            !(fabs(toc->entries[i].seLat) <= 90.0) ||
-            toc->entries[i].seLong < toc->entries[i].nwLong ||
-            toc->entries[i].nwLat < toc->entries[i].seLat ||
             toc->entries[i].nHorizFrames == 0 ||
             toc->entries[i].nVertFrames == 0 ||
-            toc->entries[i].nHorizFrames > INT_MAX / toc->entries[i].nVertFrames )
+            toc->entries[i].nHorizFrames > INT_MAX / toc->entries[i].nVertFrames)
         {
             CPLError(CE_Failure, CPLE_FileIO, "Invalid TOC entry");
             toc->entries[i].nVertFrames = 0;
@@ -327,6 +322,37 @@ RPFToc* RPFTOCReadFromBuffer(const char* pszFilename, VSILFILE* fp, const char*
             return nullptr;
         }
 
+        // Overview has ZONE 'R' and Legend ZONE 'D' but because the Zone 'D' is also a valid
+        // Zone we need an additional check. -> In all cases of Overview/Legend the values of
+        // the BoundingBox are equal so we simply check here that NW == SE is. 
+        bool isLegendOrOverview =
+            (toc->entries[i].zone[0] == 'R' ||  // Overview
+                (toc->entries[i].zone[0] == 'D' &&  // Legend
+                    memcmp(&(toc->entries[i].seLong), &(toc->entries[i].nwLong), sizeof(toc->entries[i].nwLong)) == 0 &&
+                    memcmp(&(toc->entries[i].seLat), &(toc->entries[i].nwLat), sizeof(toc->entries[i].nwLat)) == 0));
+
+        bool isPolarZone = (toc->entries[i].zone[0] == '9') || (toc->entries[i].zone[0] == 'J');
+
+        // make additional checks of the bounding for charts (without Legends and Overviews)
+        if (!isLegendOrOverview)
+        {
+            if (!(fabs(toc->entries[i].seLong) <= 360.0) ||
+                !(fabs(toc->entries[i].nwLong) <= 360.0) ||
+                !(fabs(toc->entries[i].nwLat) <= 90.0) ||
+                !(fabs(toc->entries[i].seLat) <= 90.0) ||
+                // check only for non-polar zones, because the values are not always correct here
+                (!isPolarZone && (
+                    toc->entries[i].seLong < toc->entries[i].nwLong ||
+                    toc->entries[i].nwLat < toc->entries[i].seLat)))
+            {
+                CPLError(CE_Failure, CPLE_FileIO, "Invalid TOC entry");
+                toc->entries[i].nVertFrames = 0;
+                toc->entries[i].nHorizFrames = 0;
+                RPFTOCFree(toc);
+                return nullptr;
+            }
+        }
+
         // TODO: We could probably use another data structure, like a list,
         // instead of an array referenced by the frame coordinate...
         if( static_cast<int>(toc->entries[i].nHorizFrames *
