@@ -49,7 +49,7 @@ constexpr double NULL3 = -3.4028226550889044521e+38;
 #include "cpl_safemaths.hpp"
 #include "vicardataset.h"
 
-CPL_CVSID("$Id: pdsdataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 enum PDSLayout
 {
@@ -85,12 +85,12 @@ class PDSDataset final: public RawDataset
 
     void        ParseSRS();
     int         ParseCompressedImage();
-    int         ParseImage( CPLString osPrefix, CPLString osFilenamePrefix );
+    int         ParseImage( const CPLString& osPrefix, const CPLString& osFilenamePrefix );
     static void        CleanString( CPLString &osInput );
 
-    const char *GetKeyword( std::string osPath,
+    const char *GetKeyword( const std::string& osPath,
                             const char *pszDefault = "");
-    const char *GetKeywordSub( std::string osPath,
+    const char *GetKeywordSub( const std::string& osPath,
                                int iSubscript,
                                const char *pszDefault = "");
     const char *GetKeywordUnit( const char *pszPath,
@@ -158,7 +158,7 @@ PDSDataset::PDSDataset() :
 PDSDataset::~PDSDataset()
 
 {
-    PDSDataset::FlushCache();
+    PDSDataset::FlushCache(true);
     if( fpImage != nullptr )
         VSIFCloseL( fpImage );
 
@@ -733,7 +733,7 @@ static GUInt32 PDSConvertFromHex(const char* pszVal)
 /*                             ParseImage()                             */
 /************************************************************************/
 
-int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
+int PDSDataset::ParseImage( const CPLString& osPrefix, const CPLString& osFilenamePrefix )
 {
 /* ------------------------------------------------------------------- */
 /*      We assume the user is pointing to the label (i.e. .lbl) file.  */
@@ -1218,7 +1218,7 @@ class PDSWrapperRasterBand final: public GDALProxyRasterBand
   GDALRasterBand* poBaseBand;
 
   protected:
-    virtual GDALRasterBand* RefUnderlyingRasterBand() override { return poBaseBand; }
+    virtual GDALRasterBand* RefUnderlyingRasterBand() const override { return poBaseBand; }
 
   public:
     explicit PDSWrapperRasterBand( GDALRasterBand* poBaseBandIn )
@@ -1443,7 +1443,7 @@ GDALDataset *PDSDataset::Open( GDALOpenInfo * poOpenInfo )
 /*                             GetKeyword()                             */
 /************************************************************************/
 
-const char *PDSDataset::GetKeyword( std::string osPath,
+const char *PDSDataset::GetKeyword( const std::string& osPath,
                                     const char *pszDefault )
 
 {
@@ -1454,7 +1454,7 @@ const char *PDSDataset::GetKeyword( std::string osPath,
 /*                            GetKeywordSub()                           */
 /************************************************************************/
 
-const char *PDSDataset::GetKeywordSub( std::string osPath,
+const char *PDSDataset::GetKeywordSub( const std::string& osPath,
                                        int iSubscript,
                                        const char *pszDefault )
 
