@@ -40,7 +40,7 @@
 
 void CPL_DLL CPL_STDCALL GDALRegister_RASDAMAN();
 
-CPL_CVSID("$Id: rasdamandataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 // cppcheck-suppress copyCtorAndEqOperator
 class Subset
@@ -174,7 +174,7 @@ RasdamanDataset::~RasdamanDataset()
     transaction.commit();
   }
   database.close();
-  FlushCache();
+  FlushCache(true);
 }
 
 CPLErr RasdamanDataset::IRasterIO( GDALRWFlag eRWFlag,
