@@ -32,7 +32,7 @@
 #include "ogr_srs_api.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: ctable2dataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -335,7 +335,7 @@ const char *CTable2Dataset::_GetProjectionRef()
 GDALDataset *CTable2Dataset::Create( const char * pszFilename,
                                      int nXSize,
                                      int nYSize,
-                                     CPL_UNUSED int nBands,
+                                     int /* nBandsIn */,
                                      GDALDataType eType,
                                      char ** papszOptions )
 {
