@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2004, Frank Warmerdam
- * Copyright (c) 2008-2011, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2011, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -36,7 +36,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: landataset.cpp a542b2797f15f2ed694cfcee9ff17d86b339dfee 2018-04-02 00:24:03 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /**
 
@@ -107,11 +107,13 @@ constexpr int ERD_HEADER_SIZE = 128;
 
 class LANDataset;
 
-class LAN4BitRasterBand : public GDALPamRasterBand
+class LAN4BitRasterBand final: public GDALPamRasterBand
 {
     GDALColorTable *poCT;
     GDALColorInterp eInterp;
 
+    CPL_DISALLOW_COPY_ASSIGN(LAN4BitRasterBand)
+
   public:
                    LAN4BitRasterBand( LANDataset *, int );
     ~LAN4BitRasterBand() override;
@@ -130,18 +132,20 @@ class LAN4BitRasterBand : public GDALPamRasterBand
 /* ==================================================================== */
 /************************************************************************/
 
-class LANDataset : public RawDataset
+class LANDataset final: public RawDataset
 {
+    CPL_DISALLOW_COPY_ASSIGN(LANDataset)
+
   public:
     VSILFILE    *fpImage;  // Image data file.
 
     char        pachHeader[ERD_HEADER_SIZE];
 
-    char        *pszProjection;
+    OGRSpatialReference* m_poSRS = nullptr;
 
     double      adfGeoTransform[6];
 
-    CPLString   osSTAFilename;
+    CPLString   osSTAFilename{};
     void        CheckForStatistics(void);
 
     char **GetFileList() override;
@@ -152,12 +156,13 @@ class LANDataset : public RawDataset
 
     CPLErr GetGeoTransform( double * padfTransform ) override;
     CPLErr SetGeoTransform( double * padfTransform ) override;
-    const char *GetProjectionRef() override;
-    CPLErr SetProjection( const char * ) override;
+
+    const OGRSpatialReference* GetSpatialRef() const override ;
+    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override;
 
     static GDALDataset *Open( GDALOpenInfo * );
     static GDALDataset *Create( const char * pszFilename,
-                                int nXSize, int nYSize, int nBands,
+                                int nXSize, int nYSize, int nBandsIn,
                                 GDALDataType eType, char ** papszOptions );
 };
 
@@ -312,8 +317,7 @@ GDALColorInterp LAN4BitRasterBand::GetColorInterpretation()
 /************************************************************************/
 
 LANDataset::LANDataset() :
-    fpImage(nullptr),
-    pszProjection(nullptr)
+    fpImage(nullptr)
 {
     memset( pachHeader, 0, sizeof(pachHeader) );
     adfGeoTransform[0] =  0.0;
@@ -331,7 +335,7 @@ LANDataset::LANDataset() :
 LANDataset::~LANDataset()
 
 {
-    FlushCache();
+    FlushCache(true);
 
     if( fpImage != nullptr )
     {
@@ -341,7 +345,8 @@ LANDataset::~LANDataset()
         }
     }
 
-    CPLFree( pszProjection );
+    if( m_poSRS )
+        m_poSRS->Release();
 }
 
 /************************************************************************/
@@ -364,6 +369,12 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
                             "HEAD74" ) )
         return nullptr;
 
+    if( memcmp(poOpenInfo->pabyHeader + 16, "S LAT   ", 8) == 0 )
+    {
+        // NTV1 format
+        return nullptr;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
@@ -414,9 +425,9 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
     {
         float fTmp = 0.0;
         memcpy(&fTmp, poDS->pachHeader + 16, 4);
-        poDS->nRasterXSize = (int) fTmp;
+        poDS->nRasterXSize = static_cast<int>(fTmp);
         memcpy(&fTmp, poDS->pachHeader + 20, 4);
-        poDS->nRasterYSize = (int) fTmp;
+        poDS->nRasterYSize = static_cast<int>(fTmp);
     }
     else
     {
@@ -467,6 +478,7 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
         return nullptr;
     }
 
+    // cppcheck-suppress knownConditionTrueFalse
     if( nPixelOffset != -1 &&
         poDS->nRasterXSize > INT_MAX / (nPixelOffset * nBandCount) )
     {
@@ -493,7 +505,8 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
                                    * nPixelOffset * poDS->nRasterXSize,
                                    nPixelOffset,
                                    poDS->nRasterXSize*nPixelOffset*nBandCount,
-                                   eDataType, !bNeedSwap, TRUE ));
+                                   eDataType, !bNeedSwap,
+                                   RawRasterBand::OwnFP::NO ));
         if( CPLGetLastErrorType() != CE_None )
         {
             delete poDS;
@@ -551,26 +564,27 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
     memcpy(&nTmp16, poDS->pachHeader + 88, 2);
     int nCoordSys = nTmp16;
 
+    poDS->m_poSRS = new OGRSpatialReference();
+    poDS->m_poSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     if( nCoordSys == 0 )
     {
-        poDS->pszProjection = CPLStrdup(SRS_WKT_WGS84);
+        poDS->m_poSRS->SetFromUserInput(SRS_WKT_WGS84_LAT_LONG);
     }
     else if( nCoordSys == 1 )
     {
-        poDS->pszProjection =
-            CPLStrdup( "LOCAL_CS[\"UTM - Zone Unknown\",UNIT[\"Meter\",1]]" );
+        poDS->m_poSRS->SetFromUserInput(
+            "LOCAL_CS[\"UTM - Zone Unknown\",UNIT[\"Meter\",1]]" );
     }
     else if( nCoordSys == 2 )
     {
-        poDS->pszProjection =
-            CPLStrdup(
+        poDS->m_poSRS->SetFromUserInput(
                 "LOCAL_CS[\"State Plane - Zone Unknown\","
                 "UNIT[\"US survey foot\",0.3048006096012192]]" );
     }
     else
     {
-        poDS->pszProjection =
-            CPLStrdup( "LOCAL_CS[\"Unknown\",UNIT[\"Meter\",1]]" );
+        poDS->m_poSRS->SetFromUserInput(
+            "LOCAL_CS[\"Unknown\",UNIT[\"Meter\",1]]" );
     }
 
 /* -------------------------------------------------------------------- */
@@ -676,51 +690,51 @@ CPLErr LANDataset::SetGeoTransform( double * padfTransform )
 }
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                          GetSpatialRef()                             */
 /*                                                                      */
 /*      Use PAM coordinate system if available in preference to the     */
 /*      generally poor value derived from the file itself.              */
 /************************************************************************/
 
-const char *LANDataset::GetProjectionRef()
+const OGRSpatialReference *LANDataset::GetSpatialRef() const
 
 {
-    const char* pszPamPrj = GDALPamDataset::GetProjectionRef();
+    const auto poSRS = GDALPamDataset::GetSpatialRef();
+    if( poSRS )
+        return poSRS;
 
-    if( pszProjection != nullptr && strlen(pszPamPrj) == 0 )
-        return pszProjection;
-
-    return pszPamPrj;
+    return m_poSRS;
 }
 
 /************************************************************************/
-/*                           SetProjection()                            */
+/*                           SetSpatialRef()                            */
 /************************************************************************/
 
-CPLErr LANDataset::SetProjection( const char * pszWKT )
+CPLErr LANDataset::SetSpatialRef( const OGRSpatialReference * poSRS )
 
 {
+    if( poSRS == nullptr )
+        return GDALPamDataset::SetSpatialRef( poSRS );
+
     unsigned char abyHeader[128] = { '\0' };
 
     CPL_IGNORE_RET_VAL(VSIFSeekL( fpImage, 0, SEEK_SET ));
     CPL_IGNORE_RET_VAL(VSIFReadL( abyHeader, 128, 1, fpImage ));
 
-    OGRSpatialReference oSRS( pszWKT );
-
     GUInt16 nProjCode = 0;
 
-    if( oSRS.IsGeographic() )
+    if( poSRS->IsGeographic() )
     {
         nProjCode = 0;
     }
-    else if( oSRS.GetUTMZone() != 0 )
+    else if( poSRS->GetUTMZone() != 0 )
     {
         nProjCode = 1;
     }
     // Too bad we have no way of recognising state plane projections.
     else
     {
-        const char *l_pszProjection = oSRS.GetAttrValue("PROJECTION");
+        const char *l_pszProjection = poSRS->GetAttrValue("PROJECTION");
 
         if( l_pszProjection == nullptr )
             ;
@@ -783,7 +797,7 @@ CPLErr LANDataset::SetProjection( const char * pszWKT )
     CPL_IGNORE_RET_VAL(VSIFSeekL( fpImage, 0, SEEK_SET ));
     CPL_IGNORE_RET_VAL(VSIFWriteL( abyHeader, 128, 1, fpImage ));
 
-    return GDALPamDataset::SetProjection( pszWKT );
+    return GDALPamDataset::SetSpatialRef( poSRS );
 }
 
 /************************************************************************/
@@ -879,7 +893,7 @@ void LANDataset::CheckForStatistics()
 GDALDataset *LANDataset::Create( const char * pszFilename,
                                  int nXSize,
                                  int nYSize,
-                                 int nBands,
+                                 int nBandsIn,
                                  GDALDataType eType,
                                  char ** /* papszOptions */ )
 {
@@ -921,7 +935,7 @@ GDALDataset *LANDataset::Create( const char * pszFilename,
     memcpy( abyHeader + 6, &n16Val, 2 );
 
     // Number of Bands.
-    n16Val = static_cast<GInt16>( nBands );
+    n16Val = static_cast<GInt16>( nBandsIn );
     memcpy( abyHeader + 8, &n16Val, 2 );
 
     // Unknown (6).
@@ -992,7 +1006,7 @@ GDALDataset *LANDataset::Create( const char * pszFilename,
         const vsi_l_offset nWriteThisTime
             = std::min( static_cast<size_t>( nImageBytes ), sizeof(abyHeader) );
 
-        if( VSIFWriteL( abyHeader, 1, (size_t)nWriteThisTime, fp )
+        if( VSIFWriteL( abyHeader, 1, static_cast<size_t>(nWriteThisTime), fp )
             != nWriteThisTime )
         {
             CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
@@ -1028,7 +1042,7 @@ void GDALRegister_LAN()
     poDriver->SetDescription( "LAN" );
     poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Erdas .LAN/.GIS" );
-    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#LAN" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/lan.html" );
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, "Byte Int16" );
 
