@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2003, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -42,7 +42,7 @@
 #include "gdal.h"
 #include "gdal_priv.h"
 
-CPL_CVSID("$Id: vrtfilters.cpp 64e5d4fba213ab6255a4dddcd09e8cda46848d7d 2019-10-04 13:50:44 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /*! @cond Doxygen_Suppress */
 
@@ -146,6 +146,18 @@ VRTFilteredSource::RasterIO( GDALDataType eBandDataType,
                                            psExtraArg );
     }
 
+    double dfXOff = nXOff;
+    double dfYOff = nYOff;
+    double dfXSize = nXSize;
+    double dfYSize = nYSize;
+    if( psExtraArg != nullptr && psExtraArg->bFloatingPointWindowValidity )
+    {
+        dfXOff = psExtraArg->dfXOff;
+        dfYOff = psExtraArg->dfYOff;
+        dfXSize = psExtraArg->dfXSize;
+        dfYSize = psExtraArg->dfYSize;
+    }
+
     // The window we will actually request from the source raster band.
     double dfReqXOff = 0.0;
     double dfReqYOff = 0.0;
@@ -162,11 +174,15 @@ VRTFilteredSource::RasterIO( GDALDataType eBandDataType,
     int nOutXSize = 0;
     int nOutYSize = 0;
 
-    if( !GetSrcDstWindow( nXOff, nYOff, nXSize, nYSize, nBufXSize, nBufYSize,
+    bool bError = false;
+    if( !GetSrcDstWindow( dfXOff, dfYOff, dfXSize, dfYSize, nBufXSize, nBufYSize,
                         &dfReqXOff, &dfReqYOff, &dfReqXSize, &dfReqYSize,
                         &nReqXOff, &nReqYOff, &nReqXSize, &nReqYSize,
-                        &nOutXOff, &nOutYOff, &nOutXSize, &nOutYSize ) )
-        return CE_None;
+                        &nOutXOff, &nOutYOff, &nOutXSize, &nOutYSize,
+                        bError ) )
+    {
+        return bError ? CE_Failure : CE_None;
+    }
 
     pData = reinterpret_cast<GByte *>( pData )
         + nPixelSpace * nOutXOff
@@ -183,9 +199,13 @@ VRTFilteredSource::RasterIO( GDALDataType eBandDataType,
     if( IsTypeSupported( eBufType ) )
         eOperDataType = eBufType;
 
+    auto l_band = GetRasterBand();
+    if( !l_band )
+        return CE_Failure;
+
     if( eOperDataType == GDT_Unknown
-        && IsTypeSupported( m_poRasterBand->GetRasterDataType() ) )
-        eOperDataType = m_poRasterBand->GetRasterDataType();
+        && IsTypeSupported( l_band->GetRasterDataType() ) )
+        eOperDataType = l_band->GetRasterDataType();
 
     if( eOperDataType == GDT_Unknown )
     {
@@ -282,15 +302,15 @@ VRTFilteredSource::RasterIO( GDALDataType eBandDataType,
         nFileYSize -= nTopFill;
     }
 
-    if( nFileXOff + nFileXSize > m_poRasterBand->GetXSize() )
+    if( nFileXOff + nFileXSize > l_band->GetXSize() )
     {
-        nRightFill = nFileXOff + nFileXSize - m_poRasterBand->GetXSize();
+        nRightFill = nFileXOff + nFileXSize - l_band->GetXSize();
         nFileXSize -= nRightFill;
     }
 
-    if( nFileYOff + nFileYSize > m_poRasterBand->GetYSize() )
+    if( nFileYOff + nFileYSize > l_band->GetYSize() )
     {
-        nBottomFill = nFileYOff + nFileYSize - m_poRasterBand->GetYSize();
+        nBottomFill = nFileYOff + nFileYSize - l_band->GetYSize();
         nFileYSize -= nBottomFill;
     }
 
@@ -496,14 +516,17 @@ CPLErr VRTKernelFilteredSource::FilterData( int nXSize, int nYSize,
 /* -------------------------------------------------------------------- */
 /*      Float32 case.                                                   */
 /* -------------------------------------------------------------------- */
-    if( eType == GDT_Float32 )
+    // if( eType == GDT_Float32 )
     {
         float *pafSrcData = reinterpret_cast<float *>( pabySrcData );
         float *pafDstData = reinterpret_cast<float *>( pabyDstData );
 
         int bHasNoData = FALSE;
+        auto l_poBand = GetRasterBand();
+        if( !l_poBand )
+            return CE_Failure;
         const float fNoData =
-            static_cast<float>( m_poRasterBand->GetNoDataValue(&bHasNoData) );
+            static_cast<float>( l_poBand->GetNoDataValue(&bHasNoData) );
 
         const int nAxisCount = m_bSeparable ? 2 : 1;
 
@@ -577,13 +600,11 @@ CPLErr VRTKernelFilteredSource::FilterData( int nXSize, int nYSize,
 
 CPLErr VRTKernelFilteredSource::XMLInit( CPLXMLNode *psTree,
                                          const char *pszVRTPath,
-                                         void* pUniqueHandle,
                                          std::map<CPLString, GDALDataset*>& oMapSharedSources )
 
 {
     {
         const CPLErr eErr = VRTFilteredSource::XMLInit( psTree, pszVRTPath,
-                                                        pUniqueHandle,
                                                         oMapSharedSources );
         if( eErr != CE_None )
             return eErr;
@@ -681,14 +702,13 @@ CPLXMLNode *VRTKernelFilteredSource::SerializeToXML( const char *pszVRTPath )
 /************************************************************************/
 
 VRTSource *VRTParseFilterSources( CPLXMLNode *psChild, const char *pszVRTPath,
-                                  void* pUniqueHandle,
                                   std::map<CPLString, GDALDataset*>& oMapSharedSources )
 
 {
     if( EQUAL(psChild->pszValue, "KernelFilteredSource") )
     {
         VRTSource *poSrc = new VRTKernelFilteredSource();
-        if( poSrc->XMLInit( psChild, pszVRTPath, pUniqueHandle, oMapSharedSources ) == CE_None )
+        if( poSrc->XMLInit( psChild, pszVRTPath, oMapSharedSources ) == CE_None )
             return poSrc;
 
         delete poSrc;
