@@ -50,7 +50,7 @@
 
 /*! @cond Doxygen_Suppress */
 
-CPL_CVSID("$Id: vrtrasterband.cpp 126b0897e64c233ed06ca072549e110bb6b28ced 2021-04-20 16:42:23 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -62,28 +62,9 @@ CPL_CVSID("$Id: vrtrasterband.cpp 126b0897e64c233ed06ca072549e110bb6b28ced 2021-
 /*                           VRTRasterBand()                            */
 /************************************************************************/
 
-VRTRasterBand::VRTRasterBand() :
-    m_bIsMaskBand(FALSE),
-    m_bNoDataValueSet(FALSE),
-    m_bHideNoDataValue(FALSE),
-    m_dfNoDataValue(-10000.0),
-    m_eColorInterp(GCI_Undefined),
-    m_pszUnitType(nullptr),
-    m_papszCategoryNames(nullptr),
-    m_dfOffset(0.0),
-    m_dfScale(1.0),
-    m_psSavedHistograms(nullptr),
-    m_poMaskBand(nullptr)
+VRTRasterBand::VRTRasterBand()
 {
-    // Initialize( 0, 0 );
-    poDS = nullptr;
-    nBand = 0;
-    eAccess = GA_ReadOnly;
-    eDataType = GDT_Byte;
-    nRasterXSize = 0;
-    nRasterYSize = 0;
-    nBlockXSize = 0;
-    nBlockYSize = 0;
+    VRTRasterBand::Initialize( 0, 0 );
 }
 
 /************************************************************************/
@@ -103,23 +84,6 @@ void VRTRasterBand::Initialize( int nXSize, int nYSize )
 
     nBlockXSize = std::min( 128, nXSize );
     nBlockYSize = std::min( 128, nYSize );
-
-    m_bIsMaskBand = FALSE;
-    m_bNoDataValueSet = FALSE;
-    m_bHideNoDataValue = FALSE;
-    m_dfNoDataValue = -10000.0;
-    m_poColorTable.reset();
-    m_eColorInterp = GCI_Undefined;
-    m_poRAT.reset();
-
-    m_pszUnitType = nullptr;
-    m_papszCategoryNames = nullptr;
-    m_dfOffset = 0.0;
-    m_dfScale = 1.0;
-
-    m_psSavedHistograms = nullptr;
-
-    m_poMaskBand = nullptr;
 }
 
 /************************************************************************/
@@ -158,12 +122,7 @@ CPLErr VRTRasterBand::CopyCommonInfoFrom( GDALRasterBand * poSrcBand )
     if( strlen(poSrcBand->GetDescription()) > 0 )
         SetDescription( poSrcBand->GetDescription() );
 
-    int bSuccess;
-    double dfNoData;
-    dfNoData = poSrcBand->GetNoDataValue( &bSuccess );
-    if( bSuccess )
-        SetNoDataValue( dfNoData );
-
+    GDALCopyNoDataValue(this, poSrcBand);
     SetOffset( poSrcBand->GetOffset() );
     SetScale( poSrcBand->GetScale() );
     SetCategoryNames( poSrcBand->GetCategoryNames() );
@@ -329,7 +288,6 @@ CPLErr VRTRasterBand::SetCategoryNames( char ** papszNewNames )
 
 CPLErr VRTRasterBand::XMLInit( CPLXMLNode * psTree,
                                const char *pszVRTPath,
-                               void* pUniqueHandle,
                                std::map<CPLString, GDALDataset*>& oMapSharedSources )
 
 {
@@ -401,8 +359,24 @@ CPLErr VRTRasterBand::XMLInit( CPLXMLNode * psTree,
 /* -------------------------------------------------------------------- */
     SetDescription( CPLGetXMLValue( psTree, "Description", "" ) );
 
-    if( CPLGetXMLValue( psTree, "NoDataValue", nullptr ) != nullptr )
-        SetNoDataValue( CPLAtofM(CPLGetXMLValue( psTree, "NoDataValue", "0" )) );
+    const char* pszNoDataValue = CPLGetXMLValue( psTree, "NoDataValue", nullptr );
+    if( pszNoDataValue != nullptr )
+    {
+        if( eDataType == GDT_Int64 )
+        {
+            SetNoDataValueAsInt64(
+                static_cast<int64_t>(std::strtoll(pszNoDataValue, nullptr, 10)));
+        }
+        else if( eDataType == GDT_UInt64 )
+        {
+            SetNoDataValueAsUInt64(
+                static_cast<uint64_t>(std::strtoull(pszNoDataValue, nullptr, 10)));
+        }
+        else
+        {
+            SetNoDataValue( CPLAtofM(pszNoDataValue) );
+        }
+    }
 
     if( CPLGetXMLValue( psTree, "HideNoDataValue", nullptr ) != nullptr )
         m_bHideNoDataValue = CPLTestBool( CPLGetXMLValue( psTree, "HideNoDataValue", "0" ) );
@@ -593,7 +567,7 @@ CPLErr VRTRasterBand::XMLInit( CPLXMLNode * psTree,
             break;
         }
 
-        if( poBand->XMLInit( psNode, pszVRTPath, pUniqueHandle,
+        if( poBand->XMLInit( psNode, pszVRTPath,
                              oMapSharedSources ) == CE_None )
         {
             SetMaskBand(poBand);
@@ -658,11 +632,16 @@ CPLXMLNode *VRTRasterBand::SerializeToXML( const char *pszVRTPath )
     if( nBand > 0 )
         CPLSetXMLValue( psTree, "#band", CPLSPrintf( "%d", GetBand() ) );
 
-    if( nBlockXSize != 128 && nBlockXSize != nRasterXSize )
-        CPLSetXMLValue( psTree, "#blockXSize", CPLSPrintf( "%d", nBlockXSize ) );
+    // Do not serialize block size of VRTWarpedRasterBand since it is already
+    // serialized at the dataset level.
+    if( dynamic_cast<VRTWarpedRasterBand*>(this) == nullptr )
+    {
+        if( nBlockXSize != 128 && nBlockXSize != nRasterXSize )
+            CPLSetXMLValue( psTree, "#blockXSize", CPLSPrintf( "%d", nBlockXSize ) );
 
-    if( nBlockYSize != 128 && nBlockYSize != nRasterYSize )
-        CPLSetXMLValue( psTree, "#blockYSize", CPLSPrintf( "%d", nBlockYSize ) );
+        if( nBlockYSize != 128 && nBlockYSize != nRasterYSize )
+            CPLSetXMLValue( psTree, "#blockYSize", CPLSPrintf( "%d", nBlockYSize ) );
+    }
 
     CPLXMLNode *psMD = oMDMD.Serialize();
     if( psMD != nullptr )
@@ -678,6 +657,16 @@ CPLXMLNode *VRTRasterBand::SerializeToXML( const char *pszVRTPath )
         CPLSetXMLValue( psTree, "NoDataValue",
             VRTSerializeNoData(m_dfNoDataValue, eDataType, 16).c_str());
     }
+    else if( m_bNoDataSetAsInt64 )
+    {
+        CPLSetXMLValue( psTree, "NoDataValue",
+            CPLSPrintf( CPL_FRMT_GIB, static_cast<GIntBig>(m_nNoDataValueInt64)) );
+    }
+    else if( m_bNoDataSetAsUInt64 )
+    {
+        CPLSetXMLValue( psTree, "NoDataValue",
+            CPLSPrintf( CPL_FRMT_GUIB, static_cast<GUIntBig>(m_nNoDataValueUInt64)) );
+    }
 
     if( m_bHideNoDataValue )
         CPLSetXMLValue( psTree, "HideNoDataValue",
@@ -821,6 +810,22 @@ CPLXMLNode *VRTRasterBand::SerializeToXML( const char *pszVRTPath )
     return psTree;
 }
 
+/************************************************************************/
+/*                         ResetNoDataValues()                          */
+/************************************************************************/
+
+void VRTRasterBand::ResetNoDataValues()
+{
+    m_bNoDataValueSet = FALSE;
+    m_dfNoDataValue = VRT_DEFAULT_NODATA_VALUE;
+
+    m_bNoDataSetAsInt64 = false;
+    m_nNoDataValueInt64 = GDAL_PAM_DEFAULT_NODATA_VALUE_INT64;
+
+    m_bNoDataSetAsUInt64 = false;
+    m_nNoDataValueUInt64 = GDAL_PAM_DEFAULT_NODATA_VALUE_UINT64;
+}
+
 /************************************************************************/
 /*                           SetNoDataValue()                           */
 /************************************************************************/
@@ -833,6 +838,8 @@ CPLErr VRTRasterBand::SetNoDataValue( double dfNewValue )
         dfNewValue = GDALAdjustNoDataCloseToFloatMax(dfNewValue);
     }
 
+    ResetNoDataValues();
+
     m_bNoDataValueSet = TRUE;
     m_dfNoDataValue = dfNewValue;
 
@@ -841,14 +848,47 @@ CPLErr VRTRasterBand::SetNoDataValue( double dfNewValue )
     return CE_None;
 }
 
+/************************************************************************/
+/*                       SetNoDataValueAsInt64()                        */
+/************************************************************************/
+
+CPLErr VRTRasterBand::SetNoDataValueAsInt64( int64_t nNewValue )
+
+{
+    ResetNoDataValues();
+
+    m_bNoDataSetAsInt64 = true;
+    m_nNoDataValueInt64 = nNewValue;
+
+    static_cast<VRTDataset *>( poDS )->SetNeedsFlush();
+
+    return CE_None;
+}
+
+/************************************************************************/
+/*                      SetNoDataValueAsUInt64()                        */
+/************************************************************************/
+
+CPLErr VRTRasterBand::SetNoDataValueAsUInt64( uint64_t nNewValue )
+
+{
+    ResetNoDataValues();
+
+    m_bNoDataSetAsUInt64 = true;
+    m_nNoDataValueUInt64 = nNewValue;
+
+    static_cast<VRTDataset *>( poDS )->SetNeedsFlush();
+
+    return CE_None;
+}
+
 /************************************************************************/
 /*                         DeleteNoDataValue()                          */
 /************************************************************************/
 
 CPLErr VRTRasterBand::DeleteNoDataValue()
 {
-    m_bNoDataValueSet = FALSE;
-    m_dfNoDataValue = -10000.0;
+    ResetNoDataValues();
 
     static_cast<VRTDataset *>( poDS )->SetNeedsFlush();
 
@@ -870,12 +910,87 @@ CPLErr VRTRasterBand::UnsetNoDataValue()
 double VRTRasterBand::GetNoDataValue( int *pbSuccess )
 
 {
+    if( m_bNoDataSetAsInt64 )
+    {
+        if( pbSuccess )
+            *pbSuccess = !m_bHideNoDataValue;
+        return GDALGetNoDataValueCastToDouble(m_nNoDataValueInt64);
+    }
+
+    if( m_bNoDataSetAsUInt64 )
+    {
+        if( pbSuccess )
+            *pbSuccess = !m_bHideNoDataValue;
+        return GDALGetNoDataValueCastToDouble(m_nNoDataValueUInt64);
+    }
+
     if( pbSuccess )
         *pbSuccess = m_bNoDataValueSet && !m_bHideNoDataValue;
 
     return m_dfNoDataValue;
 }
 
+/************************************************************************/
+/*                        GetNoDataValueAsInt64()                       */
+/************************************************************************/
+
+int64_t VRTRasterBand::GetNoDataValueAsInt64( int *pbSuccess )
+
+{
+    if( eDataType == GDT_UInt64 )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "GetNoDataValueAsUInt64() should be called instead");
+        if( pbSuccess )
+            *pbSuccess = FALSE;
+        return GDAL_PAM_DEFAULT_NODATA_VALUE_INT64;
+    }
+    if( eDataType != GDT_Int64 )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "GetNoDataValue() should be called instead");
+        if( pbSuccess )
+            *pbSuccess = FALSE;
+        return GDAL_PAM_DEFAULT_NODATA_VALUE_INT64;
+    }
+
+    if( pbSuccess )
+        *pbSuccess = m_bNoDataSetAsInt64 && !m_bHideNoDataValue;
+
+    return m_nNoDataValueInt64;
+}
+
+/************************************************************************/
+/*                       GetNoDataValueAsUInt64()                       */
+/************************************************************************/
+
+uint64_t VRTRasterBand::GetNoDataValueAsUInt64( int *pbSuccess )
+
+{
+    if( eDataType == GDT_Int64 )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "GetNoDataValueAsInt64() should be called instead");
+        if( pbSuccess )
+            *pbSuccess = FALSE;
+        return GDAL_PAM_DEFAULT_NODATA_VALUE_UINT64;
+    }
+    if( eDataType != GDT_UInt64 )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "GetNoDataValue() should be called instead");
+        if( pbSuccess )
+            *pbSuccess = FALSE;
+        return GDAL_PAM_DEFAULT_NODATA_VALUE_UINT64;
+    }
+
+    if( pbSuccess )
+        *pbSuccess = m_bNoDataSetAsUInt64 && !m_bHideNoDataValue;
+
+    return m_nNoDataValueUInt64;
+}
+
+
 /************************************************************************/
 /*                           SetColorTable()                            */
 /************************************************************************/
@@ -1152,7 +1267,7 @@ void VRTRasterBand::GetFileList(char*** ppapszFileList, int *pnSize,
 int VRTRasterBand::GetOverviewCount()
 
 {
-    VRTDataset* poVRTDS = static_cast<VRTDataset *>( poDS );
+    VRTDataset* poVRTDS = cpl::down_cast<VRTDataset *>( poDS );
     if( !poVRTDS->AreOverviewsEnabled() )
         return 0;
 
@@ -1165,8 +1280,27 @@ int VRTRasterBand::GetOverviewCount()
     if( nOverviewCount )
         return nOverviewCount;
 
-    // If not found, implicit virtual overviews
-    poVRTDS->BuildVirtualOverviews();
+    if( poVRTDS->m_apoOverviews.empty() )
+    {
+        // If not found, implicit virtual overviews
+
+        const std::string osFctId("VRTRasterBand::GetOverviewCount");
+        GDALAntiRecursionGuard oGuard(osFctId);
+        if( oGuard.GetCallDepth() >= 32 )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "Recursion detected");
+            return 0;
+        }
+
+        GDALAntiRecursionGuard oGuard2(oGuard, poVRTDS->GetDescription());
+        if( oGuard2.GetCallDepth() >= 2 )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "Recursion detected");
+            return 0;
+        }
+
+        poVRTDS->BuildVirtualOverviews();
+    }
     if( !poVRTDS->m_apoOverviews.empty() && poVRTDS->m_apoOverviews[0] )
         return static_cast<int>( poVRTDS->m_apoOverviews.size() );
 
@@ -1336,6 +1470,16 @@ void VRTRasterBand::SetIsMaskBand()
     m_bIsMaskBand = TRUE;
 }
 
+/************************************************************************/
+/*                            IsMaskBand()                              */
+/************************************************************************/
+
+bool VRTRasterBand::IsMaskBand() const
+{
+    return m_bIsMaskBand ||
+           m_eColorInterp == GCI_AlphaBand;
+}
+
 /************************************************************************/
 /*                        CloseDependentDatasets()                      */
 /************************************************************************/
