@@ -53,7 +53,7 @@
 #include "gdalwarper.h"
 #include "ogr_geometry.h"
 
-CPL_CVSID("$Id: vrtwarped.cpp 831355f692b610e0562802d39c65c015e68467d5 2021-10-28 11:30:49 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                      GDALAutoCreateWarpedVRT()                       */
@@ -79,6 +79,29 @@ CPL_CVSID("$Id: vrtwarped.cpp 831355f692b610e0562802d39c65c015e68467d5 2021-10-2
  * dataset should be honoured.  That is, don't just GDALClose() it unless it
  * was opened with GDALOpenShared().
  *
+ * It is possible to "transfer" the ownership of the source dataset
+ * to the warped dataset in the following way:
+ *
+ * \code{.c}
+ *      GDALDatasetH src_ds = GDALOpen("source.tif");
+ *      GDALDatasetH warped_ds = GDALAutoCreateWarpedVRT( src_ds, ... );
+ *      GDALReleaseDataset(src_ds); // src_ds is not "owned" fully by warped_ds. Do NOT use GDALClose(src_ds) here
+ *      ...
+ *      ...
+ *      GDALReleaseDataset(warped_ds); // or GDALClose(warped_ds);
+ * \endcode
+ *
+ * Traditonal nested calls are also possible of course:
+ *
+ * \code{.c}
+ *      GDALDatasetH src_ds = GDALOpen("source.tif");
+ *      GDALDatasetH warped_ds = GDALAutoCreateWarpedVRT( src_ds, ... );
+ *      ...
+ *      ...
+ *      GDALReleaseDataset(warped_ds); // or GDALClose(warped_ds);
+ *      GDALReleaseDataset(src_ds); // or GDALClose(src_ds);
+ * \endcode
+ *
  * The returned dataset will have no associated filename for itself.  If you
  * want to write the virtual dataset description to a file, use the
  * GDALSetDescription() function (or SetDescription() method) on the dataset
@@ -304,6 +327,34 @@ GDALAutoCreateWarpedVRTEx( GDALDatasetH hSrcDS,
  * dataset should be honoured.  That is, don't just GDALClose() it unless it
  * was opened with GDALOpenShared().
  *
+ * It is possible to "transfer" the ownership of the source dataset
+ * to the warped dataset in the following way:
+ *
+ * \code{.c}
+ *      GDALDatasetH src_ds = GDALOpen("source.tif");
+ *      GDALDatasetH warped_ds = GDALAutoCreateWarpedVRT( src_ds, ... );
+ *      GDALReleaseDataset(src_ds); // src_ds is not "owned" fully by warped_ds. Do NOT use GDALClose(src_ds) here
+ *      ...
+ *      ...
+ *      GDALReleaseDataset(warped_ds); // or GDALClose(warped_ds);
+ * \endcode
+ *
+ * Traditonal nested calls are also possible of course:
+ *
+ * \code{.c}
+ *      GDALDatasetH src_ds = GDALOpen("source.tif");
+ *      GDALDatasetH warped_ds = GDALAutoCreateWarpedVRT( src_ds, ... );
+ *      ...
+ *      ...
+ *      GDALReleaseDataset(warped_ds); // or GDALClose(warped_ds);
+ *      GDALReleaseDataset(src_ds); // or GDALClose(src_ds);
+ * \endcode
+ *
+ * The returned dataset will have no associated filename for itself.  If you
+ * want to write the virtual dataset description to a file, use the
+ * GDALSetDescription() function (or SetDescription() method) on the dataset
+ * to assign a filename before it is closed.
+ *
  * @param hSrcDS The source dataset.
  *
  * @param nPixels Width of the virtual warped dataset to create
@@ -1583,8 +1634,29 @@ CPLXMLNode *VRTWarpedDataset::SerializeToXML( const char *pszVRTPathIn )
         if( VSIStatExL( psSDS->psChild->pszValue, &sStat,
                         VSI_STAT_EXISTS_FLAG) == 0 )
         {
+            std::string osVRTFilename = pszVRTPathIn;
+            std::string osSourceDataset = psSDS->psChild->pszValue;
+            char* pszCurDir = CPLGetCurrentDir();
+            if( CPLIsFilenameRelative(osSourceDataset.c_str()) &&
+                !CPLIsFilenameRelative(osVRTFilename.c_str()) &&
+                pszCurDir != nullptr )
+            {
+                osSourceDataset = CPLFormFilename(pszCurDir,
+                                                  osSourceDataset.c_str(),
+                                                  nullptr);
+            }
+            else if( !CPLIsFilenameRelative(osSourceDataset.c_str()) &&
+                     CPLIsFilenameRelative(osVRTFilename.c_str()) &&
+                     pszCurDir != nullptr )
+            {
+                osVRTFilename = CPLFormFilename(pszCurDir,
+                                                osVRTFilename.c_str(),
+                                                nullptr);
+            }
+            CPLFree(pszCurDir);
             char *pszRelativePath = CPLStrdup(
-                CPLExtractRelativePath( pszVRTPathIn, psSDS->psChild->pszValue,
+                CPLExtractRelativePath( osVRTFilename.c_str(),
+                                        osSourceDataset.c_str(),
                                         &bRelativeToVRT ) );
 
             CPLFree( psSDS->psChild->pszValue );
@@ -1775,17 +1847,34 @@ CPLErr VRTWarpedRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
 {
     VRTWarpedDataset *poWDS = static_cast<VRTWarpedDataset *>( poDS );
+    const GPtrDiff_t nDataBytes
+        = static_cast<GPtrDiff_t>(GDALGetDataTypeSizeBytes(eDataType)) *
+            nBlockXSize * nBlockYSize;
+
     GDALRasterBlock *poBlock = GetLockedBlockRef( nBlockXOff, nBlockYOff, TRUE );
     if( poBlock == nullptr )
         return CE_Failure;
 
+    if( poWDS->m_poWarper )
+    {
+        const GDALWarpOptions *psWO = poWDS->m_poWarper->GetOptions();
+        if( nBand == psWO->nDstAlphaBand )
+        {
+            // For a reader starting by asking on band 1, we should normally
+            // not reach here, because ProcessBlock() on band 1 will have
+            // populated the block cache for the regular bands and the alpha
+            // band.
+            // But if there's no source window corresponding to the block,
+            // the alpha band block will not be written through RasterIO(),
+            // so we nee to initialize it.
+            memset(poBlock->GetDataRef(), 0, nDataBytes);
+        }
+    }
+
     const CPLErr eErr = poWDS->ProcessBlock( nBlockXOff, nBlockYOff );
 
     if( eErr == CE_None && pImage != poBlock->GetDataRef() )
     {
-        const GPtrDiff_t nDataBytes
-            = static_cast<GPtrDiff_t>(GDALGetDataTypeSize(poBlock->GetDataType()) / 8)
-            * poBlock->GetXSize() * poBlock->GetYSize();
         memcpy( pImage, poBlock->GetDataRef(), nDataBytes );
     }
 
@@ -1807,8 +1896,9 @@ CPLErr VRTWarpedRasterBand::IWriteBlock( int nBlockXOff, int nBlockYOff,
     // This is a bit tricky. In the case we are warping a VRTWarpedDataset
     // with a destination alpha band, IWriteBlock can be called on that alpha
     // band by GDALWarpDstAlphaMasker
-    // We don't need to do anything since the data will be kept in the block
-    // cache by VRTWarpedRasterBand::IReadBlock.
+    // We don't need to do anything since the data will have hopefully been
+    // read from the block cache before if the reader processes all the bands
+    // of a same block.
     if (poWDS->m_poWarper->GetOptions()->nDstAlphaBand != nBand)
     {
         /* Otherwise, call the superclass method, that will fail of course */
