@@ -2,10 +2,10 @@
  *
  * Project:  GDAL WEBP Driver
  * Purpose:  Implement GDAL WEBP Support based on libwebp
- * Author:   Even Rouault, <even dot rouault at mines dash paris dot org>
+ * Author:   Even Rouault, <even dot rouault at spatialys.com>
  *
  ******************************************************************************
- * Copyright (c) 2011-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2011-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +32,7 @@
 
 #include "webp_headers.h"
 
-CPL_CVSID("$Id: webpdataset.cpp 3189229c71a9620126f6b349f4f80399baeaf528 2019-04-20 20:33:36 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -42,7 +42,7 @@ CPL_CVSID("$Id: webpdataset.cpp 3189229c71a9620126f6b349f4f80399baeaf528 2019-04
 
 class WEBPRasterBand;
 
-class WEBPDataset : public GDALPamDataset
+class WEBPDataset final: public GDALPamDataset
 {
     friend class WEBPRasterBand;
 
@@ -83,7 +83,7 @@ class WEBPDataset : public GDALPamDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class WEBPRasterBand : public GDALPamRasterBand
+class WEBPRasterBand final: public GDALPamRasterBand
 {
     friend class WEBPDataset;
 
@@ -174,7 +174,7 @@ WEBPDataset::WEBPDataset() :
 WEBPDataset::~WEBPDataset()
 
 {
-    FlushCache();
+    FlushCache(true);
     if (fpImage)
         VSIFCloseL(fpImage);
     VSIFree(pabyUncompressed);
@@ -695,6 +695,9 @@ WEBPDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     if (sConfig.lossless)
         sPicture.use_argb = 1;
 #endif
+#if WEBP_ENCODER_ABI_VERSION >= 0x0209
+    FETCH_AND_SET_OPTION_INT("EXACT", exact, 0, 1);
+#endif
 
     if (!WebPValidateConfig(&sConfig))
     {
@@ -874,7 +877,7 @@ void GDALRegister_WEBP()
     poDriver->SetDescription( "WEBP" );
     poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "WEBP" );
-    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_webp.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/webp.html" );
     poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "webp" );
     poDriver->SetMetadataItem( GDAL_DMD_MIMETYPE, "image/webp" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, "Byte" );
@@ -908,6 +911,9 @@ void GDALRegister_WEBP()
 #if WEBP_ENCODER_ABI_VERSION >= 0x0002
 "   <Option name='PARTITION_LIMIT' type='int' description='quality degradation allowed to fit the 512k limit on prediction modes coding (0=no degradation, 100=full)' default='0'/>\n"
 #endif
+#if WEBP_ENCODER_ABI_VERSION >= 0x0209
+"   <Option name='EXACT' type='int' description='preserve the exact RGB values under transparent area. off=0, on=1' default='0'/>\n"
+#endif
 "</CreationOptionList>\n" );
 
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
