@@ -677,35 +677,45 @@ bool ZarrGroupV2::InitFromZGroup(const CPLJSONObject& obj)
 
         const auto vars = nczarrGroup["vars"].ToArray();
         // open first indexing variables
+        std::set<std::string> oSetIndexingArrayNames;
         for( const auto& var: vars )
         {
             const auto osVarName = var.ToString();
             if( IsValidName(osVarName) &&
-                m_oMapDimensions.find(osVarName) != m_oMapDimensions.end() )
+                m_oMapDimensions.find(osVarName) != m_oMapDimensions.end() &&
+                m_oMapMDArrays.find(osVarName) == m_oMapMDArrays.end() &&
+                oSetIndexingArrayNames.find(osVarName) == oSetIndexingArrayNames.end() )
             {
-                m_aosArrays.emplace_back(osVarName);
+                oSetIndexingArrayNames.insert(osVarName);
                 OpenMDArray(osVarName);
             }
         }
 
         // add regular arrays
+        std::set<std::string> oSetRegularArrayNames;
         for( const auto& var: vars )
         {
             const auto osVarName = var.ToString();
             if( IsValidName(osVarName) &&
-                m_oMapDimensions.find(osVarName) == m_oMapDimensions.end() )
+                m_oMapDimensions.find(osVarName) == m_oMapDimensions.end() &&
+                m_oMapMDArrays.find(osVarName) == m_oMapMDArrays.end() &&
+                oSetRegularArrayNames.find(osVarName) == oSetRegularArrayNames.end() )
             {
+                oSetRegularArrayNames.insert(osVarName);
                 m_aosArrays.emplace_back(osVarName);
             }
         }
 
         // Finally list groups
+        std::set<std::string> oSetGroups;
         const auto groups = nczarrGroup["groups"].ToArray();
         for( const auto& group: groups )
         {
             const auto osGroupName = group.ToString();
-            if( IsValidName(osGroupName) )
+            if( IsValidName(osGroupName) &&
+                oSetGroups.find(osGroupName) == oSetGroups.end() )
             {
+                oSetGroups.insert(osGroupName);
                 m_aosGroups.emplace_back(osGroupName);
             }
         }
@@ -902,6 +912,18 @@ static CPLJSONObject FillDTypeElts(const GDALExtendedDataType& oDataType,
                     dtype.Set(dummy, "<i4");
                     break;
                 }
+                case GDT_UInt64:
+                {
+                    elt.nativeType = DtypeElt::NativeType::UNSIGNED_INT;
+                    dtype.Set(dummy, "<u8");
+                    break;
+                }
+                case GDT_Int64:
+                {
+                    elt.nativeType = DtypeElt::NativeType::SIGNED_INT;
+                    dtype.Set(dummy, "<i8");
+                    break;
+                }
                 case GDT_Float32:
                 {
                     elt.nativeType = DtypeElt::NativeType::IEEEFP;
@@ -945,7 +967,7 @@ static CPLJSONObject FillDTypeElts(const GDALExtendedDataType& oDataType,
                 }
                 case GDT_TypeCount:
                 {
-                    static_assert(GDT_TypeCount == GDT_CFloat64 + 1, "GDT_TypeCount == GDT_CFloat64 + 1");
+                    static_assert(GDT_TypeCount == GDT_Int64 + 1, "GDT_TypeCount == GDT_Int64 + 1");
                     break;
                 }
             }
@@ -1232,6 +1254,8 @@ std::shared_ptr<GDALMDArray> ZarrGroupV2::CreateMDArray(
                 case GDT_Int16: oFilter.Add("dtype", "<i2"); break;
                 case GDT_UInt32: oFilter.Add("dtype", "<u4"); break;
                 case GDT_Int32: oFilter.Add("dtype", "<i4"); break;
+                case GDT_UInt64: oFilter.Add("dtype", "<u8"); break;
+                case GDT_Int64: oFilter.Add("dtype", "<i8"); break;
                 case GDT_Float32: oFilter.Add("dtype", "<f4"); break;
                 case GDT_Float64: oFilter.Add("dtype", "<f8"); break;
                 case GDT_CInt16: oFilter.Add("dtype", "<i2"); break;
@@ -1794,8 +1818,8 @@ void ZarrSharedResource::SetZMetadataItem(const std::string& osFilename,
 {
     if( m_bZMetadataEnabled )
     {
-        const std::string osNormalizedFilename =
-                                CPLString(osFilename).replaceAll('\\', '/');
+        CPLString osNormalizedFilename(osFilename);
+        osNormalizedFilename.replaceAll('\\', '/');
         CPLAssert( STARTS_WITH(osNormalizedFilename.c_str(),
                                (m_osRootDirectoryName + '/').c_str()) );
         m_bZMetadataModified = true;
