@@ -33,7 +33,7 @@
 
 #include <cmath>
 
-CPL_CVSID("$Id: zmapdataset.cpp 2dc350b1c738a002da1ec5c4cbe3e42fec4069dc 2021-03-04 16:15:13 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -207,7 +207,7 @@ ZMapDataset::ZMapDataset() :
 ZMapDataset::~ZMapDataset()
 
 {
-    FlushCache();
+    FlushCache(true);
     if (fp)
         VSIFCloseL(fp);
 }
@@ -281,9 +281,6 @@ GDALDataset *ZMapDataset::Open( GDALOpenInfo * poOpenInfo )
     if (!Identify(poOpenInfo) || poOpenInfo->fpL == nullptr )
         return nullptr;
 
-    if( !GDALIsDriverDeprecatedForGDAL35StillEnabled("ZMAP") )
-        return nullptr;
-
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
@@ -538,9 +535,6 @@ GDALDataset* ZMapDataset::CreateCopy( const char * pszFilename,
                                       GDALProgressFunc pfnProgress,
                                       void * pProgressData )
 {
-    if( !GDALIsDriverDeprecatedForGDAL35StillEnabled("ZMAP") )
-        return nullptr;
-
 /* -------------------------------------------------------------------- */
 /*      Some some rudimentary checks                                    */
 /* -------------------------------------------------------------------- */
