@@ -46,7 +46,7 @@ constexpr int SUBBLOCK_SIZE = 64;
 #define TO_SUBBLOCK(x) ((x) >> 6)
 #define WITHIN_SUBBLOCK(x) ((x) & 0x3f)
 
-CPL_CVSID("$Id: gdalarraybandblockcache.cpp e3511844a11959bd81f5956a1e7e2de96900e7ac 2019-04-25 11:53:15 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /* ******************************************************************** */
 /*                        GDALArrayBandBlockCache                       */
@@ -106,7 +106,7 @@ GDALArrayBandBlockCache::GDALArrayBandBlockCache(GDALRasterBand* poBandIn) :
 
 GDALArrayBandBlockCache::~GDALArrayBandBlockCache()
 {
-    FlushCache();
+    GDALArrayBandBlockCache::FlushCache();
 
     if( !bSubBlockingActive )
         CPLFree( u.papoBlocks );
@@ -322,7 +322,7 @@ CPLErr GDALArrayBandBlockCache::FlushCache()
 
     EndDirtyBlockFlushingLog();
 
-    WaitKeepAliveCounter();
+    WaitCompletionPendingTasks();
 
     return( eGlobalErr );
 }
@@ -429,7 +429,7 @@ CPLErr GDALArrayBandBlockCache::FlushBlock( int nXBlockOff, int nYBlockOff,
 
     CPLErr eErr = CE_None;
 
-    if( bWriteDirtyBlock && poBlock->GetDirty() )
+    if( m_bWriteDirtyBlocks && bWriteDirtyBlock && poBlock->GetDirty() )
     {
         UpdateDirtyBlockFlushingLog();
 
