@@ -3,11 +3,11 @@
  * Project:  GDAL
  * Purpose:  GDALGeorefPamDataset with helper to read georeferencing and other
  *           metadata from JP2Boxes
- * Author:   Even Rouault <even dot rouault at mines-paris dot org>
+ * Author:   Even Rouault <even dot rouault at spatialys.com>
  *
  ******************************************************************************
  * Copyright (c) 2002, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -52,12 +52,7 @@
 /*                     GDALJP2AbstractDataset()                         */
 /************************************************************************/
 
-GDALJP2AbstractDataset::GDALJP2AbstractDataset() :
-    pszWldFilename(nullptr),
-    poMemDS(nullptr),
-    papszMetadataFiles(nullptr),
-    m_nWORLDFILEIndex(-1)
-{}
+GDALJP2AbstractDataset::GDALJP2AbstractDataset() = default;
 
 /************************************************************************/
 /*                     ~GDALJP2AbstractDataset()                        */
@@ -66,7 +61,7 @@ GDALJP2AbstractDataset::GDALJP2AbstractDataset() :
 GDALJP2AbstractDataset::~GDALJP2AbstractDataset()
 {
     CPLFree(pszWldFilename);
-    CloseDependentDatasets();
+    GDALJP2AbstractDataset::CloseDependentDatasets();
     CSLDestroy(papszMetadataFiles);
 }
 
@@ -107,6 +102,7 @@ void GDALJP2AbstractDataset::LoadJP2Metadata(
         pszGeorefSourcesOption :
         CPLGetConfigOption("GDAL_GEOREF_SOURCES", "PAM,INTERNAL,WORLDFILE");
     size_t nInternalIdx = osGeorefSources.ifind("INTERNAL");
+    // coverity[tainted_data]
     if( nInternalIdx != std::string::npos &&
         (nInternalIdx == 0 || osGeorefSources[nInternalIdx-1] == ',') &&
         (nInternalIdx + strlen("INTERNAL") == osGeorefSources.size() ||
@@ -155,9 +151,8 @@ void GDALJP2AbstractDataset::LoadJP2Metadata(
         (nGMLJP2Index >= 0 || nGEOJP2Index >= 0 || nMSIGIndex >= 0 ||
          m_nWORLDFILEIndex >= 0) )
     {
-        CPLFree(pszProjection);
-        pszProjection = CPLStrdup(oJP2Geo.pszProjection);
-        if( strlen(pszProjection) > 0 )
+        m_oSRS = oJP2Geo.m_oSRS;
+        if( !m_oSRS.IsEmpty() )
             m_nProjectionGeorefSrcIndex = nIndexUsed;
         bGeoTransformValid = CPL_TO_BOOL( oJP2Geo.bHaveGeoTransform );
         if( bGeoTransformValid )
@@ -328,10 +323,13 @@ char **GDALJP2AbstractDataset::GetFileList()
 
     if( pszWldFilename != nullptr &&
         m_nGeoTransformGeorefSrcIndex == m_nWORLDFILEIndex &&
+        GDALCanReliablyUseSiblingFileList(pszWldFilename) &&
         CSLFindString( papszFileList, pszWldFilename ) == -1 )
     {
         double l_adfGeoTransform[6];
         GetGeoTransform(l_adfGeoTransform);
+        // GetGeoTransform() can modify m_nGeoTransformGeorefSrcIndex
+        // cppcheck-suppress knownConditionTrueFalse
         if( m_nGeoTransformGeorefSrcIndex == m_nWORLDFILEIndex )
         {
             papszFileList = CSLAddString( papszFileList, pszWldFilename );
