@@ -41,8 +41,9 @@
 #include "cpl_error.h"
 #include "cpl_minixml.h"
 #include "cpl_string.h"
+#include "cpl_time.h"
 
-CPL_CVSID("$Id: reader_pleiades.cpp 661dc7641bb5f9a51bf2cb075fb92dd7fa97876b 2020-07-16 11:36:24 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /**
  * GDALMDReaderPleiades()
@@ -275,9 +276,10 @@ void GDALMDReaderPleiades::LoadMetadata()
             pszTime = "00:00:00.0Z";
 
         char buffer[80];
-        time_t timeMid = GetAcquisitionTimeFromString(CPLSPrintf( "%sT%s",
+        GIntBig timeMid = GetAcquisitionTimeFromString(CPLSPrintf( "%sT%s",
                                                      pszDate, pszTime));
-        strftime (buffer, 80, MD_DATETIMEFORMAT, localtime(&timeMid));
+        struct tm tmBuf;
+        strftime (buffer, 80, MD_DATETIMEFORMAT, CPLUnixTimeToYMDHMS(timeMid, &tmBuf));
         m_papszIMAGERYMD = CSLAddNameValue(m_papszIMAGERYMD,
                                            MD_NAME_ACQDATETIME, buffer);
     }
@@ -327,6 +329,15 @@ char** GDALMDReaderPleiades::LoadRPCXmlFile()
     {
         papszRawRPCList = ReadXMLToList(pGRFMNode->psChild, papszRawRPCList);
     }
+    else
+    {
+        pGRFMNode = CPLSearchXMLNode(pNode, "=Rational_Function_Model");
+
+        if(pGRFMNode != nullptr)
+        {
+            papszRawRPCList = ReadXMLToList(pGRFMNode->psChild, papszRawRPCList);
+        }
+    }
 
     if( nullptr == papszRawRPCList )
     {
@@ -334,6 +345,24 @@ char** GDALMDReaderPleiades::LoadRPCXmlFile()
         return nullptr;
     }
 
+    // search Image to Ground Validity (since DIMAP v3)
+    CPLXMLNode* pValidityNode = CPLSearchXMLNode(pNode, "=ImagetoGround_Validity_Domain");
+
+    double firstCol = 1.0;
+    if( pValidityNode != nullptr )
+    {
+        char** papszValidity = ReadXMLToList(pValidityNode->psChild, nullptr);
+        if( papszValidity != nullptr )
+        {
+            const char* pszFirstCol = CSLFetchNameValue(papszValidity, "FIRST_COL");
+            if( pszFirstCol != nullptr )
+            {
+                firstCol = CPLAtofM(pszFirstCol);
+            }
+        }
+        CSLDestroy(papszValidity);
+    }
+
     // If we are not the top-left tile, then we must shift LINE_OFF and SAMP_OFF
     int nLineOffShift = 0;
     int nPixelOffShift = 0;
@@ -373,14 +402,17 @@ char** GDALMDReaderPleiades::LoadRPCXmlFile()
     char** papszRPB = nullptr;
     for( int i = 0; apszRPBMap[i] != nullptr; i += 2 )
     {
-        // Pleiades RPCs use "center of upper left pixel is 1,1" convention, convert to
+        const char *pszValue = CSLFetchNameValue(papszRawRPCList,
+                                                 apszRPBMap[i + 1]);
+        // Deprecated : Pleiades RPCs use "center of upper left pixel is 1,1" convention, convert to
         // Digital globe convention of "center of upper left pixel is 0,0".
-        if (i == 0 || i == 2)
+
+        // Since DIMAP v3, the center of upper left pixel can be 0, 0. So now it is dynamically loaded
+        // from the DIMAP.
+        if ((i == 0 || i == 2) && pszValue)
         {
             CPLString osField;
-            const char *pszOffset = CSLFetchNameValue(papszRawRPCList,
-                                                    apszRPBMap[i + 1]);
-            double dfVal = CPLAtofM( pszOffset ) -1.0 ;
+            double dfVal = CPLAtofM( pszValue ) - firstCol ;
             if( i == 0 )
                 dfVal += nLineOffShift;
             else
@@ -390,9 +422,7 @@ char** GDALMDReaderPleiades::LoadRPCXmlFile()
         }
         else
         {
-            papszRPB = CSLAddNameValue(papszRPB, apszRPBMap[i],
-                                    CSLFetchNameValue(papszRawRPCList,
-                                                        apszRPBMap[i + 1]));
+            papszRPB = CSLAddNameValue(papszRPB, apszRPBMap[i], pszValue);
         }
     }
 
@@ -408,8 +438,16 @@ char** GDALMDReaderPleiades::LoadRPCXmlFile()
             // supplies geographic coordinates (lon, lat) and an altitude (alt)"""
             const char* pszValue = CSLFetchNameValue(papszRawRPCList,
                  CPLSPrintf("Inverse_Model.%s_%d", apszRPCTXT20ValItems[i], j));
-            if(nullptr != pszValue)
+            if(nullptr != pszValue){
                 value = value + " " + CPLString(pszValue);
+            }
+            else {
+                 pszValue = CSLFetchNameValue(papszRawRPCList,
+                 CPLSPrintf("GroundtoImage_Values.%s_%d", apszRPCTXT20ValItems[i], j));
+                 if(nullptr != pszValue){
+                    value = value + " " + CPLString(pszValue);
+                 }
+            }
         }
         papszRPB = CSLAddNameValue(papszRPB, apszRPCTXT20ValItems[i], value);
     }
