@@ -31,7 +31,7 @@
 #include "gnmfile.h"
 #include "gnm_priv.h"
 
-CPL_CVSID("$Id: gnmfilenetwork.cpp 4b92a13c594df8c488d24a2299c64a31bb14668b 2019-03-23 13:03:10 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 GNMFileNetwork::GNMFileNetwork() : GNMGenericNetwork()
 {
@@ -42,7 +42,7 @@ GNMFileNetwork::GNMFileNetwork() : GNMGenericNetwork()
 
 GNMFileNetwork::~GNMFileNetwork()
 {
-    FlushCache();
+    FlushCache(true);
 
     for (std::map<OGRLayer*, GDALDataset*>::iterator
             it = m_mpLayerDatasetMap.begin();
