@@ -38,6 +38,9 @@
 #ifndef _WIN32
 #include <sys/types.h>
 #include <unistd.h>
+#if defined(HAVE_PTHREAD_ATFORK)
+#include <pthread.h>
+#endif
 #endif
 
 #include <mutex>
@@ -69,22 +72,48 @@ static unsigned g_auxDbPathsGenerationCounter = 0;
 static std::mutex g_oSearchPathMutex;
 static CPLStringList g_aosSearchpaths;
 static CPLStringList g_aosAuxDbPaths;
+#if PROJ_VERSION_MAJOR >= 7
+static int g_projNetworkEnabled = -1;
+static unsigned g_projNetworkEnabledGenerationCounter = 0;
+#endif
+
+#if !defined(_WIN32) && defined(HAVE_PTHREAD_ATFORK)
+static bool g_bForkOccured = false;
+static void ForkOccured(void)
+{
+    g_bForkOccured = true;
+}
+#endif
 
 struct OSRPJContextHolder
 {
     unsigned searchPathGenerationCounter = 0;
     unsigned auxDbPathsGenerationCounter = 0;
+#if PROJ_VERSION_MAJOR >= 7
+    unsigned projNetworkEnabledGenerationCounter = 0;
+#endif
     PJ_CONTEXT* context = nullptr;
     OSRProjTLSCache oCache{};
 #if !defined(_WIN32)
+#if !defined(HAVE_PTHREAD_ATFORK)
     pid_t curpid = 0;
+#endif
 #if defined(SIMUL_OLD_PROJ6) || !(PROJ_VERSION_MAJOR > 6 || PROJ_VERSION_MINOR >= 2)
     std::vector<PJ_CONTEXT*> oldcontexts{};
 #endif
 #endif
 
 #if !defined(_WIN32)
-    OSRPJContextHolder(): curpid(getpid()) { init(); }
+    OSRPJContextHolder()
+#if !defined(HAVE_PTHREAD_ATFORK)
+        : curpid(getpid())
+#endif
+    {
+#if HAVE_PTHREAD_ATFORK
+        pthread_atfork(nullptr, nullptr, ForkOccured);
+#endif
+        init();
+    }
 #else
     OSRPJContextHolder() { init(); }
 #endif
@@ -171,11 +200,19 @@ static OSRPJContextHolder& GetProjTLSContextHolder()
     // Detect if we are now running in a child process created by fork()
     // In that situation we must make sure *not* to use the same underlying
     // file open descriptor to the sqlite3 database, since seeks&reads in one
+#if defined(HAVE_PTHREAD_ATFORK)
+    if( g_bForkOccured )
+#else
     // of the parent or child will affect the other end.
     const pid_t curpid = getpid();
     if( curpid != l_projContext.curpid )
+#endif
     {
+#if defined(HAVE_PTHREAD_ATFORK)
+        g_bForkOccured = false;
+#else
         l_projContext.curpid = curpid;
+#endif
 #if defined(SIMUL_OLD_PROJ6) || !(PROJ_VERSION_MAJOR > 6 || PROJ_VERSION_MINOR >= 2)
         // PROJ < 6.2 ? Recreate new context
         l_projContext.oldcontexts.push_back(l_projContext.context);
@@ -227,6 +264,15 @@ PJ_CONTEXT* OSRGetProjTLSContext()
             proj_context_set_database_path(l_projContext.context, oMainPath.c_str(),
                                            g_aosAuxDbPaths.List(), nullptr);
         }
+#if PROJ_VERSION_MAJOR >= 7
+        if( l_projContext.projNetworkEnabledGenerationCounter !=
+                                        g_projNetworkEnabledGenerationCounter )
+        {
+            l_projContext.projNetworkEnabledGenerationCounter =
+                                        g_projNetworkEnabledGenerationCounter;
+            proj_context_set_enable_network(l_projContext.context, g_projNetworkEnabled);
+        }
+#endif
     }
     return l_projContext.context;
 }
@@ -254,16 +300,13 @@ void OSRProjTLSCache::clear()
 
 PJ* OSRProjTLSCache::GetPJForEPSGCode(int nCode, bool bUseNonDeprecated, bool bAddTOWGS84)
 {
-    try
+    std::shared_ptr<PJ> cached;
+    const EPSGCacheKey key(nCode, bUseNonDeprecated, bAddTOWGS84);
+    if( m_oCacheEPSG.tryGet(key, cached) )
     {
-        const EPSGCacheKey key(nCode, bUseNonDeprecated, bAddTOWGS84);
-        const auto& cached = m_oCacheEPSG.get(key);
         return proj_clone(OSRGetProjTLSContext(), cached.get());
     }
-    catch( const lru11::KeyNotFound& )
-    {
-        return nullptr;
-    }
+    return nullptr;
 }
 
 void OSRProjTLSCache::CachePJForEPSGCode(int nCode, bool bUseNonDeprecated, bool bAddTOWGS84, PJ* pj)
@@ -275,15 +318,12 @@ void OSRProjTLSCache::CachePJForEPSGCode(int nCode, bool bUseNonDeprecated, bool
 
 PJ* OSRProjTLSCache::GetPJForWKT(const std::string& wkt)
 {
-    try
+    std::shared_ptr<PJ> cached;
+    if( m_oCacheWKT.tryGet(wkt, cached) )
     {
-        const auto& cached = m_oCacheWKT.get(wkt);
         return proj_clone(OSRGetProjTLSContext(), cached.get());
     }
-    catch( const lru11::KeyNotFound& )
-    {
-        return nullptr;
-    }
+    return nullptr;
 }
 
 void OSRProjTLSCache::CachePJForWKT(const std::string& wkt, PJ* pj)
@@ -308,7 +348,7 @@ void OSRCleanupTLSContext()
 /************************************************************************/
 
 /** \brief Set the search path(s) for PROJ resource files.
- * 
+ *
  * @param papszPaths NULL terminated list of directory paths.
  * @since GDAL 3.0
  */
@@ -331,7 +371,7 @@ void OSRSetPROJSearchPaths( const char* const * papszPaths )
 char** OSRGetPROJSearchPaths()
 {
     std::lock_guard<std::mutex> oLock(g_oSearchPathMutex);
-    if( g_searchPathGenerationCounter > 0 )
+    if( g_searchPathGenerationCounter > 0 && !g_aosSearchpaths.empty() )
     {
         return CSLDuplicate(g_aosSearchpaths.List());
     }
@@ -351,7 +391,7 @@ char** OSRGetPROJSearchPaths()
 /************************************************************************/
 
 /** \brief Set list of PROJ auxiliary database filenames.
- * 
+ *
  * @param papszAux NULL-terminated list of auxiliary database filenames, or NULL
  * @since GDAL 3.3
  *
@@ -369,7 +409,7 @@ void OSRSetPROJAuxDbPaths( const char* const * papszAux )
 /************************************************************************/
 
 /** \brief Get PROJ auxiliary database filenames.
- * 
+ *
  * @return NULL terminated list of PROJ auxiliary database filenames. To be freed with CSLDestroy()
  * @since GDAL 3.3.0
  *
@@ -383,6 +423,63 @@ char** OSRGetPROJAuxDbPaths( void )
     return CSLDuplicate(g_aosAuxDbPaths.List());
 }
 
+/************************************************************************/
+/*                       OSRSetPROJEnableNetwork()                      */
+/************************************************************************/
+
+/** \brief Enable or disable PROJ networking capabilities.
+ *
+ * @param enabled Set to TRUE to enable networking capabilities.
+ * @since GDAL 3.4 and PROJ 7
+ *
+ * @see OSRGetPROJEnableNetwork, proj_context_set_enable_network
+ */
+void OSRSetPROJEnableNetwork( int enabled )
+{
+#if PROJ_VERSION_MAJOR >= 7
+    std::lock_guard<std::mutex> oLock(g_oSearchPathMutex);
+    if( g_projNetworkEnabled != enabled )
+    {
+        g_projNetworkEnabled = enabled;
+        g_projNetworkEnabledGenerationCounter ++;
+    }
+#else
+    if( enabled )
+    {
+        CPLError(CE_Failure, CPLE_NotSupported,
+                 "OSRSetPROJEnableNetwork() requires PROJ >= 7");
+    }
+#endif
+}
+
+/************************************************************************/
+/*                        OSRGetPROJEnableNetwork()                     */
+/************************************************************************/
+
+/** \brief Get whether PROJ networking capabilities are enabled.
+ *
+ * @return TRUE if PROJ networking capabilities are enabled.
+ * @since GDAL 3.4 and PROJ 7
+ *
+ * @see OSRSetPROJEnableNetwork, proj_context_is_network_enabled
+ */
+int OSRGetPROJEnableNetwork( void )
+{
+#if PROJ_VERSION_MAJOR >= 7
+    std::lock_guard<std::mutex> oLock(g_oSearchPathMutex);
+    if( g_projNetworkEnabled < 0 )
+    {
+        g_oSearchPathMutex.unlock();
+        const int ret = proj_context_is_network_enabled(OSRGetProjTLSContext());
+        g_oSearchPathMutex.lock();
+        g_projNetworkEnabled = ret;
+    }
+    return g_projNetworkEnabled;
+#else
+    return FALSE;
+#endif
+}
+
 /************************************************************************/
 /*                         OSRGetPROJVersion()                          */
 /************************************************************************/
