@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 1999,  Les Technologies SoftMap Inc.
- * Copyright (c) 2009-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2009-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,6 +31,7 @@
 #include "ogr_feature.h"
 
 #include <algorithm>
+#include <cassert>
 #include <cstring>
 
 #include "cpl_conv.h"
@@ -41,7 +42,7 @@
 #include "ogr_p.h"
 #include "ograpispy.h"
 
-CPL_CVSID("$Id: ogrfeaturedefn.cpp 555e540bc97e0201cd3ed57f7906323d476c13fb 2018-04-04 19:43:04 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                           OGRFeatureDefn()                           */
@@ -60,19 +61,10 @@ CPL_CVSID("$Id: ogrfeaturedefn.cpp 555e540bc97e0201cd3ed57f7906323d476c13fb 2018
  * need to be unique.
  */
 
-OGRFeatureDefn::OGRFeatureDefn( const char * pszName ) :
-    nRefCount(0),
-    nFieldCount(0),
-    papoFieldDefn(nullptr),
-    nGeomFieldCount(1),
-    papoGeomFieldDefn(nullptr),
-    pszFeatureClassName(nullptr),
-    bIgnoreStyle(FALSE)
+OGRFeatureDefn::OGRFeatureDefn( const char * pszName )
 {
     pszFeatureClassName = CPLStrdup( pszName );
-    papoGeomFieldDefn =
-        static_cast<OGRGeomFieldDefn**>(CPLMalloc(sizeof(OGRGeomFieldDefn*)));
-    papoGeomFieldDefn[0] = new OGRGeomFieldDefn("", wkbUnknown);
+    apoGeomFieldDefn.emplace_back(cpl::make_unique<OGRGeomFieldDefn>("", wkbUnknown));
 }
 
 /************************************************************************/
@@ -113,20 +105,6 @@ OGRFeatureDefn::~OGRFeatureDefn()
     }
 
     CPLFree( pszFeatureClassName );
-
-    for( int i = 0; i < nFieldCount; i++ )
-    {
-        delete papoFieldDefn[i];
-    }
-
-    CPLFree( papoFieldDefn );
-
-    for( int i = 0; i < nGeomFieldCount; i++ )
-    {
-        delete papoGeomFieldDefn[i];
-    }
-
-    CPLFree( papoGeomFieldDefn );
 }
 
 /************************************************************************/
@@ -202,15 +180,21 @@ OGRFeatureDefn *OGRFeatureDefn::Clone() const
 {
     OGRFeatureDefn *poCopy = new OGRFeatureDefn( GetName() );
 
-    GetFieldCount();
-    for( int i = 0; i < nFieldCount; i++ )
-        poCopy->AddFieldDefn( const_cast<OGRFieldDefn*>(GetFieldDefn( i )) );
+    {
+        const int nFieldCount = GetFieldCount();
+        poCopy->apoFieldDefn.reserve(nFieldCount);
+        for( int i = 0; i < nFieldCount; i++ )
+            poCopy->AddFieldDefn( GetFieldDefn( i ) );
+    }
 
-    // Remove the default geometry field created instantiation.
-    poCopy->DeleteGeomFieldDefn(0);
-    GetGeomFieldCount();
-    for( int i = 0; i < nGeomFieldCount; i++ )
-        poCopy->AddGeomFieldDefn( const_cast<OGRGeomFieldDefn*>(GetGeomFieldDefn( i )) );
+    {
+        // Remove the default geometry field created instantiation.
+        poCopy->DeleteGeomFieldDefn(0);
+        const int nGeomFieldCount = GetGeomFieldCount();
+        poCopy->apoGeomFieldDefn.reserve(nGeomFieldCount);
+        for( int i = 0; i < nGeomFieldCount; i++ )
+            poCopy->AddGeomFieldDefn( GetGeomFieldDefn( i ) );
+    }
 
     return poCopy;
 }
@@ -284,7 +268,7 @@ const char *OGR_FD_GetName( OGRFeatureDefnH hDefn )
 
 int OGRFeatureDefn::GetFieldCount() const
 {
-    return nFieldCount;
+    return static_cast<int>(apoFieldDefn.size());
 }
 
 /************************************************************************/
@@ -335,7 +319,7 @@ OGRFieldDefn *OGRFeatureDefn::GetFieldDefn( int iField )
         return nullptr;
     }
 
-    return papoFieldDefn[iField];
+    return apoFieldDefn[iField].get();
 }
 
 /**
@@ -360,7 +344,7 @@ const OGRFieldDefn *OGRFeatureDefn::GetFieldDefn( int iField ) const
         return nullptr;
     }
 
-    return papoFieldDefn[iField];
+    return apoFieldDefn[iField].get();
 }
 
 /************************************************************************/
@@ -377,7 +361,7 @@ const OGRFieldDefn *OGRFeatureDefn::GetFieldDefn( int iField ) const
  * from.
  * @param iField the field to fetch, between 0 and GetFieldCount()-1.
  *
- * @return an handle to an internal field definition object or NULL if invalid
+ * @return a handle to an internal field definition object or NULL if invalid
  * index.  This object should not be modified or freed by the application.
  */
 
@@ -404,9 +388,7 @@ OGRFieldDefnH OGR_FD_GetFieldDefn( OGRFeatureDefnH hDefn, int iField )
 
 void OGRFeatureDefn::ReserveSpaceForFields(int nFieldCountIn)
 {
-    papoFieldDefn = static_cast<OGRFieldDefn **>(
-        CPLRealloc(papoFieldDefn,
-                   sizeof(void *) * std::max(nFieldCount, nFieldCountIn)));
+    apoFieldDefn.reserve(nFieldCountIn);
 }
 //! @endcond
 
@@ -429,15 +411,10 @@ void OGRFeatureDefn::ReserveSpaceForFields(int nFieldCountIn)
  * @param poNewDefn the definition of the new field.
  */
 
-void OGRFeatureDefn::AddFieldDefn( OGRFieldDefn * poNewDefn )
+void OGRFeatureDefn::AddFieldDefn( const OGRFieldDefn * poNewDefn )
 
 {
-    GetFieldCount();
-    papoFieldDefn = static_cast<OGRFieldDefn **>(
-        CPLRealloc(papoFieldDefn, sizeof(void *) * (nFieldCount + 1)));
-
-    papoFieldDefn[nFieldCount] = new OGRFieldDefn( poNewDefn );
-    nFieldCount++;
+    apoFieldDefn.emplace_back(cpl::make_unique<OGRFieldDefn>(poNewDefn));
 }
 
 /************************************************************************/
@@ -494,18 +471,7 @@ OGRErr OGRFeatureDefn::DeleteFieldDefn( int iField )
     if( iField < 0 || iField >= GetFieldCount() )
         return OGRERR_FAILURE;
 
-    delete papoFieldDefn[iField];
-    papoFieldDefn[iField] = nullptr;
-
-    if( iField < nFieldCount - 1 )
-    {
-        memmove(papoFieldDefn + iField,
-                papoFieldDefn + iField + 1,
-                (nFieldCount - 1 - iField) * sizeof(void *));
-    }
-
-    nFieldCount--;
-
+    apoFieldDefn.erase(apoFieldDefn.begin() + iField);
     return OGRERR_NONE;
 }
 
@@ -559,27 +525,23 @@ OGRErr OGR_FD_DeleteFieldDefn( OGRFeatureDefnH hDefn, int iField )
  * @since OGR 1.9.0
  */
 
-OGRErr OGRFeatureDefn::ReorderFieldDefns( int* panMap )
+OGRErr OGRFeatureDefn::ReorderFieldDefns( const int* panMap )
 
 {
-    if( GetFieldCount() == 0 )
+    const int nFieldCount = GetFieldCount();
+    if( nFieldCount == 0 )
         return OGRERR_NONE;
 
     const OGRErr eErr = OGRCheckPermutation(panMap, nFieldCount);
     if( eErr != OGRERR_NONE )
         return eErr;
 
-    OGRFieldDefn** papoFieldDefnNew = static_cast<OGRFieldDefn**>(
-        CPLMalloc(sizeof(OGRFieldDefn*) * nFieldCount));
-
+    std::vector<std::unique_ptr<OGRFieldDefn>> apoFieldDefnNew(nFieldCount);
     for( int i = 0; i < nFieldCount; i++ )
     {
-        papoFieldDefnNew[i] = papoFieldDefn[panMap[i]];
+        apoFieldDefnNew[i] = std::move(apoFieldDefn[panMap[i]]);
     }
-
-    CPLFree(papoFieldDefn);
-    papoFieldDefn = papoFieldDefnNew;
-
+    apoFieldDefn = std::move(apoFieldDefnNew);
     return OGRERR_NONE;
 }
 
@@ -608,7 +570,7 @@ OGRErr OGRFeatureDefn::ReorderFieldDefns( int* panMap )
  * @since OGR 2.1.0
  */
 
-OGRErr OGR_FD_ReorderFieldDefns( OGRFeatureDefnH hDefn, int* panMap )
+OGRErr OGR_FD_ReorderFieldDefns( OGRFeatureDefnH hDefn, const int* panMap )
 
 {
     return OGRFeatureDefn::FromHandle(hDefn)->ReorderFieldDefns(panMap);
@@ -630,7 +592,7 @@ OGRErr OGR_FD_ReorderFieldDefns( OGRFeatureDefnH hDefn, int* panMap )
  */
 int OGRFeatureDefn::GetGeomFieldCount() const
 {
-    return nGeomFieldCount;
+    return static_cast<int>(apoGeomFieldDefn.size());
 }
 
 /************************************************************************/
@@ -686,7 +648,7 @@ OGRGeomFieldDefn *OGRFeatureDefn::GetGeomFieldDefn( int iGeomField )
         return nullptr;
     }
 
-    return papoGeomFieldDefn[iGeomField];
+    return apoGeomFieldDefn[iGeomField].get();
 }
 
 /**
@@ -712,7 +674,7 @@ const OGRGeomFieldDefn *OGRFeatureDefn::GetGeomFieldDefn( int iGeomField ) const
         return nullptr;
     }
 
-    return papoGeomFieldDefn[iGeomField];
+    return apoGeomFieldDefn[iGeomField].get();
 }
 /************************************************************************/
 /*                      OGR_FD_GetGeomFieldDefn()                       */
@@ -729,7 +691,7 @@ const OGRGeomFieldDefn *OGRFeatureDefn::GetGeomFieldDefn( int iGeomField ) const
  * @param iGeomField the geometry field to fetch, between 0 and
  * GetGeomFieldCount() - 1.
  *
- * @return an handle to an internal field definition object or NULL if invalid
+ * @return a handle to an internal field definition object or NULL if invalid
  * index.  This object should not be modified or freed by the application.
  *
  * @since GDAL 1.11
@@ -773,21 +735,34 @@ OGRGeomFieldDefnH OGR_FD_GetGeomFieldDefn( OGRFeatureDefnH hDefn,
  * This method is the same as the C function OGR_FD_AddGeomFieldDefn().
  *
  * @param poNewDefn the definition of the new geometry field.
- * @param bCopy whether poNewDefn should be copied.
  *
  * @since GDAL 1.11
  */
 
-void OGRFeatureDefn::AddGeomFieldDefn( OGRGeomFieldDefn * poNewDefn,
-                                       int bCopy )
+void OGRFeatureDefn::AddGeomFieldDefn( const OGRGeomFieldDefn * poNewDefn )
 {
-    GetGeomFieldCount();
-    papoGeomFieldDefn = static_cast<OGRGeomFieldDefn **>(
-        CPLRealloc( papoGeomFieldDefn, sizeof(void*) * (nGeomFieldCount + 1) ));
+    apoGeomFieldDefn.emplace_back(cpl::make_unique<OGRGeomFieldDefn>(poNewDefn));
+}
 
-    papoGeomFieldDefn[nGeomFieldCount] = bCopy ?
-        new OGRGeomFieldDefn( poNewDefn ) : poNewDefn;
-    nGeomFieldCount++;
+/**
+ * \brief Add a new geometry field definition.
+ *
+ * To add a new geometry field definition to a layer definition, do not use this
+ * function directly, but use OGRLayer::CreateGeomField() instead.
+ *
+ * This method takes ownership of the passed geometry field definition.
+ *
+ * This method should only be called while there are no OGRFeature
+ * objects in existence based on this OGRFeatureDefn.
+ *
+ * @param poNewDefn the definition of the new geometry field.
+ *
+ * @since GDAL 3.4
+ */
+
+void OGRFeatureDefn::AddGeomFieldDefn( std::unique_ptr<OGRGeomFieldDefn>&& poNewDefn )
+{
+    apoGeomFieldDefn.emplace_back(std::move(poNewDefn));
 }
 
 /************************************************************************/
@@ -849,18 +824,7 @@ OGRErr OGRFeatureDefn::DeleteGeomFieldDefn( int iGeomField )
     if( iGeomField < 0 || iGeomField >= GetGeomFieldCount() )
         return OGRERR_FAILURE;
 
-    delete papoGeomFieldDefn[iGeomField];
-    papoGeomFieldDefn[iGeomField] = nullptr;
-
-    if( iGeomField < nGeomFieldCount - 1 )
-    {
-        memmove(papoGeomFieldDefn + iGeomField,
-                papoGeomFieldDefn + iGeomField + 1,
-                (nGeomFieldCount - 1 - iGeomField) * sizeof(void*));
-    }
-
-    nGeomFieldCount--;
-
+    apoGeomFieldDefn.erase(apoGeomFieldDefn.begin() + iGeomField);
     return OGRERR_NONE;
 }
 
@@ -915,7 +879,7 @@ OGRErr OGR_FD_DeleteGeomFieldDefn( OGRFeatureDefnH hDefn, int iGeomField )
 int OGRFeatureDefn::GetGeomFieldIndex( const char * pszGeomFieldName ) const
 
 {
-    GetGeomFieldCount();
+    const int nGeomFieldCount = GetGeomFieldCount();
     for( int i = 0; i < nGeomFieldCount; i++ )
     {
         const OGRGeomFieldDefn* poGFldDefn = GetGeomFieldDefn(i);
@@ -1047,9 +1011,10 @@ OGRwkbGeometryType OGR_FD_GetGeomType( OGRFeatureDefnH hDefn )
 void OGRFeatureDefn::SetGeomType( OGRwkbGeometryType eNewType )
 
 {
-    if( GetGeomFieldCount() > 0 )
+    const int nGeomFieldCount = GetGeomFieldCount();
+    if( nGeomFieldCount > 0 )
     {
-        if( GetGeomFieldCount() == 1 && eNewType == wkbNone )
+        if( nGeomFieldCount == 1 && eNewType == wkbNone )
             DeleteGeomFieldDefn(0);
         else
             GetGeomFieldDefn(0)->SetType(eNewType);
@@ -1217,7 +1182,7 @@ int OGR_FD_GetReferenceCount( OGRFeatureDefnH hDefn )
 int OGRFeatureDefn::GetFieldIndex( const char * pszFieldName ) const
 
 {
-    GetFieldCount();
+    const int nFieldCount = GetFieldCount();
     for( int i = 0; i < nFieldCount; i++ )
     {
         const OGRFieldDefn* poFDefn = GetFieldDefn(i);
@@ -1228,6 +1193,37 @@ int OGRFeatureDefn::GetFieldIndex( const char * pszFieldName ) const
     return -1;
 }
 
+/************************************************************************/
+/*                      GetFieldIndexCaseSensitive()                    */
+/************************************************************************/
+
+/**
+ * \brief Find field by name, in a case sensitive way.
+ *
+ * The field index of the first field matching the passed field name is returned.
+ *
+ * @param pszFieldName the field name to search for.
+ *
+ * @return the field index, or -1 if no match found.
+ */
+
+int OGRFeatureDefn::GetFieldIndexCaseSensitive( const char * pszFieldName ) const
+
+{
+    const int nFieldCount = GetFieldCount();
+    for( int i = 0; i < nFieldCount; i++ )
+    {
+        const OGRFieldDefn* poFDefn = GetFieldDefn(i);
+        if( poFDefn != nullptr &&
+            strcmp(pszFieldName, poFDefn->GetNameRef() ) == 0 )
+        {
+            return i;
+        }
+    }
+
+    return -1;
+}
+
 /************************************************************************/
 /*                        OGR_FD_GetFieldIndex()                        */
 /************************************************************************/
@@ -1420,7 +1416,7 @@ int OGR_FD_IsStyleIgnored( OGRFeatureDefnH hDefn )
 
 void OGR_FD_SetStyleIgnored( OGRFeatureDefnH hDefn, int bIgnore )
 {
-    OGRFeatureDefn::FromHandle(hDefn)->SetStyleIgnored(bIgnore);
+    OGRFeatureDefn::FromHandle(hDefn)->SetStyleIgnored(CPL_TO_BOOL(bIgnore));
 }
 
 /************************************************************************/
@@ -1463,9 +1459,11 @@ void OGRFeatureDefn::DestroyFeatureDefn( OGRFeatureDefn *poDefn )
 
 int OGRFeatureDefn::IsSame( const OGRFeatureDefn * poOtherFeatureDefn ) const
 {
+    const int nFieldCount = GetFieldCount();
+    const int nGeomFieldCount = GetGeomFieldCount();
     if( strcmp(GetName(), poOtherFeatureDefn->GetName()) == 0 &&
-        GetFieldCount() == poOtherFeatureDefn->GetFieldCount() &&
-        GetGeomFieldCount() == poOtherFeatureDefn->GetGeomFieldCount() )
+        nFieldCount == poOtherFeatureDefn->GetFieldCount() &&
+        nGeomFieldCount == poOtherFeatureDefn->GetGeomFieldCount() )
     {
         for( int i = 0; i < nFieldCount; i++ )
         {
@@ -1542,11 +1540,19 @@ std::vector<int> OGRFeatureDefn::ComputeMapForSetFrom( const OGRFeatureDefn* poS
 {
     std::map<CPLString, int> oMapNameToTargetFieldIndex;
     std::map<CPLString, int> oMapNameToTargetFieldIndexUC;
-    for( int i = 0; i < GetFieldCount(); i++ )
+    const int nFieldCount = GetFieldCount();
+    for( int i = 0; i < nFieldCount; i++ )
     {
-        const char* pszName = GetFieldDefn(i)->GetNameRef();
+#ifdef __MINGW64__
+        const OGRFieldDefn* poFldDefn = GetFieldDefn(i);
+        if( poFldDefn == nullptr ) continue;
+#else
+        const OGRFieldDefn* poFldDefn = CPLAssertNotNull(GetFieldDefn(i)); /* Make GCC-8 -Wnull-dereference happy */
+#endif
+        const char* pszName = poFldDefn->GetNameRef();
+
         // In the insane case where there are several matches, arbitrarily
-        // decide for the first one (preserve past behaviour)
+        // decide for the first one (preserve past behavior)
         if( oMapNameToTargetFieldIndex.find(pszName) ==
                                         oMapNameToTargetFieldIndex.end() )
         {
@@ -1554,20 +1560,34 @@ std::vector<int> OGRFeatureDefn::ComputeMapForSetFrom( const OGRFeatureDefn* poS
         }
     }
     std::vector<int> aoMapSrcToTargetIdx;
-    aoMapSrcToTargetIdx.resize(poSrcFDefn->GetFieldCount());
-    for( int i = 0; i < poSrcFDefn->GetFieldCount(); i++ )
+    const int nSrcFieldCount = poSrcFDefn->GetFieldCount();
+    aoMapSrcToTargetIdx.resize(nSrcFieldCount);
+    for( int i = 0; i < nSrcFieldCount; i++ )
     {
-        const char* pszSrcName = poSrcFDefn->GetFieldDefn(i)->GetNameRef();
+#ifdef __MINGW64__
+        const OGRFieldDefn* poSrcFldDefn = poSrcFDefn->GetFieldDefn(i);
+        if( poSrcFldDefn == nullptr ) continue;
+#else
+        const OGRFieldDefn* poSrcFldDefn = CPLAssertNotNull(poSrcFDefn->GetFieldDefn(i)); /* Make GCC-8 -Wnull-dereference happy */
+#endif
+        const char* pszSrcName = poSrcFldDefn->GetNameRef();
+
         auto oIter = oMapNameToTargetFieldIndex.find(pszSrcName);
         if( oIter == oMapNameToTargetFieldIndex.end() )
         {
             // Build case insensitive map only if needed
             if( oMapNameToTargetFieldIndexUC.empty() )
             {
-                for( int j = 0; j < GetFieldCount(); j++ )
+                for( int j = 0; j < nFieldCount; j++ )
                 {
+#ifdef __MINGW64__
+                    const OGRFieldDefn* poFldDefn = GetFieldDefn(j);
+                    if( poFldDefn == nullptr ) continue;
+#else
+                    const OGRFieldDefn* poFldDefn = CPLAssertNotNull(GetFieldDefn(j)); /* Make GCC-8 -Wnull-dereference happy */
+#endif
                     oMapNameToTargetFieldIndexUC[
-                        CPLString(GetFieldDefn(j)->GetNameRef()).toupper()] = j;
+                        CPLString(poFldDefn->GetNameRef()).toupper()] = j;
                 }
             }
             oIter = oMapNameToTargetFieldIndexUC.find(
