@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: avc_e00write.cpp 002b050d9a9ef403a732c1210784736ef97216d4 2018-04-09 21:34:55 +0200 Even Rouault $
+ * $Id$
  *
  * Name:     avc_e00write.c
  * Project:  Arc/Info vector coverage (AVC)  E00->BIN conversion library
@@ -472,6 +472,12 @@ static void _AVCE00WriteRenameTable(AVCTableDef *psTableDef,
     char szSysId[40], szUserId[40];
     int  i;
 
+    if( psTableDef == nullptr )
+    {
+        CPLAssert( false );
+        return;
+    }
+
     snprintf(szNewName, sizeof(szNewName), "%s", pszNewCoverName);
     for(i=0; szNewName[i] != '\0'; i++)
         szNewName[i] = (char) toupper(szNewName[i]);
@@ -703,7 +709,7 @@ void  _AVCE00WriteCloseCoverFile(AVCE00WritePtr psInfo)
      *----------------------------------------------------------------*/
     if (psInfo->eCurFileType == AVCFilePRJ)
     {
-        AVCBinWriteObject(psInfo->hFile, psInfo->hParseInfo->cur.papszPrj);
+        AVCBinWriteObject(psInfo->hFile, psInfo->hParseInfo->aosPrj.List());
     }
 
     AVCBinWriteClose(psInfo->hFile);
