@@ -33,7 +33,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogravclayer.cpp 9c004490af2959176b7d70d9bd67288e15019590 2018-03-03 16:28:26Z Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                           OGRAVCLayer()                           */
@@ -179,7 +179,7 @@ int OGRAVCLayer::SetupFeatureDefinition( const char *pszName )
     if( poFeatureDefn && poFeatureDefn->GetGeomFieldDefn(0) )
     {
         poFeatureDefn->GetGeomFieldDefn(0)->SetSpatialRef(
-            poDS->GetSpatialRef());
+            poDS->DSGetSpatialRef());
     }
 
     SetDescription( pszName );
