@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2004, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -47,13 +47,13 @@
 #include "ogreditablelayer.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: ogrcsvdatasource.cpp 981af424642a34de2baf939f0515beeba38593a5 2018-07-10 13:45:02 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                     OGRCSVEditableLayerSynchronizer                  */
 /************************************************************************/
 
-class OGRCSVEditableLayerSynchronizer : public IOGREditableLayerSynchronizer
+class OGRCSVEditableLayerSynchronizer final: public IOGREditableLayerSynchronizer
 {
     OGRCSVLayer *m_poCSVLayer;
     char        **m_papszOpenOptions;
@@ -112,6 +112,8 @@ OGRErr OGRCSVEditableLayerSynchronizer::EditableSyncToDisk(
     if( m_poCSVLayer->GetGeometryFormat() == OGR_CSV_GEOM_AS_WKT )
         poCSVTmpLayer->SetWriteGeometry(wkbNone, OGR_CSV_GEOM_AS_WKT, nullptr);
 
+    const bool bKeepGeomColmuns = CPLFetchBool(m_papszOpenOptions, "KEEP_GEOM_COLUMNS", true);
+
     OGRErr eErr = OGRERR_NONE;
     OGRFeatureDefn *poEditableFDefn = poEditableLayer->GetLayerDefn();
     for( int i = 0; eErr == OGRERR_NONE && i < poEditableFDefn->GetFieldCount();
@@ -121,8 +123,9 @@ OGRErr OGRCSVEditableLayerSynchronizer::EditableSyncToDisk(
         int iGeomFieldIdx = 0;
         if( (EQUAL(oFieldDefn.GetNameRef(), "WKT") &&
              (iGeomFieldIdx = poEditableFDefn->GetGeomFieldIndex("")) >= 0) ||
-            (iGeomFieldIdx = poEditableFDefn->GetGeomFieldIndex(
-                 oFieldDefn.GetNameRef())) >= 0 )
+            (bKeepGeomColmuns &&
+             (iGeomFieldIdx = poEditableFDefn->GetGeomFieldIndex(
+                 (std::string("geom_") + oFieldDefn.GetNameRef()).c_str())) >= 0) )
         {
             OGRGeomFieldDefn oGeomFieldDefn(
                 poEditableFDefn->GetGeomFieldDefn(iGeomFieldIdx));
@@ -137,7 +140,7 @@ OGRErr OGRCSVEditableLayerSynchronizer::EditableSyncToDisk(
     const bool bHasXY = !m_poCSVLayer->GetXField().empty() &&
                         !m_poCSVLayer->GetYField().empty();
     const bool bHasZ = !m_poCSVLayer->GetZField().empty();
-    if( bHasXY && !CPLFetchBool(m_papszOpenOptions, "KEEP_GEOM_COLUMNS", true) )
+    if( bHasXY && !bKeepGeomColmuns )
     {
         if( poCSVTmpLayer->GetLayerDefn()->GetFieldIndex(
                 m_poCSVLayer->GetXField()) < 0 )
@@ -297,7 +300,7 @@ OGRErr OGRCSVEditableLayerSynchronizer::EditableSyncToDisk(
 /*                        OGRCSVEditableLayer                           */
 /************************************************************************/
 
-class OGRCSVEditableLayer: public OGREditableLayer
+class OGRCSVEditableLayer final: public OGREditableLayer
 {
     std::set<CPLString> m_oSetFields;
 
@@ -751,14 +754,18 @@ bool OGRCSVDataSource::OpenTable( const char *pszFilename,
 
     // Read and parse a line.  Did we get multiple fields?
 
-    const char *pszLine = CPLReadLineL(fp);
-    if( pszLine == nullptr )
+    std::string osLine;
     {
-        VSIFCloseL(fp);
-        return false;
+        const char *pszLine = CPLReadLine2L(fp, OGR_CSV_MAX_LINE_SIZE, nullptr);
+        if( pszLine == nullptr )
+        {
+            VSIFCloseL(fp);
+            return false;
+        }
+        osLine = pszLine;
     }
-    char chDelimiter = CSVDetectSeperator(pszLine);
-    if( chDelimiter != '\t' && strchr(pszLine, '\t') != nullptr )
+    char chDelimiter = CSVDetectSeperator(osLine.c_str());
+    if( chDelimiter != '\t' && osLine.find('\t') != std::string::npos )
     {
         // Force the delimiter to be TAB for a .tsv file that has a tabulation
         // in its first line */
@@ -772,16 +779,26 @@ bool OGRCSVDataSource::OpenTable( const char *pszFilename,
                  nDontHonourStrings <= 1;
                  nDontHonourStrings++ )
             {
-                const bool bDontHonourStrings = CPL_TO_BOOL(nDontHonourStrings);
+                const bool bHonourStrings = !CPL_TO_BOOL(nDontHonourStrings);
                 // Read the first 2 lines to see if they have the same number
                 // of fields, if using tabulation.
                 VSIRewindL(fp);
                 char **papszTokens =
-                    OGRCSVReadParseLineL(fp, '\t', bDontHonourStrings);
+                    CSVReadParseLine3L(fp, OGR_CSV_MAX_LINE_SIZE, "\t",
+                                       bHonourStrings,
+                                       false, // bKeepLeadingAndClosingQuotes
+                                       false, // bMergeDelimiter
+                                       true // bSkipBOM
+                                      );
                 const int nTokens1 = CSLCount(papszTokens);
                 CSLDestroy(papszTokens);
                 papszTokens =
-                    OGRCSVReadParseLineL(fp, '\t', bDontHonourStrings);
+                    CSVReadParseLine3L(fp, OGR_CSV_MAX_LINE_SIZE, "\t",
+                                       bHonourStrings,
+                                       false, // bKeepLeadingAndClosingQuotes
+                                       false, // bMergeDelimiter
+                                       true // bSkipBOM
+                                      );
                 const int nTokens2 = CSLCount(papszTokens);
                 CSLDestroy(papszTokens);
                 if( nTokens1 >= 2 && nTokens1 == nTokens2 )
@@ -819,10 +836,19 @@ bool OGRCSVDataSource::OpenTable( const char *pszFilename,
 #endif
 
     // GNIS specific.
-    if( pszGeonamesGeomFieldPrefix != nullptr && strchr(pszLine, '|') != nullptr )
+    if( pszGeonamesGeomFieldPrefix != nullptr && osLine.find('|') != std::string::npos )
         chDelimiter = '|';
 
-    char **papszFields = OGRCSVReadParseLineL(fp, chDelimiter, false);
+    char szDelimiter[2];
+    szDelimiter[0] = chDelimiter;
+    szDelimiter[1] = 0;
+    char **papszFields = CSVReadParseLine3L(fp, OGR_CSV_MAX_LINE_SIZE,
+                                            szDelimiter,
+                                            true, // bHonourStrings,
+                                            false, // bKeepLeadingAndClosingQuotes
+                                            false, // bMergeDelimiter
+                                            true // bSkipBOM
+                                           );
 
     if( CSLCount(papszFields) < 2 )
     {
