@@ -31,7 +31,7 @@
 
 #include "ogrdxf_polyline_smooth.h"
 
-CPL_CVSID("$Id: ogrdwglayer.cpp f86de9ef404101500003396503c43609219aa339 2021-08-19 20:35:04 +0800 GISerliang $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                            OGRDWGLayer()                             */
@@ -59,6 +59,15 @@ OGRDWGLayer::OGRDWGLayer( OGRDWGDataSource *poDSIn )
         poFeatureDefn->AddFieldDefn( &oBlockAngleField );
     }
 
+    if (!poDS->GetAttributes().empty()) {
+        std::set<CPLString>::iterator it;
+        for (it = poDS->GetAttributes().begin(); it != poDS->GetAttributes().end(); ++it) {
+            OGRFieldDefn  oAttField(*it, OFTString);
+            poFeatureDefn->AddFieldDefn(&oAttField);
+        }
+      
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Find the *Paper_Space block, which seems to contain all the     */
 /*      regular entities.                                               */
@@ -1128,6 +1137,39 @@ public:
         }
 };
 
+/************************************************************************/
+/*                        Translate3DFACE()                             */
+/************************************************************************/
+
+OGRFeature* OGRDWGLayer::Translate3DFACE(OdDbEntityPtr poEntity)
+{
+    OGRFeature* poFeature = new OGRFeature(poFeatureDefn);
+    OdDbFacePtr pFace = OdDbFace::cast(poEntity);
+
+    TranslateGenericProperties(poFeature, poEntity);
+
+    /* -------------------------------------------------------------------- */
+    /*      Create a polygon geometry from the vertices.                    */
+    /* -------------------------------------------------------------------- */
+    OGRPolygon* poPolygon = new OGRPolygon();
+
+    OGRLinearRing* poLinearRing = new OGRLinearRing();
+
+    OdInt16 index;
+    OdGePoint3d point;
+
+    for (index = 0; index <= 3; index++)
+    {
+        pFace->getVertexAt(index, point);
+        poLinearRing->addPoint(point.x,point.y,point.z);
+    }
+    poLinearRing->closeRings();
+    poPolygon->addRingDirectly(poLinearRing);
+    poFeature->SetGeometryDirectly(poPolygon);
+    PrepareLineStyle(poFeature);
+    return poFeature;
+}
+
 /************************************************************************/
 /*                          TranslateINSERT()                           */
 /************************************************************************/
@@ -1179,6 +1221,30 @@ OGRFeature *OGRDWGLayer::TranslateINSERT( OdDbEntityPtr poEntity )
         poFeature->SetField( "BlockAngle", dfAngle );
         poFeature->SetField( "BlockScale", 3, &(oTransformer.dfXScale) );
 
+        OdDbObjectIteratorPtr pEntIter = poRef->attributeIterator();
+        OdDbAttributePtr openAttr;
+
+        CPLJSONObject uAttrData = CPLJSONObject();
+        for (; !pEntIter->done(); pEntIter->step())
+        {
+            openAttr = pEntIter->entity()->objectId().safeOpenObject(OdDb::kForRead);
+            
+            CPLString attrText = TextUnescape( openAttr->textString(), false );
+            
+            const char* fieldName = CPLSPrintf("%ls", openAttr->tag().c_str());
+            if (poDS->AllAttributes() || (!openAttr->isInvisible() && openAttr->visibility() != OdDb::kInvisible)) {
+                uAttrData.Add(fieldName, attrText );
+            }
+
+            if (poDS->Attributes() && poFeatureDefn->GetFieldIndex(fieldName) != -1)
+            {
+                poFeature->SetField(fieldName, attrText);
+            }
+
+        }
+
+        poFeature->SetField( "BlockAttributes", uAttrData.ToString().c_str() );
+
         return poFeature;
     }
 
@@ -1249,7 +1315,14 @@ OGRFeature *OGRDWGLayer::TranslateINSERT( OdDbEntityPtr poEntity )
             oStyleProperties.clear();
 
             OGRFeature *poAttrFeat = TranslateTEXT( pAttr );
-
+            const char* fieldName = CPLSPrintf("%ls", pAttr->tag().c_str());
+            if( poDS->Attributes() && poFeatureDefn->GetFieldIndex(fieldName) != -1)
+            {
+                CPLString attrText = TextUnescape(pAttr->textString(), false);
+                poFeature->SetField(fieldName, attrText);
+                if (poAttrFeat)
+                    poAttrFeat->SetField(fieldName, attrText);
+            }
             if( poAttrFeat )
                 apoPendingFeatures.push( poAttrFeat );
         }
@@ -1370,6 +1443,10 @@ OGRFeature *OGRDWGLayer::GetNextUnfilteredFeature()
         {
             poFeature = TranslateHATCH( poEntity );
         }
+        else if (EQUAL(pszEntityClassName, "AcDbFace"))
+        {
+            poFeature = Translate3DFACE(poEntity);
+        }
         else if( EQUAL(pszEntityClassName,"AcDbBlockReference") )
         {
             poFeature = TranslateINSERT( poEntity );
