@@ -35,7 +35,7 @@
 #include "cpl_multiproc.h"
 #include "ogrmutexeddatasource.h"
 
-CPL_CVSID("$Id: FGdbDriver.cpp ab83782e9a1dc49e949679ea1dd055ee171d483f 2021-10-21 16:57:07 +1000 Nyall Dawson $")
+CPL_CVSID("$Id$")
 
 extern "C" void RegisterOGRFileGDB();
 
@@ -862,11 +862,14 @@ void RegisterOGRFileGDB()
 "  <Option name='FID' type='string' description='Name of OID column' default='OBJECTID' deprecated_alias='OID_NAME'/>"
 "  <Option name='XYTOLERANCE' type='float' description='Snapping tolerance, used for advanced ArcGIS features like network and topology rules, on 2D coordinates, in the units of the CRS'/>"
 "  <Option name='ZTOLERANCE' type='float' description='Snapping tolerance, used for advanced ArcGIS features like network and topology rules, on Z coordinates, in the units of the CRS'/>"
+"  <Option name='MTOLERANCE' type='float' description='Snapping tolerance, used for advanced ArcGIS features like network and topology rules, on M coordinates'/>"
 "  <Option name='XORIGIN' type='float' description='X origin of the coordinate precision grid'/>"
 "  <Option name='YORIGIN' type='float' description='Y origin of the coordinate precision grid'/>"
 "  <Option name='ZORIGIN' type='float' description='Z origin of the coordinate precision grid'/>"
+"  <Option name='MORIGIN' type='float' description='M origin of the coordinate precision grid'/>"
 "  <Option name='XYSCALE' type='float' description='X,Y scale of the coordinate precision grid'/>"
 "  <Option name='ZSCALE' type='float' description='Z scale of the coordinate precision grid'/>"
+"  <Option name='MSCALE' type='float' description='M scale of the coordinate precision grid'/>"
 "  <Option name='XML_DEFINITION' type='string' description='XML definition to create the new table. The root node of such a XML definition must be a &lt;esri:DataElement&gt; element conformant to FileGDBAPI.xsd'/>"
 "  <Option name='CREATE_MULTIPATCH' type='boolean' description='Whether to write geometries of layers of type MultiPolygon as MultiPatch' default='NO'/>"
 "  <Option name='COLUMN_TYPES' type='string' description='A list of strings of format field_name=fgdb_filed_type (separated by comma) to force the FileGDB column type of fields to be created'/>"
@@ -888,6 +891,10 @@ void RegisterOGRFileGDB()
     poDriver->SetMetadataItem( GDAL_DCAP_DEFAULT_FIELDS, "YES" );
     poDriver->SetMetadataItem( GDAL_DCAP_NOTNULL_GEOMFIELDS, "YES" );
     poDriver->SetMetadataItem( GDAL_DCAP_MULTIPLE_VECTOR_LAYERS, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_FIELD_DOMAINS, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RENAME_LAYERS, "YES" );
+
+    poDriver->SetMetadataItem( GDAL_DMD_CREATION_FIELD_DOMAIN_TYPES, "Coded Range" );
 
     poDriver->pfnOpen = OGRFileGDBDriverOpen;
     poDriver->pfnIdentify = OGRFileGDBDriverIdentify;
