@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2007, Mateusz Loskot
- * Copyright (c) 2010-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2010-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,12 +28,16 @@
  ****************************************************************************/
 
 #include "ogrgeojsonutils.h"
-#include <cpl_port.h>
-#include <cpl_conv.h>
-#include <ogr_geometry.h>
+#include <assert.h>
+#include "cpl_port.h"
+#include "cpl_conv.h"
+#include "ogr_geometry.h"
 #include <json.h> // JSON-C
 
-CPL_CVSID("$Id: ogrgeojsonutils.cpp 5e2d6cb208082d88745ee25e49c7e02063239c33 2018-04-18 22:48:59 +0200 Even Rouault $")
+#include <algorithm>
+#include <memory>
+
+CPL_CVSID("$Id$")
 
 const char szESRIJSonPotentialStart1[] =
     "{\"features\":[{\"geometry\":{\"rings\":[";
@@ -151,19 +155,51 @@ static CPLString GetCompactJSon( const char* pszText, size_t nMaxSize )
 }
 
 /************************************************************************/
-/*                           GeoJSONIsObject()                          */
+/*                          IsGeoJSONLikeObject()                       */
 /************************************************************************/
 
-bool GeoJSONIsObject( const char* pszText )
+static bool IsGeoJSONLikeObject( const char* pszText, bool& bMightBeSequence,
+                                 bool& bReadMoreBytes)
 {
+    bMightBeSequence = false;
+    bReadMoreBytes = false;
+
     if( !IsJSONObject(pszText) )
         return false;
 
     if( IsTypeSomething(pszText, "Topology") )
         return false;
 
+    if( IsTypeSomething(pszText, "FeatureCollection") )
+    {
+        return true;
+    }
+
+    CPLString osWithoutSpace = GetCompactJSon(pszText, strlen(pszText));
+    if( osWithoutSpace.find("{\"features\":[") == 0 &&
+        osWithoutSpace.find(szESRIJSonPotentialStart1) != 0 )
+    {
+        return true;
+    }
+
+    // See https://raw.githubusercontent.com/icepack/icepack-data/master/meshes/larsen/larsen_inflow.geojson
+    // "{"crs":...,"features":[..."
+    // or https://gist.githubusercontent.com/NiklasDallmann/27e339dd78d1942d524fbcd179f9fdcf/raw/527a8319d75a9e29446a32a19e4c902213b0d668/42XR9nLAh8Poh9Xmniqh3Bs9iisNm74mYMC56v3Wfyo=_isochrones_fails.geojson
+    // "{"bbox":...,"features":[..."
+    if( osWithoutSpace.find(",\"features\":[") != std::string::npos )
+    {
+        return !ESRIJSONIsObject(pszText);
+    }
+
+    // See https://github.com/OSGeo/gdal/issues/2720
+    if( osWithoutSpace.find("{\"coordinates\":[") == 0 ||
+        // and https://github.com/OSGeo/gdal/issues/2787
+        osWithoutSpace.find("{\"geometry\":{\"coordinates\":[") == 0 )
+    {
+        return true;
+    }
+
     if( IsTypeSomething(pszText, "Feature") ||
-           IsTypeSomething(pszText, "FeatureCollection") ||
            IsTypeSomething(pszText, "Point") ||
            IsTypeSomething(pszText, "LineString") ||
            IsTypeSomething(pszText, "Polygon") ||
@@ -172,20 +208,27 @@ bool GeoJSONIsObject( const char* pszText )
            IsTypeSomething(pszText, "MultiPolygon") ||
            IsTypeSomething(pszText, "GeometryCollection") )
     {
+        bMightBeSequence = true;
         return true;
     }
 
-    CPLString osWithoutSpace = GetCompactJSon(pszText,
-                                            strlen(szESRIJSonPotentialStart1));
-    if( osWithoutSpace.find("{\"features\":[") == 0 &&
-        osWithoutSpace.find(szESRIJSonPotentialStart1) != 0 )
+    // See https://github.com/OSGeo/gdal/issues/3280
+    if( osWithoutSpace.find("{\"properties\":{") == 0 )
     {
-        return true;
+        bMightBeSequence = true;
+        bReadMoreBytes = true;
+        return false;
     }
 
     return false;
 }
 
+static bool IsGeoJSONLikeObject( const char* pszText )
+{
+    bool bMightBeSequence;
+    bool bReadMoreBytes;
+    return IsGeoJSONLikeObject(pszText, bMightBeSequence, bReadMoreBytes);
+}
 
 /************************************************************************/
 /*                       ESRIJSONIsObject()                             */
@@ -233,6 +276,104 @@ bool TopoJSONIsObject(const char *pszText)
     return IsTypeSomething(pszText, "Topology");
 }
 
+/************************************************************************/
+/*                      IsLikelyNewlineSequenceGeoJSON()                */
+/************************************************************************/
+
+static bool IsLikelyNewlineSequenceGeoJSON( VSILFILE* fpL,
+                                            const GByte* pabyHeader,
+                                            const char* pszFileContent )
+{
+    const size_t nBufferSize = 4096 * 10;
+    std::vector<GByte> abyBuffer;
+    abyBuffer.resize(nBufferSize+1);
+
+    int nCurlLevel = 0;
+    bool bInString = false;
+    bool bLastIsEscape = false;
+    bool bCompatibleOfSequence = true;
+    bool bFirstIter = true;
+    bool bEOLFound = false;
+    int nCountObject = 0;
+    while( true )
+    {
+        size_t nRead;
+        bool bEnd = false;
+        if( bFirstIter )
+        {
+            const char* pszText = pszFileContent ? pszFileContent:
+                reinterpret_cast<const char*>(pabyHeader);
+            assert(pszText);
+            nRead = std::min(strlen(pszText), nBufferSize);
+            memcpy(abyBuffer.data(), pszText, nRead);
+            bFirstIter = false;
+            if( fpL )
+            {
+                VSIFSeekL(fpL, nRead, SEEK_SET);
+            }
+        }
+        else
+        {
+            nRead = VSIFReadL(abyBuffer.data() , 1, nBufferSize, fpL);
+            bEnd = nRead < nBufferSize;
+        }
+        for( size_t i = 0; i < nRead; i++ )
+        {
+            if( nCurlLevel == 0 )
+            {
+                if( abyBuffer[i] == '{' )
+                {
+                    nCountObject ++;
+                    if( nCountObject == 2 )
+                    {
+                        break;
+                    }
+                    nCurlLevel ++;
+                }
+                else if( nCountObject == 1 && abyBuffer[i] == '\n' )
+                {
+                    bEOLFound = true;
+                }
+                else if( !isspace( static_cast<int>(abyBuffer[i]) ) )
+                {
+                    bCompatibleOfSequence = false;
+                    break;
+                }
+            }
+            else if( bInString )
+            {
+                if( bLastIsEscape )
+                {
+                    bLastIsEscape = false;
+                }
+                else if( abyBuffer[i] == '\\' )
+                {
+                    bLastIsEscape = true;
+                }
+                else if( abyBuffer[i] == '"' )
+                {
+                    bInString = false;
+                }
+            }
+            else if( abyBuffer[i] == '"' )
+            {
+                bInString = true;
+            }
+            else if( abyBuffer[i] == '{' )
+            {
+                nCurlLevel ++;
+            }
+            else if( abyBuffer[i] == '}' )
+            {
+                nCurlLevel --;
+            }
+        }
+        if( !fpL || bEnd || !bCompatibleOfSequence || nCountObject == 2 )
+            break;
+    }
+    return bCompatibleOfSequence && bEOLFound && nCountObject == 2;
+}
+
 /************************************************************************/
 /*                           GeoJSONFileIsObject()                      */
 /************************************************************************/
@@ -250,12 +391,105 @@ bool GeoJSONFileIsObject( GDALOpenInfo* poOpenInfo )
         return false;
     }
 
-    if( !GeoJSONIsObject((const char*)poOpenInfo->pabyHeader) )
+    bool bMightBeSequence = false;
+    bool bReadMoreBytes = false;
+    if( !IsGeoJSONLikeObject(reinterpret_cast<const char*>(poOpenInfo->pabyHeader),
+                             bMightBeSequence, bReadMoreBytes) )
+    {
+        if( !(bReadMoreBytes && poOpenInfo->nHeaderBytes >= 6000 &&
+              poOpenInfo->TryToIngest(1000 * 1000) &&
+             !IsGeoJSONLikeObject(reinterpret_cast<const char*>(poOpenInfo->pabyHeader),
+                                  bMightBeSequence, bReadMoreBytes)) )
+        {
+            return false;
+        }
+    }
+
+    return !(bMightBeSequence && IsLikelyNewlineSequenceGeoJSON(
+        poOpenInfo->fpL, poOpenInfo->pabyHeader, nullptr));
+}
+
+/************************************************************************/
+/*                           GeoJSONIsObject()                          */
+/************************************************************************/
+
+bool GeoJSONIsObject( const char* pszText )
+{
+    bool bMightBeSequence = false;
+    bool bReadMoreBytes = false;
+    if( !IsGeoJSONLikeObject(pszText, bMightBeSequence, bReadMoreBytes) )
     {
         return false;
     }
 
-    return true;
+    return !(bMightBeSequence &&
+             IsLikelyNewlineSequenceGeoJSON(nullptr, nullptr, pszText));
+}
+
+/************************************************************************/
+/*                        GeoJSONSeqFileIsObject()                      */
+/************************************************************************/
+
+static
+bool GeoJSONSeqFileIsObject( GDALOpenInfo* poOpenInfo )
+{
+    // By default read first 6000 bytes.
+    // 6000 was chosen as enough bytes to
+    // enable all current tests to pass.
+
+    if( poOpenInfo->fpL == nullptr ||
+        !poOpenInfo->TryToIngest(6000) )
+    {
+        return false;
+    }
+
+    const char* pszText = reinterpret_cast<const char*>(poOpenInfo->pabyHeader);
+    if( pszText[0] == '\x1e' )
+        return IsGeoJSONLikeObject(pszText+1);
+
+    bool bMightBeSequence = false;
+    bool bReadMoreBytes = false;
+    if( !IsGeoJSONLikeObject(pszText, bMightBeSequence, bReadMoreBytes) )
+    {
+        if( !(bReadMoreBytes && poOpenInfo->nHeaderBytes >= 6000 &&
+              poOpenInfo->TryToIngest(1000 * 1000) &&
+             !IsGeoJSONLikeObject(reinterpret_cast<const char*>(poOpenInfo->pabyHeader),
+                                  bMightBeSequence, bReadMoreBytes)) )
+        {
+            return false;
+        }
+    }
+
+    return bMightBeSequence && IsLikelyNewlineSequenceGeoJSON(
+        poOpenInfo->fpL, poOpenInfo->pabyHeader, nullptr);
+}
+
+bool GeoJSONSeqIsObject( const char* pszText )
+{
+    if( pszText[0] == '\x1e' )
+        return IsGeoJSONLikeObject(pszText+1);
+
+    bool bMightBeSequence = false;
+    bool bReadMoreBytes = false;
+    if( !IsGeoJSONLikeObject(pszText, bMightBeSequence, bReadMoreBytes) )
+    {
+        return false;
+    }
+
+    return bMightBeSequence && IsLikelyNewlineSequenceGeoJSON(
+        nullptr, nullptr, pszText);
+}
+
+/************************************************************************/
+/*                           IsLikelyESRIJSONURL()                      */
+/************************************************************************/
+
+static bool IsLikelyESRIJSONURL(const char* pszURL)
+{
+    // URLs with f=json are strong candidates for ESRI JSON services
+    // except if they have "/items?", in which case they are likely OAPIF
+    return strstr(pszURL, "f=json") != nullptr &&
+          strstr(pszURL, "/items?") == nullptr;
 }
 
 /************************************************************************/
@@ -283,9 +517,13 @@ GeoJSONSourceType GeoJSONGetSourceType( GDALOpenInfo* poOpenInfo )
              strstr(poOpenInfo->pszFilename, "service=WFS") ||
              strstr(poOpenInfo->pszFilename, "service=wfs")) &&
              !strstr(poOpenInfo->pszFilename, "json") )
+        {
             return eGeoJSONSourceUnknown;
-        if( strstr(poOpenInfo->pszFilename, "f=json") )
+        }
+        if( IsLikelyESRIJSONURL(poOpenInfo->pszFilename) )
+        {
             return eGeoJSONSourceUnknown;
+        }
         srcType = eGeoJSONSourceService;
     }
     else if( STARTS_WITH_CI(poOpenInfo->pszFilename, "GeoJSON:") )
@@ -328,8 +566,10 @@ GeoJSONSourceType ESRIJSONDriverGetSourceType( GDALOpenInfo* poOpenInfo )
              STARTS_WITH(poOpenInfo->pszFilename, "https://") ||
              STARTS_WITH(poOpenInfo->pszFilename, "ftp://") )
     {
-        if( strstr(poOpenInfo->pszFilename, "f=json") != nullptr )
+        if( IsLikelyESRIJSONURL(poOpenInfo->pszFilename) )
+        {
             return eGeoJSONSourceService;
+        }
         return eGeoJSONSourceUnknown;
     }
 
@@ -357,8 +597,7 @@ GeoJSONSourceType ESRIJSONDriverGetSourceType( GDALOpenInfo* poOpenInfo )
     // By default read first 6000 bytes.
     // 6000 was chosen as enough bytes to
     // enable all current tests to pass.
-    if( poOpenInfo->fpL == nullptr ||
-        !poOpenInfo->TryToIngest(6000) )
+    if( !poOpenInfo->TryToIngest(6000) )
     {
         return eGeoJSONSourceUnknown;
     }
@@ -387,8 +626,10 @@ GeoJSONSourceType TopoJSONDriverGetSourceType( GDALOpenInfo* poOpenInfo )
              STARTS_WITH(poOpenInfo->pszFilename, "https://") ||
              STARTS_WITH(poOpenInfo->pszFilename, "ftp://") )
     {
-        if( strstr(poOpenInfo->pszFilename, "f=json") != nullptr )
+        if( IsLikelyESRIJSONURL(poOpenInfo->pszFilename) )
+        {
             return eGeoJSONSourceUnknown;
+        }
         return eGeoJSONSourceService;
     }
 
@@ -416,8 +657,7 @@ GeoJSONSourceType TopoJSONDriverGetSourceType( GDALOpenInfo* poOpenInfo )
     // By default read first 6000 bytes.
     // 6000 was chosen as enough bytes to
     // enable all current tests to pass.
-    if( poOpenInfo->fpL == nullptr ||
-        !poOpenInfo->TryToIngest(6000) )
+    if( !poOpenInfo->TryToIngest(6000) )
     {
         return eGeoJSONSourceUnknown;
     }
@@ -429,6 +669,55 @@ GeoJSONSourceType TopoJSONDriverGetSourceType( GDALOpenInfo* poOpenInfo )
     }
     return eGeoJSONSourceUnknown;
 }
+
+/************************************************************************/
+/*                          GeoJSONSeqGetSourceType()                   */
+/************************************************************************/
+
+GeoJSONSourceType GeoJSONSeqGetSourceType( GDALOpenInfo* poOpenInfo )
+{
+    GeoJSONSourceType srcType = eGeoJSONSourceUnknown;
+
+    if( STARTS_WITH_CI(poOpenInfo->pszFilename, "GEOJSONSeq:http://") ||
+        STARTS_WITH_CI(poOpenInfo->pszFilename, "GEOJSONSeq:https://") ||
+        STARTS_WITH_CI(poOpenInfo->pszFilename, "GEOJSONSeq:ftp://") )
+    {
+        srcType = eGeoJSONSourceService;
+    }
+    else if( STARTS_WITH_CI(poOpenInfo->pszFilename, "http://") ||
+             STARTS_WITH_CI(poOpenInfo->pszFilename, "https://") ||
+             STARTS_WITH_CI(poOpenInfo->pszFilename, "ftp://") )
+    {
+        if( IsLikelyESRIJSONURL(poOpenInfo->pszFilename) )
+        {
+            return eGeoJSONSourceUnknown;
+        }
+        srcType = eGeoJSONSourceService;
+    }
+    else if( STARTS_WITH_CI(poOpenInfo->pszFilename, "GEOJSONSeq:") )
+    {
+        VSIStatBufL sStat;
+        if( VSIStatL(poOpenInfo->pszFilename + strlen("GEOJSONSeq:"), &sStat) == 0 )
+        {
+            return eGeoJSONSourceFile;
+        }
+        const char* pszText = poOpenInfo->pszFilename + strlen("GEOJSONSeq:");
+        if( GeoJSONSeqIsObject(pszText) )
+            return eGeoJSONSourceText;
+        return eGeoJSONSourceUnknown;
+    }
+    else if( GeoJSONSeqIsObject( poOpenInfo->pszFilename ) )
+    {
+        srcType = eGeoJSONSourceText;
+    }
+    else if( GeoJSONSeqFileIsObject( poOpenInfo ) )
+    {
+        srcType = eGeoJSONSourceFile;
+    }
+
+    return srcType;
+}
+
 /************************************************************************/
 /*                           GeoJSONPropertyToFieldType()               */
 /************************************************************************/
@@ -483,43 +772,106 @@ OGRFieldType GeoJSONPropertyToFieldType( json_object* poObject,
     else if( json_type_array == type )
     {
         if( bArrayAsString )
+        {
+            eSubType = OFSTJSON;
             return OFTString;
-        const int nSize = json_object_array_length(poObject);
+        }
+        const auto nSize = json_object_array_length(poObject);
         if( nSize == 0 )
-            // We don't know, so let's assume it is a string list.
-            return OFTStringList;
+        {
+            eSubType = OFSTJSON;
+            return OFTString;
+        }
         OGRFieldType eType = OFTIntegerList;
-        bool bOnlyBoolean = true;
-        for( int i = 0; i < nSize; i++ )
+        for( auto i = decltype(nSize){0}; i < nSize; i++ )
         {
             json_object* poRow = json_object_array_get_idx(poObject, i);
             if( poRow != nullptr )
             {
                 type = json_object_get_type( poRow );
-                bOnlyBoolean &= type == json_type_boolean;
                 if( type == json_type_string )
-                    return OFTStringList;
-                else if( type == json_type_double )
-                    eType = OFTRealList;
-                else if( eType == OFTIntegerList &&
-                         type == json_type_int )
                 {
-                    GIntBig nVal = json_object_get_int64(poRow);
-                    if( !CPL_INT64_FITS_ON_INT32(nVal) )
-                        eType = OFTInteger64List;
+                    if( i == 0 || eType == OFTStringList )
+                    {
+                        eType = OFTStringList;
+                    }
+                    else
+                    {
+                        eSubType = OFSTJSON;
+                        return OFTString;
+                    }
                 }
-                else if( type != json_type_int &&
-                         type != json_type_boolean )
+                else if( type == json_type_double )
+                {
+                    if( eSubType == OFSTNone &&
+                        (i == 0 ||
+                        eType == OFTRealList ||
+                        eType == OFTIntegerList ||
+                        eType == OFTInteger64List) )
+                    {
+                        eType = OFTRealList;
+                    }
+                    else
+                    {
+                        eSubType = OFSTJSON;
+                        return OFTString;
+                    }
+                }
+                else if( type == json_type_int )
+                {
+                    if( eSubType == OFSTNone &&
+                        eType == OFTIntegerList )
+                    {
+                        GIntBig nVal = json_object_get_int64(poRow);
+                        if( !CPL_INT64_FITS_ON_INT32(nVal) )
+                            eType = OFTInteger64List;
+                    }
+                    else if( eSubType == OFSTNone &&
+                             (eType == OFTInteger64List ||
+                              eType == OFTRealList) )
+                    {
+                        // ok
+                    }
+                    else
+                    {
+                        eSubType = OFSTJSON;
+                        return OFTString;
+                    }
+                }
+                else if( type == json_type_boolean )
+                {
+                    if( i == 0 || (eType == OFTIntegerList && eSubType == OFSTBoolean) )
+                    {
+                        eSubType = OFSTBoolean;
+                    }
+                    else
+                    {
+                        eSubType = OFSTJSON;
+                        return OFTString;
+                    }
+                }
+                else
+                {
+                    eSubType = OFSTJSON;
                     return OFTString;
+                }
+            }
+            else
+            {
+                eSubType = OFSTJSON;
+                return OFTString;
             }
         }
-        if( bOnlyBoolean )
-            eSubType = OFSTBoolean;
 
         return eType;
     }
+    else if( json_type_object == type )
+    {
+        eSubType = OFSTJSON;
+        return OFTString;
+    }
 
-    return OFTString; // null, object
+    return OFTString; // null
 }
 
 /************************************************************************/
