@@ -54,7 +54,7 @@
 #  include "ogr_xerces.h"
 #endif
 
-CPL_CVSID("$Id: gmlhandler.cpp 286a02e0797a5b5469b608c4b3868f976620e6cc 2020-10-22 11:54:43 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 #ifdef HAVE_XERCES
 
@@ -67,6 +67,22 @@ GMLXercesHandler::GMLXercesHandler( GMLReader *poReader ) :
     m_nEntityCounter(0)
 {}
 
+/************************************************************************/
+/*                    GMLXercesHandlerDealWithError()                   */
+/************************************************************************/
+
+static void GMLXercesHandlerDealWithError(OGRErr eErr)
+{
+    if( eErr == OGRERR_NOT_ENOUGH_MEMORY )
+    {
+        throw SAXNotSupportedException("Out of memory");
+    }
+    else if( eErr != OGRERR_NONE )
+    {
+        throw SAXNotSupportedException("Other error during parsing");
+    }
+}
+
 /************************************************************************/
 /*                            startElement()                            */
 /************************************************************************/
@@ -80,13 +96,10 @@ void GMLXercesHandler::startElement( const XMLCh* const /*uri*/,
 
     transcode(localname, m_osElement);
 
-    if( GMLHandler::startElement(
+    GMLXercesHandlerDealWithError(GMLHandler::startElement(
             m_osElement.c_str(),
             static_cast<int>(m_osElement.size()),
-            const_cast<Attributes *>(&attrs)) == OGRERR_NOT_ENOUGH_MEMORY )
-    {
-        throw SAXNotSupportedException("Out of memory");
-    }
+            const_cast<Attributes *>(&attrs)));
 }
 
 /************************************************************************/
@@ -98,10 +111,7 @@ void GMLXercesHandler::endElement(const XMLCh* const /*uri*/,
 {
     m_nEntityCounter = 0;
 
-    if (GMLHandler::endElement() == OGRERR_NOT_ENOUGH_MEMORY)
-    {
-        throw SAXNotSupportedException("Out of memory");
-    }
+    GMLXercesHandlerDealWithError(GMLHandler::endElement());
 }
 
 /************************************************************************/
@@ -113,12 +123,8 @@ void GMLXercesHandler::characters(const XMLCh* const chars_in,
 
 {
     transcode( chars_in, m_osCharacters, static_cast<int>(length) );
-    OGRErr eErr = GMLHandler::dataHandler(m_osCharacters.c_str(),
-                                    static_cast<int>(m_osCharacters.size()));
-    if (eErr == OGRERR_NOT_ENOUGH_MEMORY)
-    {
-        throw SAXNotSupportedException("Out of memory");
-    }
+    GMLXercesHandlerDealWithError(GMLHandler::dataHandler(m_osCharacters.c_str(),
+                                    static_cast<int>(m_osCharacters.size())));
 }
 
 /************************************************************************/
@@ -264,6 +270,21 @@ GMLExpatHandler::GMLExpatHandler( GMLReader *poReader, XML_Parser oParser ) :
     m_nDataHandlerCounter(0)
 {}
 
+/************************************************************************/
+/*                  GMLExpatHandler::DealWithError()                    */
+/************************************************************************/
+
+void GMLExpatHandler::DealWithError(OGRErr eErr)
+{
+    if( eErr != OGRERR_NONE )
+    {
+        m_bStopParsing = true;
+        XML_StopParser(m_oParser, XML_FALSE);
+        if( eErr == OGRERR_NOT_ENOUGH_MEMORY )
+            CPLError(CE_Failure, CPLE_OutOfMemory, "Out of memory");
+    }
+}
+
 /************************************************************************/
 /*                           startElementCbk()                          */
 /************************************************************************/
@@ -286,14 +307,9 @@ void XMLCALL GMLExpatHandler::startElementCbk( void *pUserData,
         pszIter ++;
     }
 
-    if( pThis->GMLHandler::startElement(pszName,
+    pThis->DealWithError( pThis->GMLHandler::startElement(pszName,
                                         static_cast<int>(pszIter - pszName),
-                                        ppszAttr) == OGRERR_NOT_ENOUGH_MEMORY )
-    {
-        CPLError(CE_Failure, CPLE_OutOfMemory, "Out of memory");
-        pThis->m_bStopParsing = true;
-        XML_StopParser(pThis->m_oParser, XML_FALSE);
-    }
+                                        ppszAttr) );
 }
 
 /************************************************************************/
@@ -306,12 +322,7 @@ void XMLCALL GMLExpatHandler::endElementCbk( void *pUserData,
     if( pThis->m_bStopParsing )
         return;
 
-    if( pThis->GMLHandler::endElement() == OGRERR_NOT_ENOUGH_MEMORY )
-    {
-        CPLError(CE_Failure, CPLE_OutOfMemory, "Out of memory");
-        pThis->m_bStopParsing = true;
-        XML_StopParser(pThis->m_oParser, XML_FALSE);
-    }
+    pThis->DealWithError( pThis->GMLHandler::endElement() );
 }
 
 /************************************************************************/
@@ -343,12 +354,7 @@ GMLExpatHandler::dataHandlerCbk(void *pUserData, const char *data, int nLen)
         return;
     }
 
-    if( pThis->GMLHandler::dataHandler(data, nLen) == OGRERR_NOT_ENOUGH_MEMORY )
-    {
-        CPLError(CE_Failure, CPLE_OutOfMemory, "Out of memory");
-        pThis->m_bStopParsing = true;
-        XML_StopParser(pThis->m_oParser, XML_FALSE);
-    }
+    pThis->DealWithError( pThis->GMLHandler::dataHandler(data, nLen) );
 }
 
 /************************************************************************/
@@ -592,6 +598,25 @@ OGRErr GMLHandler::startElement(const char *pszName, int nLenName, void* attr)
         default:                        eRet = OGRERR_NONE; break;
     }
     m_nDepth++;
+    if( m_nDepth == 64 )
+    {
+        // Avoid performance issues on files like
+        // https://bugs.chromium.org/p/oss-fuzz/issues/detail?id=21737
+        if( m_nUnlimitedDepth < 0 )
+        {
+            m_nUnlimitedDepth =
+                EQUAL(CPLGetConfigOption("OGR_GML_NESTING_LEVEL", ""), "UNLIMITED");
+        }
+        if( !m_nUnlimitedDepth )
+        {
+            CPLError(CE_Failure, CPLE_NotSupported,
+                     "Too deep XML nesting level (%d). "
+                     "Set the OGR_GML_NESTING_LEVEL configuration option to "
+                     "UNLIMITED to remove that limitation.",
+                     m_nDepth);
+            eRet = OGRERR_FAILURE;
+        }
+    }
     return eRet;
 }
 
@@ -808,7 +833,7 @@ void GMLHandler::DealWithAttributes(const char *pszName, int nLenName, void* att
             }
             else if( (!poClass->IsSchemaLocked() && (m_bReportHref || m_poReader->ReportAllAttributes())) ||
                         (poClass->IsSchemaLocked() && (nAttrIndex =
-                        m_poReader->GetAttributeElementIndex( CPLSPrintf("%s_href", pszName ),
+                        m_poReader->GetAttributeElementIndex( (std::string(pszName) + "_href").c_str(),
                                                     nLenName + 5 )) != -1) )
             {
                 poState->PushPath( pszName, nLenName );
@@ -1127,7 +1152,22 @@ OGRErr GMLHandler::startElementFeatureAttribute(const char *pszName, int nLenNam
             }
 
             else
+            {
+                if( m_poReader->IsConsistentSingleGeomElemPath() )
+                {
+                    const std::string& osGeomElemPath = m_poReader->GetSingleGeomElemPath();
+                    if( osGeomElemPath.empty() )
+                    {
+                        m_poReader->SetSingleGeomElemPath(poState->osPath);
+                    }
+                    else if( poState->osPath != osGeomElemPath )
+                    {
+                        m_poReader->SetConsistentSingleGeomElemPath(false);
+                        m_poReader->SetSingleGeomElemPath(std::string());
+                    }
+                }
                 bReadGeometry = true;
+            }
         }
         if (bReadGeometry)
         {
