@@ -2,10 +2,10 @@
  *
  * Project:  GML Utils
  * Purpose:  GML reader
- * Author:   Even Rouault, <even dot rouault at mines dash paris dot org>
+ * Author:   Even Rouault, <even dot rouault at spatialys.com>
  *
  ******************************************************************************
- * Copyright (c) 2010-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2010-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -43,7 +43,7 @@
 #include "ogr_p.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: gmlutils.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                GML_ExtractSrsNameFromGeometry()                      */
@@ -109,7 +109,7 @@ bool GML_IsSRSLatLongOrder(const char *pszSRSName)
     else if( !EQUALN(pszSRSName, "EPSG:", 5) )
     {
         OGRSpatialReference oSRS;
-        if(oSRS.SetFromUserInput(pszSRSName) == OGRERR_NONE)
+        if(oSRS.SetFromUserInput(pszSRSName, OGRSpatialReference::SET_FROM_USER_INPUT_LIMITATIONS_get()) == OGRERR_NONE)
         {
             if(oSRS.EPSGTreatsAsLatLong() || oSRS.EPSGTreatsAsNorthingEasting())
                 return true;
@@ -166,7 +166,7 @@ class SRSCache
         oLastDesc.bAxisInvert = GML_IsSRSLatLongOrder(osSRSName.c_str());
         oLastDesc.poSRS = new OGRSpatialReference();
         oLastDesc.poSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
-        if( oLastDesc.poSRS->SetFromUserInput(osSRSName.c_str()) !=
+        if( oLastDesc.poSRS->SetFromUserInput(osSRSName.c_str(), OGRSpatialReference::SET_FROM_USER_INPUT_LIMITATIONS_get()) !=
             OGRERR_NONE )
         {
             delete oLastDesc.poSRS;
@@ -248,7 +248,7 @@ OGRGeometry* GML_BuildOGRGeometryFromList(
                                 wkbMultiPolygon)
                     {
                         OGRMultiPolygon *poGeomColl = poGeom->toMultiPolygon();
-                        for( auto&& poMember: poSubGeom->toMultiPolygon() ) 
+                        for( auto&& poMember: poSubGeom->toMultiPolygon() )
                         {
                             poGeomColl->addGeometry(poMember);
                         }
