@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2010, Brian Case
- * Copyright (c) 2010-2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2010-2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -36,7 +36,7 @@
 #include "ogr_p.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: ogrlibkmlfield.cpp a1751d77d34c65c736bd985bb4e2153ae9ecf518 2017-12-13 19:08:01Z Even Rouault $")
+CPL_CVSID("$Id$")
 
 using kmldom::CameraPtr;
 using kmldom::DataPtr;
@@ -680,8 +680,8 @@ void field2kml(
                     if( poKmlPlacemark->has_geometry()
                         && -1 < poOgrFeat->GetFieldAsInteger( i ) )
                     {
-                        const int iTesselate = poOgrFeat->GetFieldAsInteger( i );
-                        if( iTesselate &&
+                        const int iTessellate = poOgrFeat->GetFieldAsInteger( i );
+                        if( iTessellate &&
                             !(!isGX &&
                               static_cast<kmldom::AltitudeModeEnum>(iAltitudeMode) ==
                               kmldom::ALTITUDEMODE_CLAMPTOGROUND) &&
@@ -700,7 +700,7 @@ void field2kml(
                         {
                             GeometryPtr poKmlGeometry =
                                 poKmlPlacemark->get_geometry();
-                            ogr2tessellate_rec( CPL_TO_BOOL(iTesselate),
+                            ogr2tessellate_rec( CPL_TO_BOOL(iTessellate),
                                                 poKmlGeometry );
                             if( !isGX &&
                                 iAltitudeMode ==
@@ -1357,20 +1357,18 @@ void kml2field( OGRFeature * poOgrFeat, FeaturePtr poKmlFeature )
                 poOgrFeat->SetField( iField, camera->get_roll() );
         }
 
-        int nAltitudeMode = -1;
-
         int iField = poOgrFeat->GetFieldIndex( oFC.altitudeModefield );
 
         if( iField > -1 )
         {
             if( camera->has_altitudemode() )
             {
-                nAltitudeMode = camera->get_altitudemode();
+                const int nAltitudeMode = camera->get_altitudemode();
                 ogrkmlSetAltitudeMode(poOgrFeat, iField, nAltitudeMode, false);
             }
             else if( camera->has_gx_altitudemode() )
             {
-                nAltitudeMode = camera->get_gx_altitudemode();
+                const int nAltitudeMode = camera->get_gx_altitudemode();
                 ogrkmlSetAltitudeMode(poOgrFeat, iField, nAltitudeMode, true);
             }
         }
@@ -1681,31 +1679,33 @@ void kml2FeatureDef(
         {
             osName = poKmlSimpleField->get_name();
         }
-
-        if( EQUAL( pszType, "bool" ) ||
-            EQUAL( pszType, "boolean" ) ||
-            EQUAL( pszType, "int" ) ||
-            EQUAL( pszType, "short" ) ||
-            EQUAL( pszType, "ushort" ) )
+        if( poOgrFeatureDefn->GetFieldIndex(osName.c_str()) < 0 )
         {
-            OGRFieldDefn oOgrFieldName( osName.c_str(), OFTInteger );
-            poOgrFeatureDefn->AddFieldDefn( &oOgrFieldName );
-        }
-        else if( EQUAL( pszType, "uint" ) )
-        {
-            OGRFieldDefn oOgrFieldName( osName.c_str(), OFTInteger64 );
-            poOgrFeatureDefn->AddFieldDefn( &oOgrFieldName );
-        }
-        else if( EQUAL( pszType, "float" ) ||
-                 EQUAL( pszType, "double" ) )
-        {
-            OGRFieldDefn oOgrFieldName( osName.c_str(), OFTReal );
-            poOgrFeatureDefn->AddFieldDefn( &oOgrFieldName );
-        }
-        else  // string, or any other unrecognized type.
-        {
-            OGRFieldDefn oOgrFieldName( osName.c_str(), OFTString );
-            poOgrFeatureDefn->AddFieldDefn( &oOgrFieldName );
+            if( EQUAL( pszType, "bool" ) ||
+                EQUAL( pszType, "boolean" ) ||
+                EQUAL( pszType, "int" ) ||
+                EQUAL( pszType, "short" ) ||
+                EQUAL( pszType, "ushort" ) )
+            {
+                OGRFieldDefn oOgrFieldName( osName.c_str(), OFTInteger );
+                poOgrFeatureDefn->AddFieldDefn( &oOgrFieldName );
+            }
+            else if( EQUAL( pszType, "uint" ) )
+            {
+                OGRFieldDefn oOgrFieldName( osName.c_str(), OFTInteger64 );
+                poOgrFeatureDefn->AddFieldDefn( &oOgrFieldName );
+            }
+            else if( EQUAL( pszType, "float" ) ||
+                    EQUAL( pszType, "double" ) )
+            {
+                OGRFieldDefn oOgrFieldName( osName.c_str(), OFTReal );
+                poOgrFeatureDefn->AddFieldDefn( &oOgrFieldName );
+            }
+            else  // string, or any other unrecognized type.
+            {
+                OGRFieldDefn oOgrFieldName( osName.c_str(), OFTString );
+                poOgrFeatureDefn->AddFieldDefn( &oOgrFieldName );
+            }
         }
     }
 }
