@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2004, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,7 +34,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrmysqldatasource.cpp 241fb3ce52db2a876ef46df945ece3bf1fcdaa28 2019-03-23 13:58:57 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 /************************************************************************/
 /*                         OGRMySQLDataSource()                         */
 /************************************************************************/
@@ -387,6 +387,10 @@ OGRLayer *OGRMySQLDataSource::GetLayer( int iLayer )
         return papoLayers[iLayer];
 }
 
+/* =====================================================================*/
+/* Handle spatial reference id and index for older MySQL and MariaDB    */
+/* =====================================================================*/
+
 /************************************************************************/
 /*                      InitializeMetadataTables()                      */
 /*                                                                      */
@@ -402,37 +406,37 @@ OGRErr OGRMySQLDataSource::InitializeMetadataTables()
     MYSQL_RES       *hResult;
     OGRErr          eErr = OGRERR_NONE;
 
-    pszCommand = "DESCRIBE geometry_columns";
-    if( mysql_query(GetConn(), pszCommand ) )
-    {
-        pszCommand =
-                "CREATE TABLE geometry_columns "
-                "( F_TABLE_CATALOG VARCHAR(256), "
-                "F_TABLE_SCHEMA VARCHAR(256), "
-                "F_TABLE_NAME VARCHAR(256) NOT NULL,"
-                "F_GEOMETRY_COLUMN VARCHAR(256) NOT NULL, "
-                "COORD_DIMENSION INT, "
-                "SRID INT,"
-                "TYPE VARCHAR(256) NOT NULL)";
-        if( mysql_query(GetConn(), pszCommand ) )
-        {
-            ReportError( pszCommand );
-            eErr = OGRERR_FAILURE;
-        }
-        else
-            CPLDebug("MYSQL","Creating geometry_columns metadata table");
-    }
-
-    // make sure to attempt to free results of successful queries
-    hResult = mysql_store_result( GetConn() );
-    if( hResult != nullptr )
-    {
-        mysql_free_result( hResult );
-        hResult = nullptr;
-    }
-
     if( GetMajorVersion() < 8 || IsMariaDB() )
     {
+        pszCommand = "DESCRIBE geometry_columns";
+        if( mysql_query(GetConn(), pszCommand ) )
+        {
+            pszCommand =
+                    "CREATE TABLE geometry_columns "
+                    "( F_TABLE_CATALOG VARCHAR(256), "
+                    "F_TABLE_SCHEMA VARCHAR(256), "
+                    "F_TABLE_NAME VARCHAR(256) NOT NULL,"
+                    "F_GEOMETRY_COLUMN VARCHAR(256) NOT NULL, "
+                    "COORD_DIMENSION INT, "
+                    "SRID INT,"
+                    "TYPE VARCHAR(256) NOT NULL)";
+            if( mysql_query(GetConn(), pszCommand ) )
+            {
+                ReportError( pszCommand );
+                eErr = OGRERR_FAILURE;
+            }
+            else
+                CPLDebug("MYSQL","Creating geometry_columns metadata table");
+        }
+
+        // make sure to attempt to free results of successful queries
+        hResult = mysql_store_result( GetConn() );
+        if( hResult != nullptr )
+        {
+            mysql_free_result( hResult );
+            hResult = nullptr;
+        }
+
         pszCommand = "DESCRIBE spatial_ref_sys";
         if( mysql_query(GetConn(), pszCommand ) )
         {
@@ -463,6 +467,93 @@ OGRErr OGRMySQLDataSource::InitializeMetadataTables()
     return eErr;
 }
 
+OGRErr OGRMySQLDataSource::UpdateMetadataTables(const char *pszLayerName,
+                                                OGRwkbGeometryType eType,
+                                                const char *pszGeomColumnName,
+                                                const int nSRSId)
+
+{
+    MYSQL_RES *hResult = nullptr;
+    CPLString osCommand;
+    const char *pszGeometryType;
+
+    if (GetMajorVersion() < 8 || IsMariaDB())
+    {
+/* -------------------------------------------------------------------- */
+/*      Sometimes there is an old crufty entry in the geometry_columns  */
+/*      table if things were not properly cleaned up before.  We make   */
+/*      an effort to clean out such cruft.                              */
+/*                                                                      */
+/* -------------------------------------------------------------------- */
+        osCommand.Printf(
+                "DELETE FROM geometry_columns WHERE f_table_name = '%s'",
+                pszLayerName);
+
+        if (mysql_query(GetConn(), osCommand))
+        {
+                ReportError(osCommand);
+                return OGRERR_FAILURE;
+        }
+
+        // make sure to attempt to free results of successful queries
+        hResult = mysql_store_result(GetConn());
+        if (hResult != nullptr)
+            mysql_free_result(hResult);
+        hResult = nullptr;
+
+/* -------------------------------------------------------------------- */
+/*      Attempt to add this table to the geometry_columns table, if     */
+/*      it is a spatial layer.                                          */
+/* -------------------------------------------------------------------- */
+        if (eType != wkbNone) {
+            const int nCoordDimension = eType == wkbFlatten(eType) ? 2 : 3;
+
+            pszGeometryType = OGRToOGCGeomType(eType);
+
+            if (nSRSId == GetUnknownSRID())
+                osCommand.Printf(
+                        "INSERT INTO geometry_columns "
+                        " (F_TABLE_NAME, "
+                        "  F_GEOMETRY_COLUMN, "
+                        "  COORD_DIMENSION, "
+                        "  TYPE) values "
+                        "  ('%s', '%s', %d, '%s')",
+                        pszLayerName,
+                        pszGeomColumnName,
+                        nCoordDimension,
+                        pszGeometryType);
+            else
+                osCommand.Printf(
+                        "INSERT INTO geometry_columns "
+                        " (F_TABLE_NAME, "
+                        "  F_GEOMETRY_COLUMN, "
+                        "  COORD_DIMENSION, "
+                        "  SRID, "
+                        "  TYPE) values "
+                        "  ('%s', '%s', %d, %d, '%s')",
+                        pszLayerName,
+                        pszGeomColumnName,
+                        nCoordDimension,
+                        nSRSId,
+                        pszGeometryType);
+
+            if (mysql_query(GetConn(), osCommand)) {
+                ReportError(osCommand);
+                return OGRERR_FAILURE;
+            }
+
+            // make sure to attempt to free results of successful queries
+            hResult = mysql_store_result(GetConn());
+            if (hResult != nullptr)
+                mysql_free_result(hResult);
+            hResult = nullptr;
+        }
+    }
+    return OGRERR_NONE;
+}
+
+/* =====================================================================*/
+
 /************************************************************************/
 /*                              FetchSRS()                              */
 /*                                                                      */
@@ -606,11 +697,10 @@ int OGRMySQLDataSource::FetchSRSId( OGRSpatialReference * poSRS )
 /*      SRS ID.                                                         */
 /* -------------------------------------------------------------------- */
     CPLString osCommand;
-    int nAuthorityCode = 0;
     if( pszAuthorityName != nullptr )
     {
         /* Check that the authority code is integral */
-        nAuthorityCode = atoi( oSRS.GetAuthorityCode(nullptr) );
+        const int nAuthorityCode = atoi( oSRS.GetAuthorityCode(nullptr) );
         if( nAuthorityCode > 0 )
         {
             if( GetMajorVersion() < 8 || IsMariaDB() )
@@ -965,7 +1055,6 @@ OGRMySQLDataSource::ICreateLayer( const char * pszLayerNameIn,
 {
     MYSQL_RES  *hResult=nullptr;
     CPLString   osCommand;
-    const char *pszGeometryType;
     const char *pszGeomColumnName;
     const char *pszExpectedFIDName;
     char       *pszLayerName;
@@ -1033,6 +1122,18 @@ OGRMySQLDataSource::ICreateLayer( const char * pszLayerNameIn,
     const char *pszSI = CSLFetchNameValue( papszOptions, "SPATIAL_INDEX" );
     const bool bHasSI = ( eType != wkbNone && (pszSI == nullptr || CPLTestBool(pszSI)) );
 
+    // Calling this does no harm
+    InitializeMetadataTables();
+
+/* -------------------------------------------------------------------- */
+/*      Try to get the SRS Id of this spatial reference system,         */
+/*      adding to the srs table if needed.                             */
+/* -------------------------------------------------------------------- */
+
+    int nSRSId = GetUnknownSRID();
+    if (poSRS != nullptr)
+        nSRSId = FetchSRSId(poSRS);
+
     if( wkbFlatten(eType) == wkbNone )
     {
         osCommand.Printf(
@@ -1042,12 +1143,21 @@ OGRMySQLDataSource::ICreateLayer( const char * pszLayerNameIn,
     }
     else
     {
-        osCommand.Printf(
-                 "CREATE TABLE `%s` ( "
-                 "   %s %s UNIQUE NOT NULL AUTO_INCREMENT, "
-                 "   %s GEOMETRY %s)",
-                 pszLayerName, pszExpectedFIDName, pszFIDType, pszGeomColumnName,
-                 bHasSI ? "NOT NULL" : "");
+        // when using mysql8 and SRS is specified, use SRID option for geometry.
+        if( GetMajorVersion() < 8 || IsMariaDB() || nSRSId == GetUnknownSRID() )
+            osCommand.Printf(
+                    "CREATE TABLE `%s` ( "
+                    "   %s %s UNIQUE NOT NULL AUTO_INCREMENT, "
+                    "   %s GEOMETRY %s)",
+                    pszLayerName, pszExpectedFIDName, pszFIDType, pszGeomColumnName,
+                    bHasSI ? "NOT NULL" : "");
+        else
+            osCommand.Printf(
+                    "CREATE TABLE `%s` ( "
+                    "   %s %s UNIQUE NOT NULL AUTO_INCREMENT, "
+                    "   %s GEOMETRY %s /*!80003 SRID %d */)",
+                    pszLayerName, pszExpectedFIDName, pszFIDType, pszGeomColumnName,
+                    bHasSI ? "NOT NULL" : "", nSRSId);
     }
 
     if( CSLFetchNameValue( papszOptions, "ENGINE" ) != nullptr )
@@ -1078,89 +1188,8 @@ OGRMySQLDataSource::ICreateLayer( const char * pszLayerNameIn,
         mysql_free_result( hResult );
     hResult = nullptr;
 
-    // Calling this does no harm
-    InitializeMetadataTables();
-
-/* -------------------------------------------------------------------- */
-/*      Try to get the SRS Id of this spatial reference system,         */
-/*      adding tot the srs table if needed.                             */
-/* -------------------------------------------------------------------- */
-    int nSRSId = GetUnknownSRID();
-
-    if( poSRS != nullptr )
-        nSRSId = FetchSRSId( poSRS );
-
-/* -------------------------------------------------------------------- */
-/*      Sometimes there is an old crufty entry in the geometry_columns  */
-/*      table if things were not properly cleaned up before.  We make   */
-/*      an effort to clean out such cruft.                              */
-/*                                                                      */
-/* -------------------------------------------------------------------- */
-    osCommand.Printf(
-             "DELETE FROM geometry_columns WHERE f_table_name = '%s'",
-             pszLayerName );
-
-    if( mysql_query(GetConn(), osCommand ) )
-    {
-        ReportError( osCommand );
+    if( UpdateMetadataTables(pszLayerName, eType, pszGeomColumnName, nSRSId) != OGRERR_NONE )
         return nullptr;
-    }
-
-    // make sure to attempt to free results of successful queries
-    hResult = mysql_store_result( GetConn() );
-    if( hResult != nullptr )
-        mysql_free_result( hResult );
-    hResult = nullptr;
-
-/* -------------------------------------------------------------------- */
-/*      Attempt to add this table to the geometry_columns table, if     */
-/*      it is a spatial layer.                                          */
-/* -------------------------------------------------------------------- */
-    if( eType != wkbNone )
-    {
-        const int nCoordDimension = eType == wkbFlatten(eType) ? 2 : 3;
-
-        pszGeometryType = OGRToOGCGeomType(eType);
-
-        if( nSRSId == GetUnknownSRID() )
-            osCommand.Printf(
-                     "INSERT INTO geometry_columns "
-                     " (F_TABLE_NAME, "
-                     "  F_GEOMETRY_COLUMN, "
-                     "  COORD_DIMENSION, "
-                     "  TYPE) values "
-                     "  ('%s', '%s', %d, '%s')",
-                     pszLayerName,
-                     pszGeomColumnName,
-                     nCoordDimension,
-                     pszGeometryType );
-        else
-            osCommand.Printf(
-                     "INSERT INTO geometry_columns "
-                     " (F_TABLE_NAME, "
-                     "  F_GEOMETRY_COLUMN, "
-                     "  COORD_DIMENSION, "
-                     "  SRID, "
-                     "  TYPE) values "
-                     "  ('%s', '%s', %d, %d, '%s')",
-                     pszLayerName,
-                     pszGeomColumnName,
-                     nCoordDimension,
-                     nSRSId,
-                     pszGeometryType );
-
-        if( mysql_query(GetConn(), osCommand ) )
-        {
-            ReportError( osCommand );
-            return nullptr;
-        }
-
-        // make sure to attempt to free results of successful queries
-        hResult = mysql_store_result( GetConn() );
-        if( hResult != nullptr )
-            mysql_free_result( hResult );
-        hResult = nullptr;
-    }
 
 /* -------------------------------------------------------------------- */
 /*      Create the spatial index.                                       */
