@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2004, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +32,7 @@
 #include "cpl_string.h"
 #include "ogr_mysql.h"
 
-CPL_CVSID("$Id: ogrmysqltablelayer.cpp 8e4a8befc5cdeaa291a36825ea70a6fac905dced 2018-09-08 19:44:31 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                         OGRMySQLTableLayer()                         */
@@ -361,11 +361,8 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
     // set to none for now... if we have a geometry column it will be set layer.
     poDefn->SetGeomType( wkbNone );
 
-    if( hResult != nullptr )
-    {
-        mysql_free_result( hResult );
-        hResultSet = nullptr;
-    }
+    mysql_free_result( hResult );
+    hResult = nullptr;
 
     if( bHasFid )
         CPLDebug( "MySQL", "table %s has FID column %s.",
@@ -383,11 +380,14 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
         poDefn->SetGeomType( wkbUnknown );
         poDefn->GetGeomFieldDefn(0)->SetName( pszGeomColumn );
 
-        osCommand = "SELECT type, coord_dimension FROM geometry_columns WHERE f_table_name='";
-        osCommand += pszTable;
-        osCommand += "'";
+        if( poDS->GetMajorVersion() < 8 || poDS->IsMariaDB() )
+            osCommand.Printf("SELECT type, coord_dimension FROM geometry_columns WHERE f_table_name='%s'",
+                             pszTable);
+
+        else
+            osCommand.Printf("SELECT GEOMETRY_TYPE_NAME FROM INFORMATION_SCHEMA.ST_GEOMETRY_COLUMNS "
+                             "WHERE TABLE_NAME = '%s'", pszTable);
 
-        hResult = nullptr;
         if( !mysql_query( poDS->GetConn(), osCommand ) )
             hResult = mysql_store_result( poDS->GetConn() );
 
@@ -402,8 +402,9 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
 
             OGRwkbGeometryType l_nGeomType = OGRFromOGCGeomType(pszType);
 
-            if( papszRow[1] != nullptr && atoi(papszRow[1]) == 3 )
-                l_nGeomType = wkbSetZ(l_nGeomType);
+            if( poDS->GetMajorVersion() < 8 || poDS->IsMariaDB() )
+                if( papszRow[1] != nullptr && atoi(papszRow[1]) == 3 )
+                    l_nGeomType = wkbSetZ(l_nGeomType);
 
             poDefn->SetGeomType( l_nGeomType );
         }
@@ -773,19 +774,14 @@ OGRErr OGRMySQLTableLayer::ICreateFeature( OGRFeature *poFeature )
     osCommand += ") VALUES (";
 
     // Set the geometry
-    bNeedComma = poFeature->GetGeometryRef() != nullptr;
-    if( poFeature->GetGeometryRef() != nullptr)
+    OGRGeometry *poGeom = poFeature->GetGeometryRef();
+    bNeedComma = poGeom != nullptr;
+    if( poGeom != nullptr)
     {
         char    *pszWKT = nullptr;
-
-        if( poFeature->GetGeometryRef() != nullptr )
-        {
-            OGRGeometry *poGeom = (OGRGeometry *) poFeature->GetGeometryRef();
-
-            poGeom->closeRings();
-            poGeom->flattenTo2D();
-            poGeom->exportToWkt( &pszWKT );
-        }
+        poGeom->closeRings();
+        poGeom->flattenTo2D();
+        poGeom->exportToWkt( &pszWKT );
 
         if( pszWKT != nullptr )
         {
@@ -1213,9 +1209,7 @@ GIntBig OGRMySQLTableLayer::GetFeatureCount( CPL_UNUSED int bForce )
     if( papszRow != nullptr && papszRow[0] != nullptr )
         nCount = CPLAtoGIntBig(papszRow[0]);
 
-    if( hResult != nullptr )
-        mysql_free_result( hResult );
-    hResult = nullptr;
+    mysql_free_result( hResult );
 
     return nCount;
 }
@@ -1288,12 +1282,12 @@ OGRErr OGRMySQLTableLayer::GetExtent(OGREnvelope *psExtent, CPL_UNUSED int bForc
 
             if ( poGeometry != nullptr )
             {
-                if (poGeometry && !bExtentSet)
+                if (!bExtentSet)
                 {
                     poGeometry->getEnvelope(psExtent);
                     bExtentSet = TRUE;
                 }
-                else if (poGeometry)
+                else
                 {
                     poGeometry->getEnvelope(&oEnv);
                     if (oEnv.MinX < psExtent->MinX)
