@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_ods.h 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  ODS Translator
  * Purpose:  Definition of classes for OGR OpenOfficeSpreadsheet .ods driver.
@@ -137,7 +137,7 @@ class OGRODSDataSource final: public GDALDataset
     VSILFILE*           fpSettings;
     std::string         osCurrentConfigTableName;
     std::string         osConfigName;
-    int                 nFlags;
+    int                 nVerticalSplitFlags;
     std::set<std::string> osSetLayerHasSplitter;
     void                AnalyseSettings();
 
@@ -156,6 +156,8 @@ class OGRODSDataSource final: public GDALDataset
     int                 nRowsRepeated;
     int                 nCurCol;
     int                 nCellsRepeated;
+    // Accumulated memory allocations related to repeated cells.
+    size_t              m_nAccRepeatedMemory = 0;
     bool                bEndTableParsing;
 
     OGRODSLayer        *poCurLayer;
@@ -166,6 +168,7 @@ class OGRODSDataSource final: public GDALDataset
 
     CPLString           osValueType;
     CPLString           osValue;
+    bool                m_bValueFromTableCellAttribute = false;
     std::string         osFormula;
 
     std::vector<std::string>  apoFirstLineValues;
