@@ -9,7 +9,7 @@
  ******************************************************************************
  * Copyright (c) 2011, Ben Ahmed Daho Ali
  * Copyright (c) 2013, NextGIS
- * Copyright (c) 2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -37,7 +37,7 @@
 #include "ogr_srs_api.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: ogrsxflayer.cpp dd78be7491f410ef103c05b37100754430c60420 2018-01-03 09:09:16Z Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                        OGRSXFLayer()                                 */
@@ -143,6 +143,7 @@ bool OGRSXFLayer::AddRecord( long nFID, unsigned nClassCode, vsi_l_offset nOffse
                 int nReadObj = static_cast<int>(VSIFReadL(&stAttrInfo, 4, 1, fpSXF));
                 if (nReadObj == 1)
                 {
+                    CPL_LSBPTR16(&(stAttrInfo.nCode));
                     CPLString oFieldName;
                     if (snAttributeCodes.find(stAttrInfo.nCode) == snAttributeCodes.end())
                     {
@@ -400,7 +401,9 @@ OGRFeature *OGRSXFLayer::GetNextFeature()
 int OGRSXFLayer::TestCapability( const char * pszCap )
 
 {
-    if (EQUAL(pszCap, OLCStringsAsUTF8))
+    if (EQUAL(pszCap, OLCStringsAsUTF8) &&
+        CPLCanRecode("test", "CP1251", CPL_ENC_UTF8) &&
+        CPLCanRecode("test", "KOI8-R", CPL_ENC_UTF8))
         return TRUE;
     else if (EQUAL(pszCap, OLCRandomRead))
         return TRUE;
@@ -611,11 +614,25 @@ OGRFeature *OGRSXFLayer::GetNextRawFeature(long nFID)
     int nObjectRead = static_cast<int>(
         VSIFReadL(&stRecordHeader, sizeof(SXFRecordHeader), 1, fpSXF));
 
-    if (nObjectRead != 1 || stRecordHeader.nID != IDSXFOBJ)
+    if (nObjectRead != 1)
     {
         CPLError(CE_Failure, CPLE_FileIO, "SXF. Read record failed.");
         return nullptr;
     }
+    CPL_LSBPTR32(&(stRecordHeader.nID));
+    if (stRecordHeader.nID != IDSXFOBJ)
+    {
+        CPLError(CE_Failure, CPLE_FileIO, "SXF. Read record failed.");
+        return nullptr;
+    }
+    CPL_LSBPTR32(&(stRecordHeader.nFullLength));
+    CPL_LSBPTR32(&(stRecordHeader.nGeometryLength));
+    CPL_LSBPTR32(&(stRecordHeader.nClassifyCode));
+    CPL_LSBPTR16(&(stRecordHeader.anGroup[0]));
+    CPL_LSBPTR16(&(stRecordHeader.anGroup[1]));
+    CPL_LSBPTR32(&(stRecordHeader.nPointCount));
+    CPL_LSBPTR16(&(stRecordHeader.nSubObjectCount));
+    CPL_LSBPTR16(&(stRecordHeader.nPointCountSmall));
 
     SXFGeometryType eGeomType = SXF_GT_Unknown;
     GByte code = 0;
@@ -803,7 +820,7 @@ OGRFeature *OGRSXFLayer::GetNextRawFeature(long nFID)
       return NULL;
     }
 #endif
-    else if (eGeomType == SXF_GT_TextTemplate ) // TODO realise this
+    else if (eGeomType == SXF_GT_TextTemplate ) // TODO realize this
     {
       CPLError( CE_Warning, CPLE_NotSupported,
       "SXF. Geometry type Text Template do not support." );
@@ -872,6 +889,7 @@ OGRFeature *OGRSXFLayer::GetNextRawFeature(long nFID)
             {
                 char *psSemanticsdBufBeg = psSemanticsdBuf + offset;
                 SXFRecordAttributeInfo stAttInfo = *(SXFRecordAttributeInfo*)psSemanticsdBufBeg;
+                CPL_LSBPTR16(&(stAttInfo.nCode));
                 offset += 4;
 
                 CPLString oFieldName;
@@ -986,8 +1004,9 @@ OGRFeature *OGRSXFLayer::GetNextRawFeature(long nFID)
                 }
                 case SXF_RAT_UNICODE:
                 {
-                    unsigned nLen = (unsigned(stAttInfo.nScale) + 1) * 2;
-                    if(nLen < 2 || nLen > nSemanticsSize || nSemanticsSize - nLen < offset )
+                    uint64_t nLen64 = (static_cast<uint64_t>(stAttInfo.nScale) + 1) * 2;
+                    unsigned nLen = static_cast<unsigned>(nLen64);
+                    if(/* nLen < 2 || */ nLen64 > nSemanticsSize || nSemanticsSize - nLen < offset )
                     {
                         nSemanticsSize = 0;
                         break;
@@ -1161,7 +1180,6 @@ OGRFeature *OGRSXFLayer::TranslateLine(const SXFRecordDescription& certifInfo,
     double dfY = 1.0;
     double dfZ = 0.0;
     GUInt32 nOffset = 0;
-    GUInt32 nDelta = 0;
 
     //OGRFeatureDefn *fd = poFeatureDefn->Clone();
     //fd->SetGeomType( wkbMultiLineString );
@@ -1178,6 +1196,7 @@ OGRFeature *OGRSXFLayer::TranslateLine(const SXFRecordDescription& certifInfo,
     {
         const char * psCoords = psRecordBuf + nOffset ;
 
+        GInt32 nDelta;
         if (certifInfo.bDim == 1)
         {
             nDelta = TranslateXYH( certifInfo, psCoords, nBufLen - nOffset, &dfX, &dfY, &dfZ );
@@ -1218,7 +1237,8 @@ OGRFeature *OGRSXFLayer::TranslateLine(const SXFRecordDescription& certifInfo,
 
         for (GUInt16 i=0; i < nCoords ; i++)
         {
-            const char * psCoords = psRecordBuf + nOffset ;
+            const char * psCoords = psRecordBuf + nOffset;
+            GInt32 nDelta;
             if (certifInfo.bDim == 1)
             {
                 nDelta = TranslateXYH( certifInfo, psCoords, nBufLen - nOffset, &dfX, &dfY, &dfZ );
@@ -1266,7 +1286,6 @@ OGRFeature *OGRSXFLayer::TranslateVetorAngle(const SXFRecordDescription& certifI
     }
 
     GUInt32 nOffset = 0;
-    GUInt32 nDelta = 0;
 
     OGRFeature *poFeature = new OGRFeature(poFeatureDefn);
     OGRPoint *poPT = new OGRPoint();
@@ -1282,6 +1301,7 @@ OGRFeature *OGRSXFLayer::TranslateVetorAngle(const SXFRecordDescription& certifI
         double dfX = 1.0;
         double dfY = 1.0;
         double dfZ = 0.0;
+        GInt32 nDelta;
         if (certifInfo.bDim == 1)
         {
             nDelta = TranslateXYH( certifInfo, psCoords, nBufLen - nOffset, &dfX, &dfY, &dfZ );
@@ -1439,7 +1459,6 @@ OGRFeature *OGRSXFLayer::TranslateText(const SXFRecordDescription& certifInfo,
     double dfY = 1.0;
     double dfZ = 0.0;
     GUInt32 nOffset = 0;
-    GUInt32 nDelta = 0;
 
     OGRFeature *poFeature = new OGRFeature(poFeatureDefn);
     OGRMultiLineString *poMLS = new  OGRMultiLineString ();
@@ -1452,6 +1471,7 @@ OGRFeature *OGRSXFLayer::TranslateText(const SXFRecordDescription& certifInfo,
     {
         const char * psCoords = psRecordBuf + nOffset ;
 
+        GUInt32 nDelta;
         if (certifInfo.bDim == 1)
         {
             nDelta = TranslateXYH( certifInfo, psCoords, nBufLen - nOffset,
@@ -1522,6 +1542,7 @@ OGRFeature *OGRSXFLayer::TranslateText(const SXFRecordDescription& certifInfo,
         for (int i=0; i < nCoords ; i++)
         {
             const char * psCoords = psRecordBuf + nOffset ;
+            GUInt32 nDelta;
             if (certifInfo.bDim == 1)
             {
                 nDelta = TranslateXYH( certifInfo, psCoords, nBufLen - nOffset, &dfX, &dfY, &dfZ );
