@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2012-2018, Martin Landa <landa.martin gmail.com>
- * Copyright (c) 2012-2018, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2012-2018, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -41,13 +41,13 @@
 
 #include "ogr_geometry.h"
 
-CPL_CVSID("$Id: vfkreadersqlite.cpp 4ecc7b663477fde8050d45e4e3882dc2cfdb0f0a 2018-04-29 12:13:13 +0200 Martin Landa $")
+CPL_CVSID("$Id$")
 
 /*!
   \brief VFKReaderSQLite constructor
 */
-VFKReaderSQLite::VFKReaderSQLite( const char *pszFileName ) :
-    VFKReader(pszFileName),
+VFKReaderSQLite::VFKReaderSQLite( const GDALOpenInfo* poOpenInfo ) :
+    VFKReader( poOpenInfo ),
     m_pszDBname(nullptr),
     m_poDB(nullptr),
     // True - build geometry from DB
@@ -58,12 +58,9 @@ VFKReaderSQLite::VFKReaderSQLite( const char *pszFileName ) :
 {
     size_t nLen = 0;
     VSIStatBufL sStatBufDb;
-    {
-        GDALOpenInfo *poOpenInfo = new GDALOpenInfo(pszFileName, GA_ReadOnly);
-        m_bDbSource = poOpenInfo->nHeaderBytes >= 16 &&
-            STARTS_WITH((const char*)poOpenInfo->pabyHeader, "SQLite format 3");
-        delete poOpenInfo;
-    }
+
+    m_bDbSource = poOpenInfo->nHeaderBytes >= 16 &&
+        STARTS_WITH((const char*)poOpenInfo->pabyHeader, "SQLite format 3");
 
     const char *pszDbNameConf = CPLGetConfigOption("OGR_VFK_DB_NAME", nullptr);
     CPLString osDbName;
@@ -90,8 +87,8 @@ VFKReaderSQLite::VFKReaderSQLite( const char *pszFileName ) :
     else
     {
         // m_bNewDb = false;
-        nLen = strlen(pszFileName);
-        osDbName = pszFileName;
+        nLen = strlen(m_pszFilename);
+        osDbName = m_pszFilename;
     }
 
     m_pszDBname = new char [nLen+1];
@@ -137,15 +134,14 @@ VFKReaderSQLite::VFKReaderSQLite( const char *pszFileName ) :
                  sqlite3_errmsg(m_poDB));
     }
 
-    int nRowCount = 0;
-    int nColCount = 0;
     CPLString osCommand;
     if( m_bDbSource )
     {
         /* check if it is really VFK DB datasource */
         char* pszErrMsg = nullptr;
         char** papszResult = nullptr;
-        nRowCount = nColCount = 0;
+        int nRowCount = 0;
+        int nColCount = 0;
 
         osCommand.Printf("SELECT * FROM sqlite_master WHERE type='table' AND name='%s'",
                          VFK_DB_TABLE);
@@ -169,7 +165,9 @@ VFKReaderSQLite::VFKReaderSQLite( const char *pszFileName ) :
         /* check if DB is up-to-date datasource */
         char* pszErrMsg = nullptr;
         char** papszResult = nullptr;
-        nRowCount = nColCount = 0;
+        int nRowCount = 0;
+        int nColCount = 0;
+
         osCommand.Printf("SELECT * FROM %s LIMIT 1", VFK_DB_TABLE);
         sqlite3_get_table(m_poDB,
                           osCommand.c_str(),
@@ -242,6 +240,7 @@ VFKReaderSQLite::VFKReaderSQLite( const char *pszFileName ) :
 
         /* insert S-JTSK into spatial_ref_sys table */
         poSRS = new OGRSpatialReference();
+        poSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
         if (poSRS->importFromEPSG(5514) != OGRERR_FAILURE)
         {
             char *pszWKT = nullptr;
@@ -261,7 +260,11 @@ VFKReaderSQLite::VFKReaderSQLite( const char *pszFileName ) :
 */
 VFKReaderSQLite::~VFKReaderSQLite()
 {
-    // Close tmp SQLite DB.
+    /* clean loaded properties */
+    for (int i = 0; i < m_nDataBlockCount; i++)
+        m_papoDataBlock[i]->CleanProperties();
+
+    /* close backend SQLite DB */
     if( SQLITE_OK != sqlite3_close(m_poDB) )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
@@ -270,7 +273,7 @@ VFKReaderSQLite::~VFKReaderSQLite()
     }
     CPLDebug("OGR-VFK", "Internal DB (%s) closed", m_pszDBname);
 
-    /* delete tmp SQLite DB if requested */
+    /* delete backend SQLite DB if requested */
     if( CPLTestBool(CPLGetConfigOption("OGR_VFK_DB_DELETE", "NO")) )
     {
         CPLDebug("OGR-VFK", "Internal DB (%s) deleted", m_pszDBname);
@@ -403,8 +406,8 @@ int VFKReaderSQLite::ReadDataRecords(IVFKDataBlock *poDataBlock)
 
             osSQL.Printf("SELECT %s,_rowid_ FROM %s ",
                          FID_COLUMN, pszName);
-            if (EQUAL(pszName, "SBP"))
-              osSQL += "WHERE PORADOVE_CISLO_BODU = 1 ";
+            if (EQUAL(pszName, "SBP") || EQUAL(pszName, "SBPG"))
+                osSQL += "WHERE PORADOVE_CISLO_BODU = 1 ";
             osSQL += "ORDER BY ";
             osSQL += FID_COLUMN;
             hStmt = PrepareStatement(osSQL.c_str());
@@ -454,6 +457,7 @@ int VFKReaderSQLite::ReadDataRecords(IVFKDataBlock *poDataBlock)
             if (poDataBlock && poDataBlock != poDataBlockCurrent)
                 continue;
 
+            /* update number of records in metadata table */
             osSQL.Printf("UPDATE %s SET num_records = %d WHERE "
                          "table_name = '%s'",
                          VFK_DB_TABLE, poDataBlockCurrent->GetRecordCount(),
@@ -526,7 +530,7 @@ void VFKReaderSQLite::CreateIndices()
 
         /* create index on ogr_fid */
         CreateIndex(osIndexName.c_str(), pszBlockName, FID_COLUMN,
-                    !EQUAL(pszBlockName, "SBP"));
+                    !(EQUAL(pszBlockName, "SBP") || EQUAL(pszBlockName, "SBPG")));
 
         if ( poDataBlock->GetGeometryType() == wkbNone ) {
             /* skip geometry-related indices */
@@ -540,6 +544,7 @@ void VFKReaderSQLite::CreateIndices()
             EQUAL (pszBlockName, "OP") ||
             EQUAL (pszBlockName, "OBPEJ") ||
             EQUAL (pszBlockName, "SBP") ||
+            EQUAL (pszBlockName, "SBPG") ||
             EQUAL (pszBlockName, "HP") ||
             EQUAL (pszBlockName, "DPM") ||
             EQUAL (pszBlockName, "ZVB") ||
@@ -825,13 +830,13 @@ OGRErr VFKReaderSQLite::AddFeature( IVFKDataBlock *poDataBlock,
     if (poDataBlock->GetGeometryType() != wkbNone) {
         osValue += ",NULL";
     }
-    osValue += ")";
     osCommand += osValue;
+    osCommand += ")";
 
     if( ExecuteSQL(osCommand.c_str(), CE_Warning) != OGRERR_NONE )
         return OGRERR_FAILURE;
 
-    if (EQUAL(pszBlockName, "SBP")) {
+    if ( EQUAL(pszBlockName, "SBP") || EQUAL(pszBlockName, "SBPG") ) {
         poProperty = poFeature->GetProperty("PORADOVE_CISLO_BODU");
         if( poProperty == nullptr )
         {
