@@ -49,7 +49,7 @@
 #include "cpl_string.h"
 #include "cpl_vsi.h"
 
-CPL_CVSID("$Id: cpl_path.cpp c6d968cff9ac8a35496596945b63b5ca6fc74be5 2020-09-09 15:52:31 +0200 Thomas Bonfort $")
+CPL_CVSID("$Id$")
 
 // Should be size of larged possible filename.
 constexpr int CPL_PATH_BUF_SIZE = 2048;
@@ -392,12 +392,24 @@ const char *CPLGetExtension( const char *pszFullFilename )
  * If HAVE_GETCWD macro is not defined, the function returns NULL.
  **/
 
-#ifdef HAVE_GETCWD
+#ifdef _WIN32
 char *CPLGetCurrentDir()
 {
-#ifdef _MAX_PATH
     const size_t nPathMax = _MAX_PATH;
-#elif PATH_MAX
+    wchar_t* pwszDirPath = static_cast<wchar_t *>( VSI_MALLOC_VERBOSE( nPathMax * sizeof(wchar_t) ) );
+    char* pszRet = nullptr;
+    if( pwszDirPath != nullptr &&
+        _wgetcwd(pwszDirPath, nPathMax) != nullptr )
+    {
+        pszRet = CPLRecodeFromWChar(pwszDirPath,CPL_ENC_UCS2,CPL_ENC_UTF8);
+    }
+    CPLFree(pwszDirPath);
+    return pszRet;
+}
+#elif defined(HAVE_GETCWD)
+char *CPLGetCurrentDir()
+{
+#if PATH_MAX
     const size_t nPathMax = PATH_MAX;
 #else
     const size_t nPathMax = 8192;
@@ -494,6 +506,7 @@ static bool RequiresUnixPathSeparator(const char* pszPath)
             STARTS_WITH(pszPath, "/vsigs_streaming/") ||
             STARTS_WITH(pszPath, "/vsiaz/") ||
             STARTS_WITH(pszPath, "/vsiaz_streaming/") ||
+            STARTS_WITH(pszPath, "/vsiadls/") ||
             STARTS_WITH(pszPath, "/vsioss/") ||
             STARTS_WITH(pszPath, "/vsioss_streaming/") ||
             STARTS_WITH(pszPath, "/vsiswift/") ||
@@ -937,8 +950,8 @@ const char *CPLExtractRelativePath( const char *pszBaseDir,
  * <pre>
  * CPLCleanTrailingSlash( "abc/def/" ) == "abc/def"
  * CPLCleanTrailingSlash( "abc/def" ) == "abc/def"
- * CPLCleanTrailingSlash( "c:\abc\def\" ) == "c:\abc\def"
- * CPLCleanTrailingSlash( "c:\abc\def" ) == "c:\abc\def"
+ * CPLCleanTrailingSlash( "c:\\abc\\def\\" ) == "c:\\abc\\def"
+ * CPLCleanTrailingSlash( "c:\\abc\\def" ) == "c:\\abc\\def"
  * CPLCleanTrailingSlash( "abc" ) == "abc"
  * </pre>
  *
