@@ -102,7 +102,6 @@ struct WriteFuncStruct
     char*           pBuffer = nullptr;
     size_t          nSize = 0;
     bool            bIsHTTP = false;
-    bool            bIsInHeader = false;
     bool            bMultiRange = false;
     vsi_l_offset    nStartOffset = 0;
     vsi_l_offset    nEndOffset = 0;
@@ -110,7 +109,7 @@ struct WriteFuncStruct
     vsi_l_offset    nContentLength = 0;
     bool            bFoundContentRange = false;
     bool            bError = false;
-    bool            bDownloadHeaderOnly = false;
+    bool            bInterruptDownload = false;
     bool            bDetectRangeDownloadingError = false;
     GIntBig         nTimestampDate = 0; // Corresponds to Date: header field
 
@@ -152,9 +151,9 @@ struct PutData
 
 class VSICurlHandle;
 
-class VSICurlFilesystemHandler : public VSIFilesystemHandler
+class VSICurlFilesystemHandlerBase : public VSIFilesystemHandler
 {
-    CPL_DISALLOW_COPY_ASSIGN(VSICurlFilesystemHandler)
+    CPL_DISALLOW_COPY_ASSIGN(VSICurlFilesystemHandlerBase)
 
     struct FilenameOffsetPair
     {
@@ -192,7 +191,12 @@ class VSICurlFilesystemHandler : public VSIFilesystemHandler
     std::unique_ptr<RegionCacheType> m_poRegionCacheDoNotUseDirectly{}; // do not access directly. Use GetRegionCache();
     RegionCacheType* GetRegionCache();
 
-    lru11::Cache<std::string, FileProp>  oCacheFileProp;
+    // LRU cache that just keeps in memory if this file system handler is
+    // spposed to know the file properties of a file. The actual cache is a
+    // shared one among all network file systems.
+    // The aim of that design is that invalidating /vsis3/foo results in
+    // /vsis3_streaming/foo to be invalidated as well.
+    lru11::Cache<std::string, bool>  oCacheFileProp;
 
     int                                       nCachedFilesInDirList = 0;
     lru11::Cache<std::string, CachedDirList>  oCacheDirList;
@@ -216,7 +220,7 @@ protected:
                             const char* pszXML,
                             CPLStringList& osFileList,
                             int nMaxFiles,
-                            bool bIgnoreGlacierStorageClass,
+                            const std::set<std::string>& oSetIgnoredStorageClasses,
                             bool& bIsTruncated );
 
     void AnalyseSwiftFileList( const CPLString& osBaseURL,
@@ -232,9 +236,10 @@ protected:
 
     static bool IsAllowedFilename( const char* pszFilename );
 
+    VSICurlFilesystemHandlerBase();
+
 public:
-    VSICurlFilesystemHandler();
-    ~VSICurlFilesystemHandler() override;
+    ~VSICurlFilesystemHandlerBase() override;
 
     VSIVirtualHandle *Open( const char *pszFilename,
                             const char *pszAccess,
@@ -266,9 +271,9 @@ public:
                             bool* pbGotFileList );
     void InvalidateDirContent( const char *pszDirname );
 
-    virtual const char* GetDebugKey() const { return "VSICURL"; }
+    virtual const char* GetDebugKey() const = 0;
 
-    virtual CPLString GetFSPrefix() const { return "/vsicurl/"; }
+    virtual CPLString GetFSPrefix() const = 0;
     virtual bool      AllowCachedDataFor(const char* pszFilename);
 
     std::shared_ptr<std::string> GetRegion( const char* pszURL,
@@ -298,6 +303,25 @@ public:
     bool ExistsInCacheDirList( const CPLString& osDirname, bool *pbIsDir );
 
     virtual CPLString GetURLFromFilename( const CPLString& osFilename );
+
+    std::string GetStreamingFilename(const std::string& osFilename) const override = 0;
+
+    static std::set<std::string> GetS3IgnoredStorageClasses();
+};
+
+
+class VSICurlFilesystemHandler: public VSICurlFilesystemHandlerBase
+{
+    CPL_DISALLOW_COPY_ASSIGN(VSICurlFilesystemHandler)
+
+public:
+    VSICurlFilesystemHandler() = default;
+
+    const char* GetDebugKey() const override { return "VSICURL"; }
+
+    CPLString GetFSPrefix() const override { return "/vsicurl/"; }
+
+    std::string GetStreamingFilename(const std::string& osFilename) const override;
 };
 
 /************************************************************************/
@@ -309,7 +333,7 @@ class VSICurlHandle : public VSIVirtualHandle
     CPL_DISALLOW_COPY_ASSIGN(VSICurlHandle)
 
   protected:
-    VSICurlFilesystemHandler* poFS = nullptr;
+    VSICurlFilesystemHandlerBase* poFS = nullptr;
 
     bool            m_bCached = true;
 
@@ -348,6 +372,7 @@ class VSICurlHandle : public VSIVirtualHandle
     virtual std::string DownloadRegion(vsi_l_offset startOffset, int nBlocks);
 
     bool                m_bUseHead = false;
+    bool                m_bUseRedirectURLIfNoQueryStringParams = false;
 
     int          ReadMultiRangeSingleGet( int nRanges, void ** ppData,
                                          const vsi_l_offset* panOffsets,
@@ -364,11 +389,11 @@ class VSICurlHandle : public VSIVirtualHandle
     virtual bool IsDirectoryFromExists( const char* /*pszVerb*/, int /*response_code*/ ) { return false; }
     virtual void ProcessGetFileSizeResult(const char* /* pszContent */ ) {}
     void SetURL(const char* pszURL);
-    virtual bool Authenticate() { return false; }
+    virtual bool Authenticate(const char* /* pszFilename */) { return false; }
 
   public:
 
-    VSICurlHandle( VSICurlFilesystemHandler* poFS,
+    VSICurlHandle( VSICurlFilesystemHandlerBase* poFS,
                    const char* pszFilename,
                    const char* pszURLIn = nullptr );
     ~VSICurlHandle() override;
@@ -405,7 +430,7 @@ class VSICurlHandle : public VSIVirtualHandle
 /*                        IVSIS3LikeFSHandler                           */
 /************************************************************************/
 
-class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
+class IVSIS3LikeFSHandler: public VSICurlFilesystemHandlerBase
 {
     CPL_DISALLOW_COPY_ASSIGN(IVSIS3LikeFSHandler)
 
@@ -413,6 +438,7 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
                      vsi_l_offset nSourceSize,
                      const char* pszSource,
                      const char* pszTarget,
+                     CSLConstList papszOptions,
                      GDALProgressFunc pProgressFunc,
                      void *pProgressData);
     virtual int MkdirInternal( const char *pszDirname, long nMode, bool bDoStatCheck );
@@ -428,6 +454,10 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
     virtual int      CopyObject( const char *oldpath, const char *newpath,
                                  CSLConstList papszMetadata );
 
+    int RmdirRecursiveInternal( const char* pszDirname, int nBatchSize);
+
+    virtual bool IsAllowedHeaderForObjectCreation( const char* /* pszHeaderName */ ) { return false; }
+
     IVSIS3LikeFSHandler() = default;
 
   public:
@@ -443,8 +473,6 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
     virtual void UpdateMapFromHandle(IVSIS3LikeHandleHelper*) {}
     virtual void UpdateHandleFromMap( IVSIS3LikeHandleHelper * ) {}
 
-    virtual CPLString GetStreamingPath( const char* pszFilename ) const;
-
     bool Sync( const char* pszSource, const char* pszTarget,
                 const char* const * papszOptions,
                 GDALProgressFunc pProgressFunc,
@@ -471,7 +499,8 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
                          size_t nBufferSize,
                          IVSIS3LikeHandleHelper *poS3HandleHelper,
                          int nMaxRetry,
-                         double dfRetryDelay);
+                         double dfRetryDelay,
+                         CSLConstList papszOptions);
     virtual bool CompleteMultipart(const CPLString& osFilename,
                            const CPLString& osUploadID,
                            const std::vector<CPLString>& aosEtags,
@@ -484,6 +513,8 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
                         IVSIS3LikeHandleHelper *poS3HandleHelper,
                         int nMaxRetry,
                         double dfRetryDelay);
+
+    bool    AbortPendingUploads(const char* pszFilename) override;
 };
 
 /************************************************************************/
@@ -509,7 +540,7 @@ class IVSIS3LikeHandle:  public VSICurlHandle
         }
 
   public:
-    IVSIS3LikeHandle( VSICurlFilesystemHandler* poFSIn,
+    IVSIS3LikeHandle( VSICurlFilesystemHandlerBase* poFSIn,
                       const char* pszFilename,
                       const char* pszURLIn ) :
         VSICurlHandle(poFSIn, pszFilename, pszURLIn) {}
@@ -591,7 +622,7 @@ class VSIAppendWriteHandle : public VSIVirtualHandle
 
     protected:
 
-    VSICurlFilesystemHandler* m_poFS = nullptr;
+    VSICurlFilesystemHandlerBase* m_poFS = nullptr;
     CPLString           m_osFSPrefix{};
     CPLString           m_osFilename{};
 
@@ -608,7 +639,7 @@ class VSIAppendWriteHandle : public VSIVirtualHandle
     virtual bool        Send(bool bIsLastBlock) = 0;
 
     public:
-        VSIAppendWriteHandle( VSICurlFilesystemHandler* poFS,
+        VSIAppendWriteHandle( VSICurlFilesystemHandlerBase* poFS,
                               const char* pszFSPrefix,
                               const char* pszFilename,
                               int nChunkSize );
@@ -624,6 +655,21 @@ class VSIAppendWriteHandle : public VSIVirtualHandle
         bool              IsOK() { return m_pabyBuffer != nullptr; }
 };
 
+/************************************************************************/
+/*                     VSIDIRWithMissingDirSynthesis                    */
+/************************************************************************/
+
+struct VSIDIRWithMissingDirSynthesis: public VSIDIR
+{
+    std::vector<std::unique_ptr<VSIDIREntry>> aoEntries{};
+
+protected:
+    std::vector<std::string> m_aosSubpathsStack{};
+
+    void SynthetizeMissingDirectories(const std::string& osCurSubdir,
+                                      bool bAddEntryForThisSubdir);
+};
+
 /************************************************************************/
 /*                         CurlRequestHelper                            */
 /************************************************************************/
@@ -638,7 +684,7 @@ struct CurlRequestHelper
     ~CurlRequestHelper();
     long perform(CURL* hCurlHandle,
                  struct curl_slist* headers, // ownership transferred
-                 VSICurlFilesystemHandler *poFS,
+                 VSICurlFilesystemHandlerBase *poFS,
                  IVSIS3LikeHandleHelper *poS3HandleHelper);
 };
 
@@ -701,6 +747,8 @@ class NetworkStatisticsLogger
         void AsJSON(CPLJSONObject& oJSON) const;
     };
 
+    // Workaround bug in Coverity Scan
+    // coverity[generated_default_constructor_used_in_field_initializer]
     Stats m_stats{};
     std::map<GIntBig, std::vector<ContextPathItem>> m_mapThreadIdToContextPath{};
 
@@ -798,6 +846,15 @@ void VSICURLResetHeaderAndWriterFunctions(CURL* hCurlHandle);
 
 int VSICurlParseUnixPermissions(const char* pszPermissions);
 
+// Cache of file properties (size, etc.)
+bool VSICURLGetCachedFileProp( const char* pszURL,
+                               FileProp& oFileProp );
+void VSICURLSetCachedFileProp( const char* pszURL,
+                               FileProp& oFileProp );
+void VSICURLInvalidateCachedFileProp( const char* pszURL );
+void VSICURLInvalidateCachedFilePropPrefix( const char* pszURL );
+void VSICURLDestroyCacheFileProp();
+
 } // namespace cpl
 
 //! @endcond
