@@ -42,7 +42,7 @@
 
 #include "cpl_swift.h"
 
-CPL_CVSID("$Id: cpl_vsil_swift.cpp a044c83f8091becdd11e27be6e9c08d0d3478126 2021-02-24 11:38:17 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 #ifndef HAVE_CURL
 
@@ -58,13 +58,15 @@ void VSIInstallSwiftFileHandler( void )
 
 #define ENABLE_DEBUG 0
 
+#define unchecked_curl_easy_setopt(handle,opt,param) CPL_IGNORE_RET_VAL(curl_easy_setopt(handle,opt,param))
+
 namespace cpl {
 
 /************************************************************************/
 /*                       AnalyseSwiftFileList()                         */
 /************************************************************************/
 
-void VSICurlFilesystemHandler::AnalyseSwiftFileList(
+void VSICurlFilesystemHandlerBase::AnalyseSwiftFileList(
     const CPLString& osBaseURL,
     const CPLString& osPrefix,
     const char* pszJson,
@@ -250,11 +252,13 @@ public:
         VSIDIR* OpenDir( const char *pszPath, int nRecurseDepth,
                                 const char* const *papszOptions) override
         {
-            return VSICurlFilesystemHandler::OpenDir(pszPath, nRecurseDepth,
+            return VSICurlFilesystemHandlerBase::OpenDir(pszPath, nRecurseDepth,
                                                      papszOptions);
         }
 
         const char* GetOptions() override;
+
+        std::string GetStreamingFilename(const std::string& osFilename) const override { return osFilename; }
 };
 
 /************************************************************************/
@@ -271,7 +275,7 @@ class VSISwiftHandle final : public IVSIS3LikeHandle
     struct curl_slist* GetCurlHeaders(
         const CPLString& osVerb,
         const struct curl_slist* psExistingHeaders ) override;
-    virtual bool Authenticate() override;
+    virtual bool Authenticate(const char* pszFilename) override;
 
   public:
     VSISwiftHandle( VSISwiftFSHandler* poFS,
@@ -325,7 +329,7 @@ VSIVirtualHandle* VSISwiftFSHandler::Open( const char *pszFilename,
     }
 
     return
-        VSICurlFilesystemHandler::Open(pszFilename, pszAccess, bSetError, papszOptions);
+        VSICurlFilesystemHandlerBase::Open(pszFilename, pszAccess, bSetError, papszOptions);
 }
 
 /************************************************************************/
@@ -344,7 +348,7 @@ VSISwiftFSHandler::~VSISwiftFSHandler()
 
 void VSISwiftFSHandler::ClearCache()
 {
-    VSICurlFilesystemHandler::ClearCache();
+    VSICurlFilesystemHandlerBase::ClearCache();
 
     VSISwiftHandleHelper::ClearCache();
 }
@@ -367,8 +371,24 @@ const char* VSISwiftFSHandler::GetOptions()
     "  <Option name='SWIFT_USER' type='string' "
         "description='User name to use with authentication V1'/>"
     "  <Option name='SWIFT_KEY' type='string' "
-        "description='Key/password to use with authentication V1'/>" +
-        VSICurlFilesystemHandler::GetOptionsStatic() +
+        "description='Key/password to use with authentication V1'/>"
+    "  <Option name='OS_IDENTITY_API_VERSION' type='string' "
+        "description='OpenStack identity API version'/>"
+    "  <Option name='OS_AUTH_TYPE' type='string' "
+        "description='Authentication URL'/>"
+    "  <Option name='OS_USERNAME' type='string' "
+        "description='User name'/>"
+    "  <Option name='OS_PASSWORD' type='string' "
+        "description='Password'/>"
+    "  <Option name='OS_USER_DOMAIN_NAME' type='string' "
+        "description='User domain name'/>"
+    "  <Option name='OS_PROJECT_NAME' type='string' "
+        "description='Project name'/>"
+    "  <Option name='OS_PROJECT_DOMAIN_NAME' type='string' "
+        "description='Project domain name'/>"
+    "  <Option name='OS_REGION_NAME' type='string' "
+        "description='Region name'/>"
+    +  VSICurlFilesystemHandlerBase::GetOptionsStatic() +
         "</Options>");
     return osOptions.c_str();
 }
@@ -434,13 +454,16 @@ int VSISwiftFSHandler::Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
     if( !STARTS_WITH_CI(pszFilename, GetFSPrefix()) )
         return -1;
 
+    if( (nFlags & VSI_STAT_CACHE_ONLY) != 0 )
+        return VSICurlFilesystemHandlerBase::Stat(pszFilename, pStatBuf, nFlags);
+
     CPLString osFilename(pszFilename);
     if( osFilename.back() == '/' )
         osFilename.resize( osFilename.size() - 1 );
 
     memset(pStatBuf, 0, sizeof(VSIStatBufL));
 
-    if( VSICurlFilesystemHandler::Stat(pszFilename, pStatBuf, nFlags) == 0 )
+    if( VSICurlFilesystemHandlerBase::Stat(pszFilename, pStatBuf, nFlags) == 0 )
     {
         // if querying /vsiswift/container_name, the GET will succeed and
         // we would consider this as a file whereas it should be exposed as
@@ -448,18 +471,19 @@ int VSISwiftFSHandler::Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
         if( std::count(osFilename.begin(), osFilename.end(), '/') <= 2 )
         {
 
-            IVSIS3LikeHandleHelper* poS3HandleHelper =
-                CreateHandleHelper(pszFilename + GetFSPrefix().size(), true);
-            CPLString osURL(poS3HandleHelper->GetURL());
-            delete poS3HandleHelper;
-
-            FileProp cachedFileProp;
-            cachedFileProp.eExists = EXIST_YES;
-            cachedFileProp.bHasComputedFileSize = false;
-            cachedFileProp.fileSize = 0;
-            cachedFileProp.bIsDirectory = true;
-            cachedFileProp.mTime = 0;
-            SetCachedFileProp(osURL, cachedFileProp);
+            auto poHandleHelper = std::unique_ptr<IVSIS3LikeHandleHelper>(
+                CreateHandleHelper(pszFilename + GetFSPrefix().size(), true));
+            if( poHandleHelper )
+            {
+                FileProp cachedFileProp;
+                cachedFileProp.eExists = EXIST_YES;
+                cachedFileProp.bHasComputedFileSize = false;
+                cachedFileProp.fileSize = 0;
+                cachedFileProp.bIsDirectory = true;
+                cachedFileProp.mTime = 0;
+                cachedFileProp.nMode = S_IFDIR;
+                SetCachedFileProp(poHandleHelper->GetURL(), cachedFileProp);
+            }
 
             pStatBuf->st_size = 0;
             pStatBuf->st_mode = S_IFDIR;
@@ -476,10 +500,31 @@ int VSISwiftFSHandler::Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
     int nRet = CSLFindStringCaseSensitive(papszContents,
                     CPLGetFilename(osFilename)) >= 0 ? 0 : -1;
     CSLDestroy(papszContents);
+
+    FileProp cachedFileProp;
     if( nRet == 0 )
     {
         pStatBuf->st_mode = S_IFDIR;
+
+        cachedFileProp.eExists = EXIST_YES;
+        cachedFileProp.bHasComputedFileSize = false;
+        cachedFileProp.fileSize = 0;
+        cachedFileProp.bIsDirectory = true;
+        cachedFileProp.mTime = 0;
+        cachedFileProp.nMode = S_IFDIR;
     }
+    else
+    {
+        cachedFileProp.eExists = EXIST_NO;
+    }
+
+    auto poHandleHelper = std::unique_ptr<IVSIS3LikeHandleHelper>(
+        CreateHandleHelper(pszFilename + GetFSPrefix().size(), true));
+    if( poHandleHelper )
+    {
+        SetCachedFileProp(poHandleHelper->GetURL(), cachedFileProp);
+    }
+
     return nRet;
 }
 
@@ -564,27 +609,27 @@ char** VSISwiftFSHandler::GetFileList( const char *pszDirname,
             struct curl_slist* headers =
                 VSICurlSetOptions(hCurlHandle, poS3HandleHelper->GetURL(), nullptr);
             // Disable automatic redirection
-            curl_easy_setopt(hCurlHandle, CURLOPT_FOLLOWLOCATION, 0 );
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_FOLLOWLOCATION, 0 );
 
-            curl_easy_setopt(hCurlHandle, CURLOPT_RANGE, nullptr);
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_RANGE, nullptr);
 
             VSICURLInitWriteFuncStruct(&sWriteFuncData, nullptr, nullptr, nullptr);
-            curl_easy_setopt(hCurlHandle, CURLOPT_WRITEDATA, &sWriteFuncData);
-            curl_easy_setopt(hCurlHandle, CURLOPT_WRITEFUNCTION,
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_WRITEDATA, &sWriteFuncData);
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_WRITEFUNCTION,
                             VSICurlHandleWriteFunc);
 
             WriteFuncStruct sWriteFuncHeaderData;
             VSICURLInitWriteFuncStruct(&sWriteFuncHeaderData, nullptr, nullptr, nullptr);
-            curl_easy_setopt(hCurlHandle, CURLOPT_HEADERDATA, &sWriteFuncHeaderData);
-            curl_easy_setopt(hCurlHandle, CURLOPT_HEADERFUNCTION,
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_HEADERDATA, &sWriteFuncHeaderData);
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_HEADERFUNCTION,
                             VSICurlHandleWriteFunc);
 
             char szCurlErrBuf[CURL_ERROR_SIZE+1] = {};
-            curl_easy_setopt(hCurlHandle, CURLOPT_ERRORBUFFER, szCurlErrBuf );
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_ERRORBUFFER, szCurlErrBuf );
 
             headers = VSICurlMergeHeaders(headers,
                                 poS3HandleHelper->GetCurlHeaders("GET", headers));
-            curl_easy_setopt(hCurlHandle, CURLOPT_HTTPHEADER, headers);
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_HTTPHEADER, headers);
 
             MultiPerform(hCurlMultiHandle, hCurlHandle);
 
@@ -702,9 +747,9 @@ struct curl_slist* VSISwiftHandle::GetCurlHeaders( const CPLString& osVerb,
 /*                           Authenticate()                             */
 /************************************************************************/
 
-bool VSISwiftHandle::Authenticate()
+bool VSISwiftHandle::Authenticate(const char* pszFilename)
 {
-    return m_poHandleHelper->Authenticate();
+    return m_poHandleHelper->Authenticate(pszFilename);
 }
 
 
@@ -719,13 +764,15 @@ bool VSISwiftHandle::Authenticate()
 /*                     VSIInstallSwiftFileHandler()                     */
 /************************************************************************/
 
-/**
- * \brief Install /vsiswift/ OpenStack Swif Object Storage (Swift) file
- * system handler (requires libcurl)
- *
- * @see <a href="gdal_virtual_file_systems.html#gdal_virtual_file_systems_vsiswift">/vsiswift/ documentation</a>
- *
- * @since GDAL 2.3
+/*!
+ \brief Install /vsiswift/ OpenStack Swif Object Storage (Swift) file
+ system handler (requires libcurl)
+
+ \verbatim embed:rst
+ See :ref:`/vsiswift/ documentation <vsiswift>`
+ \endverbatim
+
+ @since GDAL 2.3
  */
 void VSIInstallSwiftFileHandler( void )
 {
