@@ -7,7 +7,7 @@
  *
  **********************************************************************
  * Copyright (c) 2001, Frank Warmerdam
- * Copyright (c) 2010-2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2010-2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -78,6 +78,7 @@
 #include <unistd.h>
 #endif
 
+#include <limits>
 #include <new>
 
 #include "cpl_config.h"
@@ -87,7 +88,7 @@
 #include "cpl_string.h"
 #include "cpl_vsi_error.h"
 
-CPL_CVSID("$Id: cpl_vsil_unix_stdio_64.cpp 5524ee5324f7bd364d391d842a6488c90c0186a7 2018-04-02 16:20:13 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 #if defined(UNIX_STDIO_64)
 
@@ -133,6 +134,18 @@ CPL_CVSID("$Id: cpl_vsil_unix_stdio_64.cpp 5524ee5324f7bd364d391d842a6488c90c018
 
 #endif /* ndef UNIX_STDIO_64 */
 
+#ifndef BUILD_WITHOUT_64BIT_OFFSET
+// Ensure we have working 64 bit API
+static_assert( sizeof(VSI_FTELL64(nullptr)) == sizeof(vsi_l_offset),
+               "File API does not seem to support 64-bit offset. "
+               "If you still want to build GDAL without > 4GB file support, "
+               "add the -DBUILD_WITHOUT_64BIT_OFFSET define" );
+static_assert( sizeof(VSIStatBufL::st_size) == sizeof(vsi_l_offset),
+               "File API does not seem to support 64-bit file size. "
+               "If you still want to build GDAL without > 4GB file support, "
+               "add the -DBUILD_WITHOUT_64BIT_OFFSET define" );
+#endif
+
 /************************************************************************/
 /* ==================================================================== */
 /*                       VSIUnixStdioFilesystemHandler                  */
@@ -141,20 +154,23 @@ CPL_CVSID("$Id: cpl_vsil_unix_stdio_64.cpp 5524ee5324f7bd364d391d842a6488c90c018
 
 class VSIUnixStdioFilesystemHandler final : public VSIFilesystemHandler
 {
+    CPL_DISALLOW_COPY_ASSIGN(VSIUnixStdioFilesystemHandler)
+
 #ifdef VSI_COUNT_BYTES_READ
-    vsi_l_offset  nTotalBytesRead;
-    CPLMutex     *hMutex;
+    vsi_l_offset  nTotalBytesRead  = 0;
+    CPLMutex     *hMutex = nullptr;
 #endif
 
 public:
-    VSIUnixStdioFilesystemHandler();
+    VSIUnixStdioFilesystemHandler() = default;
 #ifdef VSI_COUNT_BYTES_READ
     ~VSIUnixStdioFilesystemHandler() override;
 #endif
 
     VSIVirtualHandle *Open( const char *pszFilename,
                             const char *pszAccess,
-                            bool bSetError ) override;
+                            bool bSetError,
+                            CSLConstList /* papszOptions */ ) override;
     int Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
               int nFlags ) override;
     int Unlink( const char *pszFilename ) override;
@@ -165,6 +181,9 @@ public:
     GIntBig GetDiskFreeSpace( const char* pszDirname ) override;
     int SupportsSparseFiles( const char* pszPath ) override;
 
+    VSIDIR* OpenDir( const char *pszPath, int nRecurseDepth,
+                             const char* const *papszOptions) override;
+
 #ifdef VSI_COUNT_BYTES_READ
     void             AddToTotal(vsi_l_offset nBytes);
 #endif
@@ -178,20 +197,22 @@ public:
 
 class VSIUnixStdioHandle final : public VSIVirtualHandle
 {
-    FILE          *fp;
-    vsi_l_offset  m_nOffset;
-    bool          bReadOnly;
-    bool          bLastOpWrite;
-    bool          bLastOpRead;
-    bool          bAtEOF;
-    // In a+ mode, disable any optimization since the behaviour of the file
+    CPL_DISALLOW_COPY_ASSIGN(VSIUnixStdioHandle)
+
+    FILE          *fp = nullptr;
+    vsi_l_offset  m_nOffset = 0;
+    bool          bReadOnly = true;
+    bool          bLastOpWrite = false;
+    bool          bLastOpRead = false;
+    bool          bAtEOF = false;
+    // In a+ mode, disable any optimization since the behavior of the file
     // pointer on Mac and other BSD system is to have a seek() to the end of
     // file and thus a call to our Seek(0, SEEK_SET) before a read will be a
     // no-op.
-    bool          bModeAppendReadWrite;
+    bool          bModeAppendReadWrite = false;
 #ifdef VSI_COUNT_BYTES_READ
-    vsi_l_offset  nTotalBytesRead;
-    VSIUnixStdioFilesystemHandler *poFS;
+    vsi_l_offset  nTotalBytesRead = 0;
+    VSIUnixStdioFilesystemHandler *poFS = nullptr;
 #endif
   public:
     VSIUnixStdioHandle( VSIUnixStdioFilesystemHandler *poFSIn,
@@ -224,15 +245,10 @@ CPL_UNUSED
                                        FILE* fpIn, bool bReadOnlyIn,
                                        bool bModeAppendReadWriteIn) :
     fp(fpIn),
-    m_nOffset(0),
     bReadOnly(bReadOnlyIn),
-    bLastOpWrite(false),
-    bLastOpRead(false),
-    bAtEOF(false),
     bModeAppendReadWrite(bModeAppendReadWriteIn)
 #ifdef VSI_COUNT_BYTES_READ
     ,
-    nTotalBytesRead(0),
     poFS(poFSIn)
 #endif
 {}
@@ -287,6 +303,15 @@ int VSIUnixStdioHandle::Seek( vsi_l_offset nOffsetIn, int nWhence )
         }
     }
 
+#if !defined(UNIX_STDIO_64) && SIZEOF_UNSIGNED_LONG == 4
+    if( nOffsetIn > static_cast<vsi_l_offset>(std::numeric_limits<long>::max()) )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Attempt at seeking beyond long extent. Lack of 64-bit file I/O");
+        return -1;
+    }
+#endif
+
     const int nResult = VSI_FSEEK64( fp, nOffsetIn, nWhence );
     const int nError = errno;
 
@@ -588,17 +613,6 @@ VSIRangeStatus VSIUnixStdioHandle::GetRangeStatus( vsi_l_offset
 /* ==================================================================== */
 /************************************************************************/
 
-/************************************************************************/
-/*                      VSIUnixStdioFilesystemHandler()                 */
-/************************************************************************/
-
-VSIUnixStdioFilesystemHandler::VSIUnixStdioFilesystemHandler()
-#ifdef VSI_COUNT_BYTES_READ
-     : nTotalBytesRead(0),
-       hMutex(nullptr)
-#endif
-{}
-
 #ifdef VSI_COUNT_BYTES_READ
 /************************************************************************/
 /*                     ~VSIUnixStdioFilesystemHandler()                 */
@@ -624,7 +638,8 @@ VSIUnixStdioFilesystemHandler::~VSIUnixStdioFilesystemHandler()
 VSIVirtualHandle *
 VSIUnixStdioFilesystemHandler::Open( const char *pszFilename,
                                      const char *pszAccess,
-                                     bool bSetError )
+                                     bool bSetError,
+                                     CSLConstList /* papszOptions */ )
 
 {
     FILE *fp = VSI_FOPEN64( pszFilename, pszAccess );
@@ -832,7 +847,21 @@ int VSIUnixStdioFilesystemHandler::SupportsSparseFiles( const char*
             case 0x4d44U: // msdos
                 return FALSE;
 
+            case 0x53464846U:  // Windows Subsystem for Linux fs
+            {
+                static bool bUnknownFSEmitted = false;
+                if( !bUnknownFSEmitted )
+                {
+                    CPLDebug("VSI", "Windows Subsystem for Linux FS is at "
+                             "the time of writing not known to support sparse "
+                             "files");
+                    bUnknownFSEmitted = true;
+                }
+                return FALSE;
+            }
+
             default:
+            {
                 static bool bUnknownFSEmitted = false;
                 if( !bUnknownFSEmitted )
                 {
@@ -842,6 +871,7 @@ int VSIUnixStdioFilesystemHandler::SupportsSparseFiles( const char*
                     bUnknownFSEmitted = true;
                 }
                 return FALSE;
+            }
         }
     }
     return FALSE;
@@ -858,6 +888,205 @@ int VSIUnixStdioFilesystemHandler::SupportsSparseFiles( const char*
 #endif
 }
 
+/************************************************************************/
+/*                            VSIDIRUnixStdio                           */
+/************************************************************************/
+
+struct VSIDIRUnixStdio final: public VSIDIR
+{
+    CPLString osRootPath{};
+    CPLString osBasePath{};
+    DIR* m_psDir = nullptr;
+    int nRecurseDepth = 0;
+    VSIDIREntry entry{};
+    std::vector<VSIDIRUnixStdio*> aoStackSubDir{};
+    VSIUnixStdioFilesystemHandler* poFS = nullptr;
+    std::string m_osFilterPrefix{};
+    bool m_bNameAndTypeOnly = false;
+
+    explicit VSIDIRUnixStdio(VSIUnixStdioFilesystemHandler* poFSIn): poFS(poFSIn) {}
+    ~VSIDIRUnixStdio();
+
+    const VSIDIREntry* NextDirEntry() override;
+
+    VSIDIRUnixStdio(const VSIDIRUnixStdio&) = delete;
+    VSIDIRUnixStdio& operator=(const VSIDIRUnixStdio&) = delete;
+};
+
+/************************************************************************/
+/*                        ~VSIDIRUnixStdio()                            */
+/************************************************************************/
+
+VSIDIRUnixStdio::~VSIDIRUnixStdio()
+{
+    while( !aoStackSubDir.empty() )
+    {
+        delete aoStackSubDir.back();
+        aoStackSubDir.pop_back();
+    }
+    closedir(m_psDir);
+}
+
+/************************************************************************/
+/*                            OpenDir()                                 */
+/************************************************************************/
+
+VSIDIR* VSIUnixStdioFilesystemHandler::OpenDir( const char *pszPath,
+                                       int nRecurseDepth,
+                                       const char* const * papszOptions)
+{
+    DIR* psDir = opendir(pszPath);
+    if( psDir == nullptr )
+    {
+        return nullptr;
+    }
+    VSIDIRUnixStdio* dir = new VSIDIRUnixStdio(this);
+    dir->osRootPath = pszPath;
+    dir->nRecurseDepth = nRecurseDepth;
+    dir->m_psDir = psDir;
+    dir->m_osFilterPrefix = CSLFetchNameValueDef(papszOptions, "PREFIX", "");
+    dir->m_bNameAndTypeOnly = CPLTestBool(CSLFetchNameValueDef(papszOptions, "NAME_AND_TYPE_ONLY", "NO"));
+    return dir;
+}
+
+/************************************************************************/
+/*                           NextDirEntry()                             */
+/************************************************************************/
+
+const VSIDIREntry* VSIDIRUnixStdio::NextDirEntry()
+{
+begin:
+    if( VSI_ISDIR(entry.nMode) && nRecurseDepth != 0 )
+    {
+        CPLString osCurFile(osRootPath);
+        if( !osCurFile.empty() )
+            osCurFile += '/';
+        osCurFile += entry.pszName;
+        auto subdir = static_cast<VSIDIRUnixStdio*>(
+            poFS->VSIUnixStdioFilesystemHandler::OpenDir(osCurFile,
+                nRecurseDepth - 1, nullptr));
+        if( subdir )
+        {
+            subdir->osRootPath = osRootPath;
+            subdir->osBasePath = entry.pszName;
+            subdir->m_osFilterPrefix = m_osFilterPrefix;
+            subdir->m_bNameAndTypeOnly = m_bNameAndTypeOnly;
+            aoStackSubDir.push_back(subdir);
+        }
+        entry.nMode = 0;
+    }
+
+    while( !aoStackSubDir.empty() )
+    {
+        auto l_entry = aoStackSubDir.back()->NextDirEntry();
+        if( l_entry )
+        {
+            return l_entry;
+        }
+        delete aoStackSubDir.back();
+        aoStackSubDir.pop_back();
+    }
+
+    while( true )
+    {
+        const auto* psEntry = readdir(m_psDir);
+        if( psEntry == nullptr )
+        {
+            return nullptr;
+        }
+        // Skip . and ..entries
+        if( psEntry->d_name[0] == '.' &&
+            (psEntry->d_name[1] == '\0' ||
+             (psEntry->d_name[1] == '.' &&
+              psEntry->d_name[2] == '\0')) )
+        {
+            // do nothing
+        }
+        else
+        {
+            CPLFree(entry.pszName);
+            CPLString osName(osBasePath);
+            if( !osName.empty() )
+                osName += '/';
+            osName += psEntry->d_name;
+
+            entry.pszName = CPLStrdup(osName);
+            entry.nMode = 0;
+            entry.nSize = 0;
+            entry.nMTime = 0;
+            entry.bModeKnown = false;
+            entry.bSizeKnown = false;
+            entry.bMTimeKnown = false;
+
+            CPLString osCurFile(osRootPath);
+            if( !osCurFile.empty() )
+                osCurFile += '/';
+            osCurFile += entry.pszName;
+
+#if !defined(__sun) && !defined(__HAIKU__)
+            if( psEntry->d_type == DT_REG )
+                entry.nMode = S_IFREG;
+            else if( psEntry->d_type == DT_DIR )
+                entry.nMode = S_IFDIR;
+            else if( psEntry->d_type == DT_LNK )
+                entry.nMode = S_IFLNK;
+#endif
+
+            const auto StatFile = [&osCurFile, this]()
+            {
+                VSIStatBufL sStatL;
+                if( VSIStatL(osCurFile, &sStatL) == 0 )
+                {
+                    entry.nMode = sStatL.st_mode;
+                    entry.nSize = sStatL.st_size;
+                    entry.nMTime = sStatL.st_mtime;
+                    entry.bModeKnown = true;
+                    entry.bSizeKnown = true;
+                    entry.bMTimeKnown = true;
+                }
+            };
+
+            if( !m_osFilterPrefix.empty() &&
+                m_osFilterPrefix.size() > osName.size() )
+            {
+                if( STARTS_WITH(m_osFilterPrefix.c_str(), osName.c_str()) &&
+                    m_osFilterPrefix[osName.size()] == '/' )
+                {
+#if !defined(__sun) && !defined(__HAIKU__)
+                    if( psEntry->d_type == DT_UNKNOWN )
+#endif
+                    {
+                        StatFile();
+                    }
+                    if( VSI_ISDIR(entry.nMode) )
+                    {
+                        goto begin;
+                    }
+                }
+                continue;
+            }
+            if( !m_osFilterPrefix.empty() &&
+                !STARTS_WITH(osName.c_str(), m_osFilterPrefix.c_str()) )
+            {
+                continue;
+            }
+
+            if( !m_bNameAndTypeOnly
+#if !defined(__sun) && !defined(__HAIKU__)
+                || psEntry->d_type == DT_UNKNOWN
+#endif
+                )
+            {
+                StatFile();
+            }
+
+            break;
+        }
+    }
+
+    return &(entry);
+}
+
 #ifdef VSI_COUNT_BYTES_READ
 /************************************************************************/
 /*                            AddToTotal()                              */
