@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2011, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2011, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2011, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -41,7 +41,7 @@
 #include "cpl_conv.h"
 #include "cpl_error.h"
 
-CPL_CVSID("$Id: cplstringlist.cpp c39d156816d937c3139360b11786c769aeabd21e 2018-05-05 19:48:08 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                           CPLStringList()                            */
@@ -74,7 +74,7 @@ CPLStringList::CPLStringList( char **papszListIn, int bTakeOwnership ):
 
 /**
  * CPLStringList constructor.
- * 
+ *
  * The input list is copied.
  *
  * @param papszListIn the NULL terminated list of strings to ingest.
@@ -129,6 +129,30 @@ CPLStringList &CPLStringList::operator=( const CPLStringList& oOther )
 /*                             operator=()                              */
 /************************************************************************/
 
+CPLStringList &CPLStringList::operator=( CPLStringList&& oOther )
+{
+    if( this != &oOther )
+    {
+        Clear();
+        papszList = oOther.papszList;
+        oOther.papszList = nullptr;
+        nCount = oOther.nCount;
+        oOther.nCount = 0;
+        nAllocation = oOther.nAllocation;
+        oOther.nAllocation = 0;
+        bOwnList = oOther.bOwnList;
+        oOther.bOwnList = false;
+        bIsSorted = oOther.bIsSorted;
+        oOther.bIsSorted = true;
+    }
+
+    return *this;
+}
+
+/************************************************************************/
+/*                             operator=()                              */
+/************************************************************************/
+
 CPLStringList &CPLStringList::operator=( CSLConstList papszListIn )
 {
     if( papszListIn != papszList )
@@ -333,7 +357,10 @@ CPLStringList &CPLStringList::AddStringDirectly( char *pszNewString )
 CPLStringList &CPLStringList::AddString( const char *pszNewString )
 
 {
-    return AddStringDirectly( CPLStrdup( pszNewString ) );
+    char* pszDupString = VSI_STRDUP_VERBOSE(pszNewString);
+    if( pszDupString == nullptr )
+        return *this;
+    return AddStringDirectly(pszDupString);
 }
 
 /************************************************************************/
