@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # ******************************************************************************
-#  $Id: validate_jp2.py e4fe7cc06270e5f38dfe78e6785a6bcca4e39e29 2021-04-01 21:02:04 +0300 Idan Miara $
+#  $Id$
 #
 #  Project:  GDAL
 #  Purpose:  Validate JPEG2000 file structure
@@ -1194,7 +1194,7 @@ def validate(filename, oidoc, inspire_tg, expected_gmljp2, ogc_schemas_location,
     return error_report
 
 
-def main(argv):
+def main(argv=sys.argv):
     i = 1
     filename = None
     oidoc = None
